-- Colour tables now taken from WoW values (Since 1.6.4) - Well, we want to be ready when there's Paladin and Shaman in same raid
-- and we can leave the colour choice up to Blizzard and it'll just all work.

-- This table used to convert class names to the internal class name, so we can pull out the class colour from Blizzard's table
local function MashXX(class)
	local c = RAID_CLASS_COLORS[class];
	PerlColourTable[class] = string.format("|c00%02X%02X%02X", 255 * c.r, 255 * c.g, 255 * c.b);
end
PerlColourTable = {}
MashXX("HUNTER");
MashXX("WARLOCK");
MashXX("PRIEST");
MashXX("PALADIN");
MashXX("MAGE");
MashXX("ROGUE");
MashXX("DRUID");
MashXX("SHAMAN");
MashXX("WARRIOR");

function Perl_GetClassColour(class)
	if (class) then
		local color = RAID_CLASS_COLORS[class];		-- Now using the WoW class color table
		if (color) then
			return color.r, color.g, color.b;
		end
	end
	return 0.5,0.5,1;
end

---------------------------------
--Loading Function             --
---------------------------------
function Perl_BlizzFramesDisable()

	if (Perl_Player_Frame) then
		-- Blizz Player Frame Events
		PlayerFrame:UnregisterAllEvents();
		PlayerFrameHealthBar:UnregisterAllEvents();
		PlayerFrameManaBar:UnregisterAllEvents();
		PlayerFrame:Hide();
	end

	if (Perl_Player_Pet_Frame) then
		-- Blizz Pet Frame Events
		PetFrame:UnregisterAllEvents();
		PetFrame:Hide();
	end

	if (Perl_Target_Frame) then
		-- Blizz Target Frame Events
		TargetFrame:UnregisterAllEvents();
		TargetFrameHealthBar:UnregisterAllEvents();
		TargetFrameManaBar:UnregisterAllEvents();
		TargetFrame:Hide();
	end

	if (Perl_party1) then
		-- Blizz Party Events
		for num = 1, 4 do
			frame = getglobal("PartyMemberFrame"..num);
			HealthBar = getglobal("PartyMemberFrame"..num.."HealthBar");
			ManaBar = getglobal("PartyMemberFrame"..num.."ManaBar");

			frame:UnregisterAllEvents();
			HealthBar:UnregisterAllEvents();
			ManaBar:UnregisterAllEvents();
		end
		HidePartyFrame();
	end

end

---------------------------------
--Smooth Health Bar Color      --
---------------------------------
function Perl_SetSmoothBarColor (bar, backBar, alpha)
	if (not alpha) then
		alpha = 1;
	end
	if (bar) then
		local barmin, barmax = bar:GetMinMaxValues();
		if (barmin == barmax) then
			return false;
		end

		local percentage = bar:GetValue()/(barmax-barmin);

		local r, g, b;
		if (Perl_Config.ClassicHealthBar == 1) then
			if (percentage < 0.5) then
				r = 1;
				g = 2*percentage;
				b = 0;
			else
				g = 1;
				r = 2*(1 - percentage);
				b = 0;
			end
		else
			r = Perl_Config.ColourHealthEmpty.r + ((Perl_Config.ColourHealthFull.r - Perl_Config.ColourHealthEmpty.r) * percentage);
			g = Perl_Config.ColourHealthEmpty.g + ((Perl_Config.ColourHealthFull.g - Perl_Config.ColourHealthEmpty.g) * percentage);
			b = Perl_Config.ColourHealthEmpty.b + ((Perl_Config.ColourHealthFull.b - Perl_Config.ColourHealthEmpty.b) * percentage);
		end

		if ((r>=0) and (g>=0) and (b>=0) and (alpha>=0) and (r<=1) and (g<=1) and (b<=1) and (alpha<=1)) then
			bar:SetStatusBarColor(r, g, b);

			if (backBar) then
				backBar:SetVertexColor(r, g, b, alpha);
			end
		end
	else
		return false;
	end
end
--------

function Perl_UnitDebuffInformation(debuff)
	local i = 1;

	while UnitDebuff( "target", i ) do
		PerlBuffStatusTooltip:SetUnitDebuff("target", i );
		if PerlBuffStatusTooltipTextLeft1:GetText() == debuff then
			_, debuffApplications = UnitDebuff("target", i);
			return debuffApplications;
		end

		i = i + 1;
	end

	return 0;
end
---------------------------------
--Class Icon Location Functions--
---------------------------------
function Perl_ClassPos (class)
	if(class=="WARRIOR") then return 0,    0.25,    0,	0.25;	end
	if(class=="MAGE")    then return 0.25, 0.5,     0,	0.25;	end
	if(class=="ROGUE")   then return 0.5,  0.75,    0,	0.25;	end
	if(class=="DRUID")   then return 0.75, 1,       0,	0.25;	end
	if(class=="HUNTER")  then return 0,    0.25,    0.25,	0.5;	end
	if(class=="SHAMAN")  then return 0.25, 0.5,     0.25,	0.5;	end
	if(class=="PRIEST")  then return 0.5,  0.75,    0.25,	0.5;	end
	if(class=="WARLOCK") then return 0.75, 1,       0.25,	0.5;	end
	if(class=="PALADIN") then return 0,    0.25,    0.5,	0.75;	end
	return 0,1,0,1;
end

function Perl_Options_SelectFrame(frame)
	if (perl_locked == 0) then
		Perl_OptionsFrame_SelectFrame(frame);
	end
end

function Perl_Toggle()
	if (perl_locked == 1) then
		Perl_UnlockFrames();
	else
		Perl_LockFrames();
	end
end

function Perl_UnlockFrames()
	EnableAddOn("Perl_Options");
	if (not IsAddOnLoaded("Perl_Options")) then
		UIParentLoadAddOn("Perl_Options");
	end

	perl_locked = 0;
	if (Perl_RaidTitles) then
		Perl_RaidTitles();
	end

	if (Perl_FrameOptions_Frame) then
		Perl_FrameOptions_Frame:Show();
		Perl_FrameOptions_Frame:SetAlpha(0);
		Perl_FrameOptions_Frame.Fading = "in";
	end
end

function Perl_LockFrames()
	perl_locked = 1;
	if (Perl_FrameOptions_Frame) then
		--Perl_FrameOptions_Frame:Hide();
		Perl_FrameOptions_Frame.Fading = "out";
	end

	if (Perl_RaidTitles) then
		Perl_RaidTitles();
	end
end

function Perl_Error(error)
	DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000X-Perl error|r - "..error);
end

-- Minimap Icon
function Perl_MinimapButton_OnClick()
	Perl_Toggle();
end

function Perl_MinimapButton_Init()
	if(Perl_Config.MinimapButtonShown == 1) then
		Perl_MinimapButton_Frame:Show();
	else
		Perl_MinimapButton_Frame:Hide();
	end
end

function Perl_MinimapButton_UpdatePosition()
	Perl_MinimapButton_Frame:SetPoint(
		"TOPLEFT",
		"Minimap",
		"TOPLEFT",
		54 - (78 * cos(Perl_Config.MinimapButtonPosition)),
		(78 * sin(Perl_Config.MinimapButtonPosition)) - 55
	);
	Perl_MinimapButton_Init();
end

function Perl_MinimapButton_Dragging()
	local xpos,ypos = GetCursorPosition();
	local xmin,ymin = Minimap:GetLeft(), Minimap:GetBottom();

	xpos = xmin-xpos/UIParent:GetScale()+70;
	ypos = ypos/UIParent:GetScale()-ymin-70;

	Perl_MinimapButton_SetPosition(math.deg(math.atan2(ypos,xpos)));
end

function Perl_MinimapButton_SetPosition(v)
	if (v < 0) then
		v = v + 360;
	end

	Perl_Config.MinimapButtonPosition = v;
	Perl_MinimapButton_UpdatePosition();
end

function Perl_MinimapButton_OnEnter()
	GameTooltip:SetOwner(this, "ANCHOR_LEFT");
	GameTooltip:SetText(Perl_Version, 1, 1, 1);
	GameTooltip:AddLine(PERL_MINIMAP_HELP1);
	GameTooltip:AddLine(PERL_MINIMAP_HELP2);
	GameTooltip:Show();
end

-- Perl_OnClick_Handler
function Perl_OnClick_Handler(button, unitid)
	if (type(Perl_Custom_ClickFunction) == "function") then
		if (Perl_Custom_ClickFunction(button, unitid)) then
			-- Perl_Custom_ClickFunction should return true if handled, then we do nothing more.
			-- no return and we'll continue with default X-Perl behaviour
			return true;
		end
	end

	if (Perl_Config.CastParty == 1 and not SpellIsTargeting() and not CursorHasItem() and (Perl_Config.CastPartyRaidOnly == 0 or string.sub(unitid, 1, 4) == "raid")) then
		if (CastParty_OnClickByUnit and CastPartyConfig) then
   			local function CastParty_BuildActionKey(button)
				local action_key = ""
				if IsAltKeyDown() then
					action_key = action_key .. 'Alt'
				end
				if IsControlKeyDown() then
					action_key = action_key .. 'Ctrl'
				end
				if IsShiftKeyDown() then
					action_key = action_key .. 'Shift'
				end
				if action_key == "" then
					action_key = "None"
				end
				return action_key
   			end

			local action_key = CastParty_BuildActionKey();
   			local action = CastPartyConfig.key_bindings[button][action_key];

			if (action ~= "CastParty_WoWDefaultClick" and action ~= "CastParty_PartyDropDown" and action ~= CASTPARTY_KEYBINDINGS_NONE) then
				CastParty_OnClickByUnit(button, unitid);
				return true;
			end

		elseif (Genesis_MouseHeal) then
			if (Genesis_MouseHeal(unitid, button)) then
				return true;
			end

		elseif (CH_Config and CH_Config.PCUFEnabled and CH_UnitClicked) then
  			local mb = CH_BuildMouseButton(button);
  			actionType = CH_MouseSpells.Friend[mb];
			if (actionType ~= "MENU" and actionType ~= "TARGET" and actionType ~= "TOOLTIP") then
				CH_UnitClicked(unitid, button);
				return true;
			end

		elseif (SmartHeal and SmartHeal.DefaultClick and SmartHeal.ClickHeal and SmartHeal.GetClickHealButton and SmartHeal.Loaded and SmartHeal.getConfig and SmartHeal:getConfig("enable") and SmartHeal:getConfig("enable", "clickmode")) then
			local KeyDownType = SmartHeal:GetClickHealButton();
			if(KeyDownType and KeyDownType ~= "undetermined") then
				SmartHeal:ClickHeal(KeyDownType..button, unitid);
				return true;
			end
		end
	end

	if (button == "LeftButton") then
		if (SpellIsTargeting()) then
			SpellTargetUnit(unitid);
			return true;
		elseif (CursorHasItem()) then
      			if (UnitIsUnit("player", unitid)) then
         			AutoEquipCursorItem();
			else
				DropItemOnUnit(unitid);
			end
			return true;
		else
			TargetUnit(unitid);
			return true;
		end

	elseif (button == "RightButton") then
		if (SpellIsTargeting()) then
			SpellStopTargeting();
			return true;
		elseif (CursorHasItem()) then
			if (ClearCursor) then
				ClearCursor();		-- New in WoW 1.12 - Might not be necessary, but covers our back
				return true;
			end
		end
	end
end

-- Perl_SetManaBarType
function Perl_SetManaBarType(argUnit, argFrame, argFrameBG)
	local power = UnitPowerType(argUnit);
	if (power) then
		local Colours = {"ColourMana", "ColourRage", "ColourFocus", "ColourEnergy"}

		local colour = Perl_Config[Colours[power + 1]];

		argFrame:SetStatusBarColor(colour.r, colour.g, colour.b, 1);
		argFrameBG:SetVertexColor(colour.r, colour.g, colour.b, 0.25);
	end
end

-- Perl_PlayerTip
function Perl_PlayerTip(unitid)

        if (SpellIsTargeting()) then
                if (SpellCanTargetUnit(unitid)) then
                        SetCursor("CAST_CURSOR");
                else
                        SetCursor("CAST_ERROR_CURSOR");
                end
        end

	GameTooltip_SetDefaultAnchor(GameTooltip, this);
	GameTooltip:SetUnit(unitid);

	if (Perl_RaidTipExtra) then
		Perl_RaidTipExtra(unitid);
	end
end

-- Perl_PlayerTipHide
function Perl_PlayerTipHide()
	GameTooltip:FadeOut();
end

function Perl_ToolTip_AddBuffDuration(partyid)
	if (Perl_Raid_AddBuffDuration) then
		Perl_Raid_AddBuffDuration(partyid);
	end
end

-- Perl_ColourFriendlyUnit
function Perl_ColourFriendlyUnit(frame, partyid)
	if (UnitCanAttack("player", partyid) and UnitIsEnemy("player", partyid)) then	-- For dueling
                frame:SetTextColor(1, 0, 0);
	else
		if (Perl_Config.ClassColouredNames == 1) then
			local _, engClass = UnitClass(partyid);
			local r, g, b = Perl_GetClassColour(engClass);
			frame:SetTextColor(r, g, b);
		else
			if (UnitIsPVP(partyid)) then
                	        frame:SetTextColor(0, 1, 0);
			else
                	        frame:SetTextColor(0.5, 0.5, 1);
			end
		end
	end
end

-- Perl_SetUnitNameColor
function Perl_SetUnitNameColor(argUnit,argFrame)

	local r, g, b = 0.5, 0.5, 1;

        if (UnitPlayerControlled(argUnit) or not UnitIsVisible(argUnit)) then
		if (Perl_Config.ClassColouredNames == 1) then
			local _, class = UnitClass(argUnit)
			r, g, b = Perl_GetClassColour(class);
		end

                if (UnitFactionGroup("player") == UnitFactionGroup(argUnit)) then
                        if (UnitIsEnemy("player", argUnit)) then
				-- Dueling
                                r, g, b = 1, 0, 0;

                        elseif (UnitIsPVP(argUnit)) then
                                r, g, b = 0, 1, 0;
			end
		else
                        if (UnitIsPVP(argUnit)) then
				if (UnitIsPVP("player")) then
					r, g, b = 1, 0, 0;
				else
					r, g, b = 1, 1, 0;
				end
			end
		end
	else
                if (UnitIsTapped(argUnit) and not UnitIsTappedByPlayer(argUnit)) then
                        r, g, b = 0.5, 0.5, 0.5;
                else
			local reaction = UnitReaction(argUnit, "player");

			if (reaction) then
				if (reaction >= 5) then
					r, g, b = 0, 1, 0;
				elseif (reaction <= 2) then
					r, g, b = 1, 0, 0;
				elseif (reaction == 3) then
					r, g, b = 1, 0.5, 0;
				else
					r, g, b = 1, 1, 0;
				end
			else
                        	if (UnitFactionGroup("player") == UnitFactionGroup(argUnit)) then
					r, g, b = 0, 1, 0;
                        	elseif (UnitIsEnemy("player", argUnit)) then
					r, g, b = 1, 0, 0;
                        	else
					r, g, b = 1, 1, 0;
                        	end
			end
		end
	end

	argFrame:SetTextColor(r, g, b, Perl_Config.TextTransparency);
end
-- Perl_RegisterBasics
function Perl_RegisterBasics(argFrame)
	if (argFrame == nil) then
		argFrame = this;
	end
	argFrame:RegisterEvent("UNIT_RAGE");
	argFrame:RegisterEvent("UNIT_MAXRAGE");
	argFrame:RegisterEvent("UNIT_ENERGY");
	argFrame:RegisterEvent("UNIT_MAXENERGY");
	argFrame:RegisterEvent("UNIT_MANA");
	argFrame:RegisterEvent("UNIT_MAXMANA");
	argFrame:RegisterEvent("UNIT_HEALTH");
	argFrame:RegisterEvent("UNIT_MAXHEALTH");
	argFrame:RegisterEvent("UNIT_LEVEL");
	argFrame:RegisterEvent("UNIT_COMBAT");
	argFrame:RegisterEvent("UNIT_DISPLAYPOWER");
	argFrame:RegisterEvent("UNIT_NAME_UPDATE");
end

-- Perl_UnregisterBasics
function Perl_UnregisterBasics(argFrame)
	if (argFrame == nil) then
		argFrame = this;
	end
	argFrame:UnregisterEvent("UNIT_RAGE");
	argFrame:UnregisterEvent("UNIT_MAXRAGE");
	argFrame:UnregisterEvent("UNIT_ENERGY");
	argFrame:UnregisterEvent("UNIT_MAXENERGY");
	argFrame:UnregisterEvent("UNIT_MANA");
	argFrame:UnregisterEvent("UNIT_MAXMANA");
	argFrame:UnregisterEvent("UNIT_HEALTH");
	argFrame:UnregisterEvent("UNIT_MAXHEALTH");
	argFrame:UnregisterEvent("UNIT_LEVEL");
	argFrame:UnregisterEvent("UNIT_COMBAT");
	argFrame:UnregisterEvent("UNIT_DISPLAYPOWER");
	argFrame:UnregisterEvent("UNIT_NAME_UPDATE");
end

function PerlSetPortrait3D(argFrame, argUnit)
	argFrame:ClearModel();
	argFrame:SetUnit(argUnit);
	argFrame:SetCamera(0);
end

function Perl_CombatFlashSet (elapsed, argFrame, argNew, argGreen)
	if (Perl_Config.PerlCombatFlash == 0) then
		argFrame.PlayerFlash = 0;
		argNew = false;
		return false;
	end

	if (argNew) then
		argFrame.PlayerFlash = 1;
		argFrame.PlayerFlashGreen = argGreen;
	else
		if (argFrame.PlayerFlash and elapsed) then
			argFrame.PlayerFlash = argFrame.PlayerFlash - elapsed;

			if (argFrame.PlayerFlash <= 0) then
				argFrame.PlayerFlash = 0;
				argFrame.PlayerFlashTime = nil;
				argFrame.PlayerFlashGreen = nil;
			end
		else
			return false;
		end
	end

	return true;
end

function Perl_CombatFlashSetFrames(argFrame, FlashIntensity, argGreen)
	if (argFrame and FlashIntensity) then
		if (FlashIntensity > 0) then
			local flashOffsetColour = FlashIntensity / 2;
			local r, g, b;
			if (argGreen) then
				r = Perl_Config.BorderColour.r - flashOffsetColour;
				g = Perl_Config.BorderColour.g + flashOffsetColour;
				b = Perl_Config.BorderColour.b - flashOffsetColour;
			else
				r = Perl_Config.BorderColour.r + flashOffsetColour;
				g = Perl_Config.BorderColour.g - flashOffsetColour;
				b = Perl_Config.BorderColour.b - flashOffsetColour;
			end

			if (r < 0) then r = 0; elseif (r > 1) then r = 1; end
			if (g < 0) then g = 0; elseif (g > 1) then g = 1; end
			if (b < 0) then b = 0; elseif (b > 1) then b = 1; end

			argFrame:SetBackdropBorderColor(r, g, b, Perl_Config.BorderColour.a);
		else
			argFrame:SetBackdropBorderColor(Perl_Config.BorderColour.r, Perl_Config.BorderColour.g, Perl_Config.BorderColour.b, Perl_Config.BorderColour.a);
		end
	end
end

function Perl_CheckDebuffs(unit, frames)

	if (Perl_Config.HighlightDebuffs == 0) then
		return;
	end

	local color, cursed;
	local Curses = {};
	local debuffCount = 0;

	for i = 1, MAX_TARGET_DEBUFFS do
		local debuff, debuffStack, debuffType = UnitDebuff(unit, i);

		if (not debuff) then
			break;
		end

		if (debuffType) then
			Curses[debuffType] = 1;
		else
			Curses["none"] = 1;
		end
		debuffCount = debuffCount + 1;
	end

	if (debuffCount > 0) then
		-- The DebuffTypeColor variable is
		if (Curses.Magic) then
			color = DebuffTypeColor["Magic"];
		elseif (Curses.Curse) then
			color = DebuffTypeColor["Curse"];
		elseif (Curses.Disease) then
			color = DebuffTypeColor["Disease"];
		elseif (Curses.Poison) then
			color = DebuffTypeColor["Poison"];
		end

		-- We also re-set the colours here so that we highlight best colour per class
		local _, class = UnitClass("player");
		if (class == "MAGE") then
			cursed = Curses["Curse"];
			if (cursed) then
				color = DebuffTypeColor["Curse"];
			end

		elseif (class == "DRUID") then
			if (Curses["Curse"]) then
				cursed = true;
				color = DebuffTypeColor["Curse"];
			elseif (Curses["Poison"]) then
				cursed = true;
				color = DebuffTypeColor["Poison"];
			end

		elseif (class == "PRIEST") then
			cursed = Curses["Magic"];
			if (cursed) then
				color = DebuffTypeColor["Magic"];
			end

		elseif (class == "WARLOCK") then
			cursed = Curses["Magic"];
			if (cursed) then
				color = DebuffTypeColor["Magic"];
			end

		elseif (class == "PALADIN") then
			if (Curses["Magic"]) then
				cursed = true;
				color = DebuffTypeColor["Magic"];
			elseif (Curses["Poison"]) then
				cursed = true;
				color = DebuffTypeColor["Poison"];
			elseif (Curses["Disease"]) then
				cursed = true;
				color = DebuffTypeColor["Disease"];
			end

		elseif (class == "SHAMAN") then
			if (Curses["Poison"]) then
				cursed = true;
				color = DebuffTypeColor["Poison"];
			elseif (Curses["Disease"]) then
				cursed = true;
				color = DebuffTypeColor["Disease"];
			end
		end
	end

	if (frames) then
		for i, f in frames do
			if (f:IsShown()) then
				if (color) then
					f:SetBackdropColor(color.r, color.g, color.b, 1);
				else
					f:SetBackdropColor(Perl_Config.BackColour.r, Perl_Config.BackColour.g, Perl_Config.BackColour.b, Perl_Config.BackColour.a);
				end

				local subFrame = getglobal(f:GetName().."_Curse");
				if (subFrame) then
					if (Perl_Config.HighlightDebuffsBorder == 1) then
						if (color) then
							subFrame:SetVertexColor(color.r, color.g, color.b);
							subFrame:Show();
						else
							subFrame:Hide();
						end
					else
						subFrame:Hide();
					end
				end
			end
		end
	end

	return cursed, color;
end

-- Perl_SavePosition(frame)
function Perl_SavePosition(frame)
	if (not Perl_Config.SavedPositions) then
		Perl_Config.SavedPositions = {};
	end
	Perl_Config.SavedPositions[frame:GetName()] = {top = frame:GetTop(), left = frame:GetLeft()};
end

-- Perl_RestorePosition(frame)
function Perl_RestorePosition(frame)
	if (Perl_Config.SavedPositions) then
		local pos = Perl_Config.SavedPositions[frame:GetName()];
		if (pos) then
			if (pos.left or pos.right) then
				frame:ClearAllPoints();
				frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", pos.left, pos.top);
			end
		end
	end
end

-- Perl_GetRaidPosition
function Perl_GetRaidPosition(findName)
	for i = 1,GetNumRaidMembers() do
		if (UnitName("raid"..i) == findName) then
			return i;
		end
	end
end
