function Perl_StatsFrame_Setup(StatsFrame, others, offset)

	if (not StatsFrame) then
		return;
	end

        local healthBar = getglobal(StatsFrame:GetName().."_HealthBar");
        local healthBarText = getglobal(StatsFrame:GetName().."_HealthBarText");
        local healthBarPercent = getglobal(StatsFrame:GetName().."_HealthBarPercent");
        local manaBar = getglobal(StatsFrame:GetName().."_ManaBar");
        local manaBarPercent = getglobal(StatsFrame:GetName().."_ManaBarPercent");
	local otherBars = {};
	local secondaryBarsShown = 0;
	local showPercent = 0;
	if (healthBarPercent:IsShown() or manaBarPercent:IsShown()) then
		showPercent = 1;
	end
        local percentSize = showPercent * 35;   -- 0 means 0, 1 means 32 offset (+3 for spacing after text), which is our size difference

	offset = (offset or 0);

	if (manaBar:IsShown()) then
		secondaryBarsShown = secondaryBarsShown + 1;
	end

	local needTicker = 0;
	if (Perl_Config.EnergyTicker == 1 and StatsFrame == Perl_Player_StatsFrame and UnitPowerType("player") == 3) then
		needTicker = 1;
	end

	if (others) then
		for i in others do
			local bar = getglobal(StatsFrame:GetName().."_"..others[i]);
			if (bar) then
				table.insert(otherBars, bar)

				if (bar:IsShown()) then
					secondaryBarsShown = secondaryBarsShown + 1;
				end
			else
				if (not bar) then Perl_Notice("No frame found called '"..StatsFrame:GetName().."_"..others[i].."'"); end
			end
		end
	end

	if (Perl_Config.FatHealthBars == 1) then
		healthBarText:SetFontObject(GameFontNormal);

        	healthBar:ClearAllPoints();
        	healthBar:SetPoint("TOPLEFT", 5, -5);
        	healthBar:SetPoint("BOTTOMRIGHT", -(5 + percentSize), 5 + needTicker + (secondaryBarsShown * 10));

        	manaBar:ClearAllPoints();
        	manaBar:SetPoint("BOTTOMLEFT", 5, -5 + needTicker + (secondaryBarsShown * 10));
        	manaBar:SetPoint("TOPRIGHT", healthBar, "BOTTOMRIGHT", 0, 0);

		local lastBar = manaBar;
		local tickerSpace = needTicker * 2;
		for i,bar in otherBars do
			if (bar:IsShown()) then
        	        	bar:ClearAllPoints();

        	        	bar:SetPoint("TOPLEFT", lastBar, "BOTTOMLEFT", 0, -tickerSpace);
        	        	bar:SetPoint("BOTTOMRIGHT", lastBar, "BOTTOMRIGHT", 0, -10 - tickerSpace);

				lastBar = bar;
				tickerSpace = 0;
			end
        	end
	else
		healthBarText:SetFontObject(GameFontNormalSmall);

        	healthBar:ClearAllPoints();
        	healthBar:SetPoint("TOPLEFT", 8, -9 + offset);
        	healthBar:SetPoint("BOTTOMRIGHT", StatsFrame, "TOPRIGHT", -(8 + percentSize), -19 + offset);

        	manaBar:ClearAllPoints();
        	manaBar:SetPoint("TOPLEFT", healthBar, "BOTTOMLEFT", 0, -2);
        	manaBar:SetPoint("BOTTOMRIGHT", healthBar, "BOTTOMRIGHT", 0, -12);

		local lastBar = manaBar;
		for i,bar in otherBars do
			if (bar:IsShown()) then
        	        	bar:ClearAllPoints();

        	        	bar:SetPoint("TOPLEFT", lastBar, "BOTTOMLEFT", 0, -2);
        	        	bar:SetPoint("BOTTOMRIGHT", lastBar, "BOTTOMRIGHT", 0, -12);

				lastBar = bar;
			end
        	end
	end
end

function Perl_Setup_StatsFrames()

	Perl_StatsFrame_Setup(Perl_Player_StatsFrame, {"DruidBar", "XPBar"});
	Perl_StatsFrame_Setup(Perl_Player_Pet_StatsFrame, {"XPBar"}, 2);

	Perl_StatsFrame_Setup(Perl_Target_StatsFrame);
	Perl_StatsFrame_Setup(Perl_TargetTarget_StatsFrame);
	Perl_StatsFrame_Setup(Perl_TargetTargetTarget_StatsFrame);

	Perl_StatsFrame_Setup(Perl_party1_StatsFrame);
	Perl_StatsFrame_Setup(Perl_party2_StatsFrame);
	Perl_StatsFrame_Setup(Perl_party3_StatsFrame);
	Perl_StatsFrame_Setup(Perl_party4_StatsFrame);

	Perl_StatsFrame_Setup(Perl_Party_Pet1_StatsFrame, nil, 2);
	Perl_StatsFrame_Setup(Perl_Party_Pet2_StatsFrame, nil, 2);
	Perl_StatsFrame_Setup(Perl_Party_Pet3_StatsFrame, nil, 2);
	Perl_StatsFrame_Setup(Perl_Party_Pet4_StatsFrame, nil, 2);
end

local function Perl_SetTransparency(num)
	local function Alpha(unit)
		if (UnitIsConnected(unit)) then
			return num;
		else
			return num / 2;
		end
	end

	local function SetAlpha(f, a)
		if (f) then
			f:SetAlpha(a);
		end
	end

	SetAlpha(Perl_Player_Frame, num);
	SetAlpha(Perl_Target_Frame, Alpha("target"));
	SetAlpha(Perl_TargetTarget, num);
	SetAlpha(Perl_TargetTargetTarget, num);
	SetAlpha(Perl_party1, Alpha("party1"));
	SetAlpha(Perl_party2, Alpha("party2"));
	SetAlpha(Perl_party3, Alpha("party3"));
	SetAlpha(Perl_party4, Alpha("party4"));
	SetAlpha(Perl_Party_Pet1, Alpha("party1"));
	SetAlpha(Perl_Party_Pet2, Alpha("party2"));
	SetAlpha(Perl_Party_Pet3, Alpha("party3"));
	SetAlpha(Perl_Party_Pet4, Alpha("party4"));
	SetAlpha(Perl_Player_Pet_Frame, num);
	SetAlpha(Perl_ArcaneBarFrame, num/2);
end

local function Perl_SetTextTransparency(num)
	local function SetFrame(frame)
		local f = getglobal(frame);

		if (f) then
			f:SetTextColor(1,1,1,Perl_Config.TextTransparency);
		end
	end

	local function SetSubFrames(frame)
		local subFrames = {"HealthBarText", "HealthBarPercent", "ManaBarText", "ManaBarPercent"};

		for j,f in subFrames do
			local n = frame.."_StatsFrame_"..f;
			SetFrame(n);
		end
	end

	local statFrames = {"Perl_Player", "Perl_Player_Pet", "Perl_Target", "Perl_TargetTarget", "Perl_TargetTargetTarget",
				"Perl_party1", "Perl_party2", "Perl_party3", "Perl_party4",
				"Perl_Party_Pet1", "Perl_Party_Pet2", "Perl_Party_Pet3", "Perl_Party_Pet4"};

	for i,f in statFrames do
		SetSubFrames(f);
	end

	if (Perl_Player_Frame) then
		Perl_Player_StatsFrame_DruidBarText:SetTextColor(1,1,1,Perl_Config.TextTransparency);
		Perl_Player_StatsFrame_DruidBarPercent:SetTextColor(1,1,1,Perl_Config.TextTransparency);
		Perl_Player_StatsFrame_XPBarText:SetTextColor(1,1,1,Perl_Config.TextTransparency);
		Perl_Player_StatsFrame_XPBarPercent:SetTextColor(1,1,1,Perl_Config.TextTransparency);
	end
end

function Perl_GetBarTexture()
	if (Perl_Config.BarTextures == 1) then
		return "Interface\\AddOns\\Perl\\Images\\Perl_StatusBar";
	elseif (Perl_Config.BarTextures == 2) then
		return "Interface\\AddOns\\Perl\\Images\\Perl_StatusBar2";
	elseif (Perl_Config.BarTextures == 3) then
		return "Interface\\AddOns\\Perl\\Images\\Perl_StatusBar3";
	else
		return "Interface\\TargetingFrame\\UI-StatusBar";
	end
end

function Perl_SetBarTextures()
	local tex = Perl_GetBarTexture();

	local function SetTexSub(prefix, name)
		local frame = getglobal(prefix.."_StatsFrame_"..name);

		if (frame) then
			frame:SetTexture(tex);
		end
	end

	local function SetTex(name)
		SetTexSub(name, "HealthBarTex");
		SetTexSub(name, "ManaBarTex");
		SetTexSub(name, "XPBarTex");
		SetTexSub(name, "XPRestBarTex");
		SetTexSub(name, "DruidBarTex");
	end

	local texFrames = {
		"Perl_Player_Pet", "Perl_Player", "Perl_Target", "Perl_TargetTarget", "Perl_TargetTargetTarget",
		"Perl_party1", "Perl_party2", "Perl_party3", "Perl_party4",
		"Perl_Party_Pet1", "Perl_Party_Pet2", "Perl_Party_Pet3", "Perl_Party_Pet4"
	}

	for i,n in texFrames do
		SetTex(n);
	end

	if (Perl_Target_Frame) then
		Perl_Target_CPMeterTex:SetTexture(tex);
	end
	if (Perl_ArcaneBarTex) then
		Perl_ArcaneBarTex:SetTexture(tex);
	end

	if (Perl_SetRaidBarTextures) then
		Perl_SetRaidBarTextures();
	end
end

local function Perl_SetCastParty()
	if (Perl_Config.CastPartyShow==0) then
		if (CastPartyMainFrame) then
			CastPartyMainFrame:Hide();
		elseif (CastPartyFrame0 and CastPartyFrame1 and CastPartyFrame2) then
			CastPartyFrame0:Hide();
			CastPartyFrame1:Hide();
			CastPartyFrame2:Hide();
		end
	else
		if (CastPartyMainFrame) then
			CastPartyMainFrame:Show();
		elseif (CastPartyFrame0 and CastPartyFrame1 and CastPartyFrame2) then
			CastPartyFrame0:Show();
			CastPartyFrame1:Show();
			CastPartyFrame2:Show();
		end
	end
end

local function Perl_ScaleParty(num)
	if (Perl_party1) then
		Perl_party4:SetScale(num);
		Perl_party3:SetScale(num);
		Perl_party2:SetScale(num);
		Perl_party1:SetScale(num);
	end
end

local function Perl_ScalePartyPets(num)
	if (Perl_Party_Pet1) then
		Perl_Party_Pet4:SetScale(num);
		Perl_Party_Pet3:SetScale(num);
		Perl_Party_Pet2:SetScale(num);
		Perl_Party_Pet1:SetScale(num);
	end
end

local function Perl_ScaleTargetTarget(num)
	if (Perl_TargetTarget) then
		Perl_TargetTarget:SetScale(num);
		if (Perl_TargetTargetTarget) then
			Perl_TargetTargetTarget:SetScale(num);
		end
		for i = 1,10 do
			local f = getglobal("Perl_TargetTarget_History"..i);
			if (f) then
				f:SetScale(num);
			end
		end
	end
end

function Perl_OptionActions()

	local function ValidAlpha(alpha)
		alpha = tonumber(alpha);
		if (alpha < 0 or alpha > 1) then
			alpha = 1;
		end
		return alpha;
	end

	local function ValidScale(scale)
		scale = tonumber(scale);
		if (scale < 0.5) then
			scale = 0.5;
		elseif (scale > Perl_Config.MaximumScale) then
			scale = Perl_Config.MaximumScale;
		end
		return scale;
	end

	if (Perl_Player_Frame) then
		Perl_Player_Set_Bits();
	end
	if (Perl_Player_Pet_Frame) then
		Perl_Player_Pet_Set_Bits();
	end
	if (Perl_Target_Frame) then
		Perl_Target_Set_Bits();
		Perl_Target_Set_BlizzCPFrame()
	end
	if (Perl_TargetTarget) then
		Perl_TargetTarget_Set_Bits();
	end
	if (Perl_party1) then
		Perl_Party_Set_Bits();
	end
	if (Perl_Party_Pet1) then
		Perl_Party_Pet_Set_Name();
	end

	Perl_Config.Transparency = ValidAlpha(Perl_Config.Transparency);

	Perl_Config.TextTransparency = ValidAlpha(Perl_Config.TextTransparency);
	Perl_SetTextTransparency(Perl_Config.TextTransparency);

	Perl_Config.Scale_PlayerFrame = ValidScale(Perl_Config.Scale_PlayerFrame);
	if (Perl_Player_Frame) then
		Perl_Player_Frame:SetScale(Perl_Config.Scale_PlayerFrame);
	end

	Perl_Config.Scale_PetFrame = ValidScale(Perl_Config.Scale_PetFrame);
	if (Perl_Player_Pet_Frame) then
		Perl_Player_Pet_Frame:SetScale(Perl_Config.Scale_PetFrame);
	end

	Perl_Config.Scale_PartyFrame = ValidScale(Perl_Config.Scale_PartyFrame);
	Perl_ScaleParty(Perl_Config.Scale_PartyFrame);

	Perl_Config.Scale_PartyPets = ValidScale(Perl_Config.Scale_PartyPets);
	Perl_ScalePartyPets(Perl_Config.Scale_PartyPets);

	Perl_Config.Scale_TargetFrame = ValidScale(Perl_Config.Scale_TargetFrame);
	Perl_Config.Scale_TargetTargetFrame = ValidScale(Perl_Config.Scale_TargetTargetFrame);
	if (Perl_Target_Frame) then
		Perl_Target_Frame:SetScale(Perl_Config.Scale_TargetFrame);
		Perl_ScaleTargetTarget(Perl_Config.Scale_TargetTargetFrame);
	end

	Perl_Config.Scale_Raid = ValidScale(Perl_Config.Scale_Raid);
	if (Perl_ScaleRaid) then
		Perl_ScaleRaid(Perl_Config.Scale_Raid);
	end

	if (Perl_ArcaneBarFrame) then
		Perl_ArcaneBar_Set();
	end

	Perl_SetCastParty();
	Perl_SetBarTextures();

	Perl_Setup_StatsFrames();

	if (Perl_Player_Frame) then			Perl_Player_UpdateDisplay();		end
	if (Perl_Player_Pet_UpdateDisplay) then		Perl_Player_Pet_UpdateDisplay ();	end
	if (Perl_Target_Frame) then
		Perl_Target_UpdateDisplay();
	end
	if (Perl_TargetTarget) then
		Perl_TargetTarget_UpdateDisplay(Perl_TargetTarget);
		if (Perl_TargetTargetTarget) then
			Perl_TargetTarget_UpdateDisplay(Perl_TargetTargetTarget);
		end
	end

	if (Perl_party1) then
		for i = 1,4 do
			Perl_Party_UpdateDisplay("party"..i);
			local frame = getglobal("Perl_Party_Pet"..i);
			if (frame) then
				Perl_Party_Pet_UpdateDisplay(frame);
			end
		end
	end

	if (Perl_Raid_Position) then
		if (Perl_Raid1) then
			Perl_Raid_Position();
			Perl_RaidTitles();
		end
	end

	--if (Perl_Config.ShowRaid == 1) then
	--	local name, title, notes, enabled = GetAddOnInfo("Perl_RaidFrames");
	--
	--	if (not enabled) then
	--		DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000X-Perl|r - Raid Frames AddOn was not enabled. Enabling");
	--		EnableAddOn("Perl_RaidFrames");
	--	end
	--end

	Perl_SetAllFrames();
	Perl_SetTransparency(Perl_Config.Transparency);
end
