local Perl_Party_Pet_Events = {};

local function Perl_Party_Pet_RegisterForPet(argFrame)
	if (argFrame == nil) then
		argFrame = this;
	end
	argFrame:RegisterEvent("UNIT_FACTION");
	argFrame:RegisterEvent("UNIT_AURA");
	argFrame:RegisterEvent("PARTY_MEMBER_ENABLE");
	argFrame:RegisterEvent("PARTY_MEMBER_DISABLE");
	argFrame:RegisterEvent("UNIT_FLAGS");
	argFrame:RegisterEvent("UNIT_DYNAMIC_FLAGS");
	Perl_RegisterBasics(argFrame);
end
local function Perl_Party_Pet_UnregisterForPet(argFrame)
	if (argFrame == nil) then
		argFrame = this;
	end
	argFrame:UnregisterEvent("UNIT_FACTION");
	argFrame:UnregisterEvent("UNIT_AURA");
	argFrame:UnregisterEvent("PARTY_MEMBER_ENABLE");
	argFrame:UnregisterEvent("PARTY_MEMBER_DISABLE");
	argFrame:UnregisterEvent("UNIT_FLAGS");
	argFrame:UnregisterEvent("UNIT_DYNAMIC_FLAGS");
	Perl_UnregisterBasics(argFrame);
end


----------------------
-- Loading Function --
----------------------

function Perl_Party_Pet_OnLoad()

	--Events
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_LEAVING_WORLD");

	getglobal("Perl_Party_Pet"..this:GetID().."_DropDown"):SetID(this:GetID());
	table.insert(UnitPopupFrames,"Perl_Party_Pet"..this:GetID().."_DropDown");

	this.partyunit = "party"..this:GetID();
	this.petunit = "partypet"..this:GetID();
	this.time = 0;

	Perl_InitFadeFrame(this);
end

-------------------------
-- The Update Function --
-------------------------

-- Perl_Party_Pet_CheckPet
-- returns true if full update required (frame shown)

function Perl_Party_Pet_CheckPet(argFrame)
	if (argFrame == nil) then
		argFrame = this;
	end

	if (Perl_Config.ShowPartyPets == 1) then
		if (UnitName(argFrame.partyunit) and UnitName(argFrame.petunit) and UnitIsConnected(argFrame.petunit) and not UnitIsDead(argFrame.petunit)) then   -- and not UnitIsDeadOrGhost(argFrame.partyunit)
			if (not argFrame:IsShowing()) then
				if (UnitName("raid1")) then
					if (Perl_Config.ShowPartyRaid == 1) then
						Perl_Party_Pet_RegisterForPet(argFrame);
						Perl_CancelFade(argFrame);
						argFrame:Show();
						return true;
					end
				else
					Perl_Party_Pet_RegisterForPet(argFrame);
					Perl_CancelFade(argFrame);
					argFrame:Show();
					return true;
				end
			end
			return false;
		end
	end

	if (argFrame:IsShowing()) then
		Perl_Party_Pet_UnregisterForPet(argFrame);
		Perl_StartFade(argFrame);
	end

	return false;
end

-- Perl_Party_Pet_UpdateName
local function Perl_Party_Pet_UpdateName(argFrame)
	if (argFrame == nil) then
		argFrame = this;
	end

	if (argFrame.petunit == nil) then
		return;
	end

	local Partypetname = UnitName(argFrame.petunit);

	if (Partypetname ~= nil) then
		-- Set name
		if (strlen(Partypetname) > 40) then
			Partypetname = strsub(Partypetname, 1, 39).."...";
		end
		if (Perl_Config.ShowPartyPetNames == 1) then
			getglobal(argFrame:GetName().."_NameFrame"):Show();
		end
		getglobal(argFrame:GetName().."_NameFrame_NameBarText"):SetText(Partypetname);

		if (UnitIsPVP(argFrame.partyunit)) then
			getglobal(argFrame:GetName().."_NameFrame_NameBarText"):SetTextColor(0,1,0);
		else
			getglobal(argFrame:GetName().."_NameFrame_NameBarText"):SetTextColor(0.5,0.5,1);
		end
	end
end

-- Perl_Party_Pet_UpdateHealth
local function Perl_Party_Pet_UpdateHealth(argFrame)
	if (argFrame == nil) then
		argFrame = this;
	end

	local Partypethealth = UnitHealth(argFrame.petunit);
	local Partypethealthmax = UnitHealthMax(argFrame.petunit);

	getglobal(argFrame:GetName().."_StatsFrame_HealthBar"):SetMinMaxValues(0, Partypethealthmax);
	getglobal(argFrame:GetName().."_StatsFrame_HealthBar"):SetValue(Partypethealth);
	Perl_SetSmoothBarColor(getglobal(argFrame:GetName().."_StatsFrame_HealthBar"),getglobal(argFrame:GetName().."_StatsFrame_HealthBarBG"), 0.25);

	local phealthPct;
	phealthPct = (Partypethealth * 100.0) / Partypethealthmax
	phealthPct = string.format("%3.0f", phealthPct);

	if (UnitIsDead(argFrame.petunit)) then
		--getglobal(this:GetName().."_StatsFrame_HealthBar_HealthBarPercentText"):SetText(PERL_LOC_DEAD);
		--getglobal(this:GetName().."_StatsFrame_HealthBar_HealthBarPercentText"):Hide();
		Perl_StartFade(argFrame);
	else
		if (Perl_Config.HealerMode == 1) then
			getglobal(argFrame:GetName().."_StatsFrame_HealthBarText"):SetText(-(Partypethealthmax - Partypethealth));
		else
			getglobal(argFrame:GetName().."_StatsFrame_HealthBarText"):SetText(string.format("%d",(100*(Partypethealth / Partypethealthmax))+0.5).."%");
		end

		getglobal(argFrame:GetName().."_StatsFrame_HealthBarText"):Show();
	end
end

-- Perl_Party_Pet_UpdateManaType
local function Perl_Party_Pet_UpdateManaType(argFrame)
	if (argFrame == nil) then
		argFrame = this;
	end

	local frameName = argFrame:GetName().."_StatsFrame_ManaBar";
	Perl_SetManaBarType(argFrame.petunit, getglobal(frameName), getglobal(frameName.."BG"));
end

-- Perl_Party_Pet_UpdateMana
local function Perl_Party_Pet_UpdateMana(argFrame)
	if (argFrame == nil) then
		argFrame = this;
	end

	local Partypetmana = UnitMana(argFrame.petunit);
	local Partypetmanamax = UnitManaMax(argFrame.petunit);

	getglobal(argFrame:GetName().."_StatsFrame_ManaBar"):SetMinMaxValues(0, Partypetmanamax);
	getglobal(argFrame:GetName().."_StatsFrame_ManaBar"):SetValue(Partypetmana);

	pmanaPct = (Partypetmana * 100.0) / Partypetmanamax
	pmanaPct =  string.format("%3.0f", pmanaPct);
	getglobal(argFrame:GetName().."_StatsFrame_ManaBarText"):Show();
	if (UnitPowerType(argFrame.petunit) >= 1) then
		getglobal(argFrame:GetName().."_StatsFrame_ManaBarText"):SetText(Partypetmana);
	else
		getglobal(argFrame:GetName().."_StatsFrame_ManaBarText"):SetText(string.format("%d",(100*(Partypetmana / Partypetmanamax))+0.5).."%");
	end
	if (Perl_Config.ShowPartyPetPercent == 0) then
		getglobal(argFrame:GetName().."_StatsFrame_ManaBarText"):Hide();
	end
end

-- Perl_Party_Pet_UpdateLevel
local function Perl_Party_Pet_UpdateLevel(argFrame)
	if (argFrame == nil) then
		argFrame = this;
	end

	local Partypetlevel = UnitLevel(argFrame.petunit);
	--local PartypetClassification = UnitClassification(this.petunit);
	local color = GetDifficultyColor(Partypetlevel);

	getglobal(argFrame:GetName().."_NameFrame_LevelBarText"):SetTextColor(color.r,color.g,color.b);

	if (Partypetlevel == 0) then
		Partypetlevel = "";
	end
	getglobal(argFrame:GetName().."_NameFrame_LevelBarText"):SetText(Partypetlevel);
end

--------------------
-- Buff Functions --
--------------------
function Perl_Party_Pet_Buff_UpdateAll(argFrame)
	if (argFrame == nil) then
		argFrame = this;
	end

	if (UnitName(argFrame.partyunit) and UnitName(argFrame.petunit)) then
		for buffnum=1,10 do
			local button = getglobal(argFrame:GetName().."_BuffFrame_Buff"..buffnum);
			if (Perl_Config.PartyBuffs == 1 and UnitBuff(argFrame.petunit, buffnum, Perl_Config.PartyCastableBuffs)) then
				local icon = getglobal(button:GetName().."Icon");
				icon:SetTexture(UnitBuff(argFrame.petunit, buffnum, Perl_Config.PartyCastableBuffs));
				button:Show();
			else
				button:Hide();
			end
		end
		for buffnum=1,8 do
			local button = getglobal(argFrame:GetName().."_BuffFrame_DeBuff"..(buffnum));
			if (Perl_Config.PartyDebuffs == 1 and UnitDebuff(argFrame.petunit, buffnum, Perl_Config.PartyCurableDebuffs)) then
				local icon = getglobal(button:GetName().."Icon");
				icon:SetTexture(UnitDebuff(argFrame.petunit, buffnum, Perl_Config.PartyCurableDebuffs));
				button:Show();
			else
				button:Hide();
			end
		end
	end

	Perl_CheckDebuffs(argFrame.petunit, {getglobal(argFrame:GetName().."_NameFrame"), getglobal(argFrame:GetName().."_StatsFrame")});
end

function Perl_Party_Pet_SetBuffTooltip()
	local partyid = "partypet"..this:GetParent():GetParent():GetID();
	GameTooltip:SetOwner(this,"ANCHOR_BOTTOMRIGHT",30,0);
	GameTooltip:SetUnitBuff(partyid, this:GetID(), Perl_Config.PartyCastableBuffs);
end
function Perl_Party_Pet_SetDeBuffTooltip()
	local partyid = "partypet"..this:GetParent():GetParent():GetID();
	GameTooltip:SetOwner(this,"ANCHOR_BOTTOMRIGHT",30,0);
	GameTooltip:SetUnitDebuff(partyid, this:GetID(), Perl_Config.PartyCurableDebuffs);
end

function Perl_Party_Pet_UpdateDisplayAll()
	for i = 1,4 do
		Perl_Party_Pet_UpdateDisplay(getglobal("Perl_Party_Pet"..i));
	end
end;

function Perl_Party_Pet_UpdateDisplay(argFrame)
	if (argFrame == nil) then
		argFrame = this;
	end

	if (Perl_Party_Pet_CheckPet(argFrame)) then
		Perl_Party_Pet_UpdateName(argFrame);
		Perl_Party_Pet_UpdateHealth(argFrame);
		Perl_Party_Pet_UpdateLevel(argFrame);
		Perl_Party_Pet_UpdateManaType(argFrame);
		Perl_Party_Pet_UpdateMana(argFrame);
		Perl_Party_Pet_UpdateCombat(argFrame);
		Perl_Party_Pet_Buff_UpdateAll(argFrame);
	end
end


--------------------
-- Click Handlers --
--------------------
local function Perl_Party_Pet_FindID(object)
	local _, _, num = string.find(object:GetName(), "(%d+)")
	return tonumber(num);
end

function Perl_Party_PetDropDown_OnLoad()
	UIDropDownMenu_Initialize(this, Perl_Party_PetDropDown_Initialize, "MENU");
end

function Perl_Party_PetDropDown_Initialize()
	local dropdown;
	if ( UIDROPDOWNMENU_OPEN_MENU ) then
		dropdown = getglobal(UIDROPDOWNMENU_OPEN_MENU);
	else
		dropdown = this;
	end
	UnitPopup_ShowMenu(dropdown, "PARTY", "partypet"..this:GetID());
end

function Perl_Party_Pet_OnClick(button)
	local id=Perl_Party_Pet_FindID(this);
	this:SetID(id);
	local unit = "partypet"..id;

	if (Perl_OnClick_Handler(button, unit)) then
		return;
	end

	if (button == "RightButton") then
		ToggleDropDownMenu(1, nil, getglobal(this:GetName().."_DropDown"), this:GetName(), 0, 0);
	end
end

function Perl_Party_Pet_PlayerTip()
	local id = Perl_Party_Pet_FindID(this);
	local unit = "partypet"..id;

	if (UnitName(unit)) then
		--GameTooltip_SetDefaultAnchor(GameTooltip, this);
		--GameTooltip:SetUnit(unit);
		Perl_PlayerTip(unit);
	end
end

local function Perl_Target_Update_Control(argFrame)
	if (argFrame == nil) then
		argFrame = this;
	end

        if (UnitIsVisible(argFrame.petunit) and UnitIsCharmed(argFrame.petunit)) then
		getglobal(argFrame:GetName().."_NameFrame_Warning"):Show();
	else
		getglobal(argFrame:GetName().."_NameFrame_Warning"):Hide();
	end
end

function Perl_Party_Pet_UpdateCombat(argFrame)
	if (argFrame == nil) then
		argFrame = this;
	end

	if (UnitName(argFrame.partyunit) and UnitName(argFrame.petunit)) then
                if (UnitAffectingCombat(argFrame.petunit)) then
                        getglobal(argFrame:GetName().."_NameFrame_LevelBarText"):Hide();
                        getglobal(argFrame:GetName().."_NameFrame_ActivityStatus"):Show();
                else
                        getglobal(argFrame:GetName().."_NameFrame_ActivityStatus"):Hide();
                        getglobal(argFrame:GetName().."_NameFrame_LevelBarText"):Show();
                end
		Perl_Target_Update_Control(argFrame);
        end
end

function Perl_Party_Pet_OnUpdate(elapsed)
	if elapsed then
		Perl_Party_Pet_CombatFlash(elapsed, false);
		--Perl_Party_Pet_CheckPet();

		this.time = this.time + elapsed;
		if (this.time > 0.25) then
			Perl_Party_Pet_UpdateCombat();
			this.time = 0;
		end
	end

	Perl_ProcessFade(this);
end

function Perl_Party_Pet_CombatFlash(elapsed, argNew, argGreen)
	if (Perl_CombatFlashSet (elapsed, this, argNew, argGreen)) then
		Perl_CombatFlashSetFrames(getglobal(this:GetName().."_NameFrame"), this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(getglobal(this:GetName().."_LevelFrame"), this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(getglobal(this:GetName().."_StatsFrame"), this.PlayerFlash, this.PlayerFlashGreen);
	end
end


-------------------
-- Event Handler --
-------------------
local function Perl_Party_Pet_RegisterSome()
	this:RegisterEvent("PARTY_MEMBERS_CHANGED");
	--this:RegisterEvent("UNIT_HAPPINESS");
	this:RegisterEvent("UNIT_PET");
end

local function Perl_Party_Pet_UnregisterSome()
	this:UnregisterEvent("PARTY_MEMBERS_CHANGED");
	--this:UnregisterEvent("UNIT_HAPPINESS");
	this:UnregisterEvent("UNIT_PET");
end

function Perl_Party_Pet_OnEvent()
	local func = Perl_Party_Pet_Events[event];
	if (func) then
		func();
	else
Perl_ShowMessage("EXTRA EVENT");
	end
end

-- PLAYER_ENTERING_WORLD
function Perl_Party_Pet_Events:PLAYER_ENTERING_WORLD()
	Perl_Party_Pet_RegisterSome();
	Perl_Party_Pet_UpdateDisplay();
end

-- PLAYER_LEAVING_WORLD
function Perl_Party_Pet_Events:PLAYER_LEAVING_WORLD()
	Perl_Party_Pet_UnregisterSome();
end

-- PARTY_MEMBERS_CHANGED
function Perl_Party_Pet_Events:PARTY_MEMBERS_CHANGED()
	Perl_Party_Pet_UpdateDisplay();
end

Perl_Party_Pet_Events.UNIT_FLAGS = Perl_Party_Pet_Events.PARTY_MEMBERS_CHANGED;
Perl_Party_Pet_Events.UNIT_DYNAMIC_FLAGS = Perl_Party_Pet_Events.PARTY_MEMBERS_CHANGED;

-- UNIT_HAPPINESS
--function Perl_Party_Pet_Events:UNIT_HAPPINESS()
--	if (arg1 == this.petunit) then
--		Perl_Party_Pet_CheckPet();
--	end
--end

-- Perl_Party_Pet_Events:UNIT_COMBAT()
function Perl_Party_Pet_Events:UNIT_COMBAT()
	if (arg1 == this.petunit) then
		if (arg2 == "HEAL") then
			Perl_Party_Pet_CombatFlash(0, true, true);
		elseif (arg4 and arg4 > 0) then
			Perl_Party_Pet_CombatFlash(0, true);
		end
		--Perl_Party_Pet_CheckPet();
	end
end

-- UNIT_PET
function Perl_Party_Pet_Events:UNIT_PET()
	if (arg1 == this.partyunit) then
		Perl_Party_Pet_UpdateDisplay();
	end
end

function Perl_Party_Pet_Events:PARTY_MEMBER_ENABLE()
	if (arg1 == this.petunit or arg1 == this.partyunit) then
		Perl_Party_Pet_UpdateDisplay();
	end
end

Perl_Party_Pet_Events.PARTY_MEMBER_DISABLE = Perl_Party_Pet_Events.PARTY_MEMBER_ENABLE;

-- UNIT_NAME_UPDATE
function Perl_Party_Pet_Events:UNIT_NAME_UPDATE()
	if (arg1 == this.petunit or arg1 == this.partyunit) then
		Perl_Party_Pet_UpdateName();
	end
end

Perl_Party_Pet_Events.UNIT_FACTION = Perl_Party_Pet_Events.UNIT_NAME_UPDATE;

-- UNIT_LEVEL
function Perl_Party_Pet_Events:UNIT_LEVEL()
	if (arg1 == this.petunit) then
		Perl_Party_Pet_UpdateLevel();
	end
end

-- UNIT_HEALTH
function Perl_Party_Pet_Events:UNIT_HEALTH()
	if (arg1 == this.petunit) then
		Perl_Party_Pet_UpdateHealth()
	end
end

-- UNIT_MAXHEALTH
function Perl_Party_Pet_Events:UNIT_MAXHEALTH()
	if (arg1 == this.petunit) then
		Perl_Party_Pet_UpdateHealth()
		Perl_Party_Pet_UpdateLevel();
	end
end

-- UNIT_AURA
function Perl_Party_Pet_Events:UNIT_AURA()
	if (arg1 == this.petunit) then
		Perl_Party_Pet_Buff_UpdateAll();
	end
end

-- UNIT_DISPLAYPOWER
function Perl_Party_Pet_Events:UNIT_DISPLAYPOWER()
	if (arg1 == this.petunit) then
		Perl_Party_Pet_UpdateManaType();
	end
end

-- UNIT_MANA
function Perl_Party_Pet_Events:UNIT_MANA()
	if (arg1 == this.petunit) then
		Perl_Party_Pet_UpdateMana();
	end
end

Perl_Party_Pet_Events.UNIT_MAXMANA	= Perl_Party_Pet_Events.UNIT_MANA;
Perl_Party_Pet_Events.UNIT_RAGE		= Perl_Party_Pet_Events.UNIT_MANA;
Perl_Party_Pet_Events.UNIT_MAXRAGE	= Perl_Party_Pet_Events.UNIT_MANA;
Perl_Party_Pet_Events.UNIT_FOCUS	= Perl_Party_Pet_Events.UNIT_MANA;
Perl_Party_Pet_Events.UNIT_MAXFOCUS	= Perl_Party_Pet_Events.UNIT_MANA;
Perl_Party_Pet_Events.UNIT_ENERGY	= Perl_Party_Pet_Events.UNIT_MANA;
Perl_Party_Pet_Events.UNIT_MAXENERGY	= Perl_Party_Pet_Events.UNIT_MANA;


-- Perl_Party_Pet_Set_Name
function Perl_Party_Pet_Set_Name()
	for i = 1,4 do
		local f = getglobal("Perl_Party_Pet"..i.."_NameFrame");
		if (Perl_Config.ShowPartyPetName == 1) then
			f:Show();
			f:SetHeight(24);
		else
			f:Hide();
			f:SetHeight(3);
		end
	end
end
