local Perl_Player_Events = {}

----------------------
-- Loading Function --
----------------------
function Perl_Player_OnLoad()

	-- Events
	CombatFeedback_Initialize(Perl_PlayerHitIndicator, 30);
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_LEAVING_WORLD");
	this:RegisterEvent("VARIABLES_LOADED");

	table.insert(UnitPopupFrames,"Perl_Player_DropDown");

	Perl_Player_StatsFrame_DruidBar:SetHeight(1);
	Perl_Player_StatsFrame_DruidBar:Hide();

	Perl_Player_Frame.PlayerFlash = 0;
	Perl_Player_Frame.EnergyTime = 0;
	Perl_Player_Frame.EnergyLast = 0;

	this.time = 0;
end

-------------------------
-- The Update Function --
-------------------------

local function Perl_Player_CombatFlash(elapsed, argNew, argGreen)
	if (Perl_CombatFlashSet (elapsed, this, argNew, argGreen)) then
		Perl_CombatFlashSetFrames(Perl_Player_PortraitFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Player_NameFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Player_LevelFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Player_StatsFrame, this.PlayerFlash, this.PlayerFlashGreen);
	end
end

function Perl_Player_UpdatePortrait()
	if (Perl_Config.ShowPlayerPortrait == 1) then
		if (Perl_Config.ShowPlayerPortrait3D == 1) then
			Perl_Player_PortraitFrame_Portrait:Hide();
			Perl_Player_PortraitFrame_Portrait3D:Show();
			PerlSetPortrait3D(Perl_Player_PortraitFrame_Portrait3D, "player");
		else
			Perl_Player_PortraitFrame_Portrait:Show();
			Perl_Player_PortraitFrame_Portrait3D:Hide();
			SetPortraitTexture(Perl_Player_PortraitFrame_Portrait, "player");
		end
	end
end

local function Perl_Player_UpdateManaType()
	Perl_SetManaBarType("player", Perl_Player_StatsFrame_ManaBar, Perl_Player_StatsFrame_ManaBarBG);

	if (UnitPowerType("player") == 3) then
		Perl_Player_Frame.EnergyEnabled = true;
	else
		Perl_Player_Frame.EnergyEnabled = nil;
	end
	Perl_Player_TickerShowHide();
end

function Perl_Player_UpdateLeader()
	if (IsPartyLeader()) then
		Perl_Player_LeaderIcon:Show();
	else
		Perl_Player_LeaderIcon:Hide();
	end

	if (Perl_Config.ShowPartyNumber == 1 and UnitName("raid1")) then
		for i = 1,GetNumRaidMembers() do
			local name, rank, subgroup = GetRaidRosterInfo(i);
			if (name == UnitName("player")) then
				Perl_Player_Group:SetText("Group "..subgroup);
				Perl_Player_GroupFrame:Show();
				return;
			end
		end
	end

	Perl_Player_GroupFrame:Hide();
end

local function Perl_Player_UpdateMasterLooter()
	local method, index = GetLootMethod();

	if (method == "master" and index == 0) then
		Perl_Player_MasterIcon:Show();
	else
		Perl_Player_MasterIcon:Hide();
	end
end

local function Perl_Player_UpdateName()
	Perl_Player_NameBarText:SetText(UnitName("player"));
end

local function Perl_Player_UpdateRep()
	local name, reaction, min, max, value = GetWatchedFactionInfo();
	local color;

	if (name) then
		color = FACTION_BAR_COLORS[reaction];
	else
		name = "none watched";
		max = 1;
		min = 0;
		value = 0;
		color = FACTION_BAR_COLORS[4];
	end

	value = value - min;
	max = max - min;
	min = 0;

	Perl_Player_StatsFrame_XPRestBar:SetMinMaxValues(0, 1);
	Perl_Player_StatsFrame_XPRestBar:SetValue(0);

	Perl_Player_StatsFrame_XPBar:SetMinMaxValues(min, max);
	Perl_Player_StatsFrame_XPBar:SetValue(value);

	Perl_Player_StatsFrame_XPBar:SetStatusBarColor(color.r, color.g, color.b, 1);
	Perl_Player_StatsFrame_XPBarBG:SetVertexColor(color.r, color.g, color.b, 0.25);

	local perc = (value * 100) / max;
	Perl_Player_StatsFrame_XPBarPercent:SetText(string.format("%.1f%%", perc));
	Perl_Player_StatsFrame_XPBarText:SetText(name);
end

local function Perl_Player_UpdateXP()
	if (UnitLevel("player") == MAX_PLAYER_LEVEL) then
		Perl_Player_UpdateRep();
		return;
	end

	local playerxp = UnitXP("player");
	local playerxpmax = UnitXPMax("player");
	local playerxprest = GetXPExhaustion() or 0;
	Perl_Player_StatsFrame_XPBar:SetMinMaxValues(0, playerxpmax);
	Perl_Player_StatsFrame_XPRestBar:SetMinMaxValues(0, playerxpmax);
	Perl_Player_StatsFrame_XPBar:SetValue(playerxp);
	local xptext;

	if (playerxpmax > 10000) then
		xptext = string.format("%.0fK/%.0fK", playerxp / 1000, playerxpmax / 1000);
	else
		xptext = playerxp.."/"..playerxpmax;
	end

	if (playerxprest > 0) then
		if (playerxpmax > 10000) then
			xptext = xptext .. string.format("(+%.0fK)", playerxprest / 1000);
		else
			xptext = xptext .. string.format("(+%d)", playerxprest);
		end

		color = {r = 0.3, g = 0.3, b = 1};
	else
		color = {r = 0.6, g = 0, b = 0.6};
	end

	Perl_Player_StatsFrame_XPRestBar:SetValue(playerxp + playerxprest);
	Perl_Player_StatsFrame_XPBar:SetStatusBarColor(color.r, color.g, color.b, 1);
	Perl_Player_StatsFrame_XPRestBar:SetStatusBarColor(color.r, color.g, color.b, 0.5);
	Perl_Player_StatsFrame_XPBarBG:SetVertexColor(color.r, color.g, color.b, 0.25);

	Perl_Player_StatsFrame_XPBarText:SetText(xptext);

	local xppercenttext=((playerxp * 100.0) / playerxpmax);
	xppercenttext=string.format("%3.0f", xppercenttext);
	Perl_Player_StatsFrame_XPBarPercent:SetText(xppercenttext.."%");
end

local function Perl_Player_UpdateCombat()
	if (UnitAffectingCombat("player")) then
		Perl_Player_NameBarText:SetTextColor(1,0,0);
		Perl_Player_ActivityStatus:SetTexCoord(0.5, 1.0, 0.0, 0.5);
		Perl_Player_ActivityStatus:Show();
	else
		if (UnitIsPVP("player")) then
			Perl_Player_NameBarText:SetTextColor(0,1,0);
		else
			Perl_ColourFriendlyUnit(Perl_Player_NameBarText, "player");
		end

		if (IsResting()) then
			Perl_Player_ActivityStatus:SetTexCoord(0, 0.5, 0.0, 0.5);
			Perl_Player_ActivityStatus:Show();
		else
			Perl_Player_ActivityStatus:Hide();
		end
	end

	if (Perl_Player_Pet_Frame) then
        	Perl_Player_Pet_UpdateCombat();
	end
end

local function Perl_Player_UpdatePVP()
	local playerrankname, playerrank=GetPVPRankInfo(UnitPVPRank("player"), "player");
	if (playerrank and Perl_Config.ShowPlayerPVPRank==1) then
		Perl_Player_PVPRankIcon:Show();
		if (playerrank==0) then
			Perl_Player_PVPRankIcon:Hide();
		else
			Perl_Player_PVPRankIcon:SetTexture(string.format("Interface\\PVPRankBadges\\PVPRank%02d", playerrank));
		end
	else
		Perl_Player_PVPRankIcon:Hide();
	end

	if (Perl_Config.ShowPlayerClassIcon == 1) then
		local _, PlayerClass = UnitClass("player");
		local r, l, t, b = Perl_ClassPos(PlayerClass);
		Perl_Player_ClassTexture:SetTexCoord(r, l, t, b);
	end

	-- PVP Status settings
	if (UnitAffectingCombat("player")) then
		Perl_Player_NameBarText:SetTextColor(1,0,0);
	elseif (UnitIsPVP("player")) then
		Perl_Player_NameBarText:SetTextColor(0,1,0);
		Perl_Player_PVPStatus:Show();
		Perl_Player_PVPStatus:SetTexture("Interface\\TargetingFrame\\UI-PVP-"..UnitFactionGroup("player"));
	else
		Perl_Player_PVPStatus:Hide();
		Perl_ColourFriendlyUnit(Perl_Player_NameBarText, "player");
	end
end

local function Perl_Player_Feigning ()
	for buffnum=1,16 do
		if (UnitBuff("player", buffnum)) then
			if (strfind(strlower(UnitBuff("player", buffnum)), "feigndeath")) then
				return true;
			end
		end
	end
	return false;
end

local function Perl_Player_CheckDeadOrGhost()
	if (UnitIsDead("player")) then
		if (Perl_Player_Feigning ()) then
			Perl_Player_StatsFrame_HealthBarText:SetText(PERL_LOC_FEIGNDEATH);
			Perl_Player_StatsFrame_HealthBarText:Show();
			return false;
		else
			Perl_Player_StatsFrame_HealthBarText:SetText(PERL_LOC_DEAD);
			Perl_Player_StatsFrame_HealthBarText:Show();
			Perl_Player_StatsFrame_ManaBarText:Hide();
			return true;
		end

	elseif (UnitIsGhost("player")) then
		Perl_Player_StatsFrame_HealthBarText:SetText(PERL_LOC_GHOST);
		Perl_Player_StatsFrame_HealthBarText:Show();
		Perl_Player_StatsFrame_ManaBarText:Hide();
		return true;
	end

	return false;
end

local function Perl_Player_DruidBarUpdate()
	Perl_Player_StatsFrame_DruidBar:SetMinMaxValues(0,DruidBarKey.maxmana);
	Perl_Player_StatsFrame_DruidBar:SetValue(DruidBarKey.keepthemana);
	manaPct = (DruidBarKey.keepthemana * 100.0) / DruidBarKey.maxmana;
	manaPct =  string.format("%3.0f", manaPct);
	Perl_Player_StatsFrame_DruidBarPercent:SetText(manaPct.."%");
	Perl_Player_StatsFrame_DruidBarText:SetText(math.floor(DruidBarKey.keepthemana).."/"..DruidBarKey.maxmana);
	if (UnitPowerType("player")>0) then
		if Perl_Config.ShowPlayerXPBar==1 then
			Perl_Player_StatsFrame:SetHeight(60);
		else
			Perl_Player_StatsFrame:SetHeight(50);
		end
		Perl_Player_StatsFrame_DruidBarText:Show();
		Perl_Player_StatsFrame_DruidBarPercent:Show();
		Perl_Player_StatsFrame_DruidBar:Show();
		Perl_Player_StatsFrame_DruidBar:SetHeight(10);
	else
		if Perl_Config.ShowPlayerXPBar == 1 then
			Perl_Player_StatsFrame:SetHeight(50);
		else
			Perl_Player_StatsFrame:SetHeight(40);
		end
		Perl_Player_StatsFrame_DruidBarPercent:Hide();
		Perl_Player_StatsFrame_DruidBarText:Hide();
		Perl_Player_StatsFrame_DruidBar:Hide();
		Perl_Player_StatsFrame_DruidBar:SetHeight(1);
	end
	Perl_StatsFrame_Setup(Perl_Player_StatsFrame, {"DruidBar", "XPBar"});
end

local function Perl_Player_UpdateMana()
	local playerhealth = UnitHealth("player");
	local playerhealthmax = UnitHealthMax("player");
	local playermana = UnitMana("player");
	local playermanamax = UnitManaMax("player");
	Perl_Player_StatsFrame_ManaBar:SetMinMaxValues(0, playermanamax);
	Perl_Player_StatsFrame_ManaBar:SetValue(playermana);
	manaPct = (playermana * 100.0) / playermanamax
	manaPct =  string.format("%3.0f", manaPct);

	Perl_Player_StatsFrame_ManaBarText:SetText(playermana.."/"..playermanamax);
	if (UnitPowerType("player")>=1) then
		Perl_Player_StatsFrame_ManaBarPercent:SetText(playermana);
	else
		Perl_Player_StatsFrame_ManaBarPercent:SetText(manaPct.."%");
	end

	if (Perl_Config.ShowPlayerValues == 1) then
		Perl_Player_StatsFrame_ManaBarText:Show();
	else
		Perl_Player_StatsFrame_ManaBarText:Hide();
	end

	if (DruidBarKey) then
		local _, engClass = UnitClass("player");
		if (engClass == "DRUID") then
			Perl_Player_DruidBarUpdate();
		end
	end
end

local function Perl_Player_UpdateHealth()
	local playerhealth = UnitHealth("player");
	local playerhealthmax = UnitHealthMax("player");

	Perl_Player_StatsFrame_HealthBar:SetMinMaxValues(0, playerhealthmax);
	Perl_Player_StatsFrame_HealthBar:SetValue(playerhealth);

	Perl_SetSmoothBarColor(Perl_Player_StatsFrame_HealthBar, Perl_Player_StatsFrame_HealthBarBG, 0.25);

	healthPct = (playerhealth * 100.0) / playerhealthmax
	healthPct = string.format("%3.0f", healthPct);

	Perl_Player_StatsFrame_HealthBarPercent:SetText(healthPct.."%");

	if (Perl_Config.HealerMode == 1) then
		Perl_Player_StatsFrame_HealthBarText:SetText(-(playerhealthmax - playerhealth));
	else
		Perl_Player_StatsFrame_HealthBarText:SetText(playerhealth.."/"..playerhealthmax);
	end

	Perl_Player_CheckDeadOrGhost();
end

local function Perl_Player_UpdateLevel()
	Perl_Player_LevelFrame_LevelBarText:SetText(UnitLevel("player"));
end

function Perl_Player_OnUpdate(elapsed)
	if elapsed then
		Perl_Player_CombatFlash(elapsed, false);
	end
end

function Perl_Player_UpdateDisplay ()
	Perl_Player_UpdateXP();
	Perl_Player_UpdateManaType();
	if (Perl_Config.ShowPlayerLevel) then
		Perl_Player_UpdateLevel();
	end
	Perl_Player_UpdateName();
	Perl_Player_UpdatePVP();
	Perl_Player_UpdateCombat();
	Perl_Player_UpdateLeader();
	Perl_Player_UpdateMasterLooter();
	Perl_Player_UpdateMana();
	Perl_Player_UpdateHealth();
end

--------------------
-- Click Handlers --
--------------------

function Perl_PlayerDropDown_OnLoad()
	UIDropDownMenu_Initialize(this, Perl_PlayerDropDown_Initialize, "MENU");
end

function Perl_PlayerDropDown_Initialize()
	UnitPopup_ShowMenu(Perl_Player_DropDown, "SELF", "player");
end

function Perl_Player_OnClick(button)
	if (Perl_OnClick_Handler(button, "player")) then
		return;
	end

	if (button == "RightButton") then
		ToggleDropDownMenu(1, nil, Perl_Player_DropDown, "Perl_Player_NameFrame", 40, 0);
	end
end

-- EVENTS AND STUFF
local function Perl_Player_RegisterSome()
	this:RegisterEvent("PARTY_MEMBERS_CHANGED");
	this:RegisterEvent("PARTY_LEADER_CHANGED");
	this:RegisterEvent("PARTY_LOOT_METHOD_CHANGED");
	this:RegisterEvent("RAID_ROSTER_UPDATE");
	this:RegisterEvent("UNIT_SPELLMISS");
	this:RegisterEvent("PLAYER_UPDATE_RESTING");
	this:RegisterEvent("UNIT_FACTION");
	this:RegisterEvent("UNIT_MODEL_CHANGED");
	--this:RegisterEvent("UNIT_PORTRAIT_UPDATE");
	this:RegisterEvent("PLAYER_REGEN_ENABLED");
	this:RegisterEvent("PLAYER_REGEN_DISABLED");
	this:RegisterEvent("PLAYER_ENTER_COMBAT");
	this:RegisterEvent("PLAYER_LEAVE_COMBAT");
	this:RegisterEvent("PLAYER_LEVEL_UP");
	this:RegisterEvent("PLAYER_DEAD");
	this:RegisterEvent("UNIT_FLAGS");
	this:RegisterEvent("PLAYER_XP_UPDATE");
	this:RegisterEvent("UPDATE_FACTION");
	this:RegisterEvent("PLAYER_AURAS_CHANGED");
	this:RegisterEvent("UPDATE_SHAPESHIFT_FORMS");			-- For DruidBar mana update
	Perl_RegisterBasics();
end
local function Perl_Player_UnregisterSome()
	this:UnregisterEvent("PARTY_MEMBERS_CHANGED");
	this:UnregisterEvent("PARTY_LEADER_CHANGED");
	this:UnregisterEvent("PARTY_LOOT_METHOD_CHANGED");
	this:UnregisterEvent("RAID_ROSTER_UPDATE");
	this:UnregisterEvent("UNIT_SPELLMISS");
	this:UnregisterEvent("PLAYER_UPDATE_RESTING");
	this:UnregisterEvent("UNIT_FACTION");
	this:UnregisterEvent("UNIT_MODEL_CHANGED");
	--this:UnregisterEvent("UNIT_PORTRAIT_UPDATE");
	this:UnregisterEvent("PLAYER_REGEN_ENABLED");
	this:UnregisterEvent("PLAYER_REGEN_DISABLED");
	this:UnregisterEvent("PLAYER_ENTER_COMBAT");
	this:UnregisterEvent("PLAYER_LEAVE_COMBAT");
	this:UnregisterEvent("PLAYER_LEVEL_UP");
	this:UnregisterEvent("PLAYER_DEAD");
	this:UnregisterEvent("UNIT_FLAGS");
	this:UnregisterEvent("PLAYER_XP_UPDATE");
	this:UnregisterEvent("UPDATE_FACTION");
	this:UnregisterEvent("PLAYER_AURAS_CHANGED");
	this:UnregisterEvent("UPDATE_SHAPESHIFT_FORMS");
	Perl_UnregisterBasics();
end

-------------------
-- Event Handler --
-------------------
function Perl_Player_OnEvent()
	local func = Perl_Player_Events[event];
	if (func) then
		func();
	else
Perl_ShowMessage("EXTRA EVENT");
	end
end

-- PLAYER_ENTERING_WORLD
function Perl_Player_Events:PLAYER_ENTERING_WORLD()
	Perl_Player_RegisterSome();
	Perl_Player_UpdateDisplay();
end

-- PLAYER_LEAVING_WORLD
function Perl_Player_Events:PLAYER_LEAVING_WORLD()
	Perl_Player_UnregisterSome();
end

-- UNIT_COMBAT
function Perl_Player_Events:UNIT_COMBAT()
	if (arg1 == "player") then
		if (Perl_Config.CombatHitIndicator == 1 and Perl_Config.ShowPlayerPortrait == 1) then
			CombatFeedback_OnCombatEvent(arg2, arg3, arg4, arg5);
		end

		if (arg2 == "HEAL") then
			Perl_Player_CombatFlash(elapsed, true, true);
		elseif (arg4 and arg4 > 0) then
			Perl_Player_CombatFlash(elapsed, true);
		end
	end
end

-- UNIT_SPELLMISS
function Perl_Player_Events:UNIT_SPELLMISS()
	if (arg1 == "player") then
		if (Perl_Config.CombatHitIndicator == 1 and Perl_Config.ShowPlayerPortrait == 1) then
			CombatFeedback_OnSpellMissEvent(arg2);
		end
	end
end

-- UNIT_MODEL_CHANGED
function Perl_Player_Events:UNIT_MODEL_CHANGED()
	if (arg1 == "player") then
		Perl_Player_UpdatePortrait();
	end
end
Perl_Player_Events.UNIT_PORTRAIT_UPDATE = Perl_Player_Events.UNIT_MODEL_CHANGED;

-- VARIABLES_LOADED
function Perl_Player_Events:VARIABLES_LOADED()
	Perl_Player_UpdatePortrait();
	this:UnregisterEvent(event);
end

-- PARTY_LOOT_METHOD_CHANGED
function Perl_Player_Events:PARTY_LOOT_METHOD_CHANGED()
	Perl_Player_UpdateLeader();
	Perl_Player_UpdateMasterLooter();
end
Perl_Player_Events.PARTY_MEMBERS_CHANGED = Perl_Player_Events.PARTY_LOOT_METHOD_CHANGED;
Perl_Player_Events.PARTY_LEADER_CHANGED  = Perl_Player_Events.PARTY_LOOT_METHOD_CHANGED;

function Perl_Player_Events:RAID_ROSTER_UPDATE()
	Perl_Player_UpdateLeader();
	Perl_Player_UpdateMasterLooter();
end

-- UNIT_HEALTH, UNIT_MAXHEALTH
function Perl_Player_Events:UNIT_HEALTH()
	if (arg1=="player") then
		Perl_Player_UpdateHealth();
	end
end
Perl_Player_Events.UNIT_MAXHEALTH = Perl_Player_Events.UNIT_HEALTH;
Perl_Player_Events.PLAYER_DEAD    = Perl_Player_Events.UNIT_HEALTH;

-- UNIT_MANA, UNIT_MAXMANA, UNIT_RAGE, UNIT_MAXRAGE, UNIT_ENERGY
-- UNIT_MAXENERGY, UNIT_FOCUS, UNIT_MAXFOCUS
function Perl_Player_Events:UNIT_MANA()
	if (arg1=="player") then
		Perl_Player_UpdateMana();
	end
end
Perl_Player_Events.UNIT_MAXMANA		= Perl_Player_Events.UNIT_MANA;
Perl_Player_Events.UNIT_RAGE		= Perl_Player_Events.UNIT_MANA;
Perl_Player_Events.UNIT_MAXRAGE         = Perl_Player_Events.UNIT_MANA;
Perl_Player_Events.UNIT_FOCUS           = Perl_Player_Events.UNIT_MANA;
Perl_Player_Events.UNIT_MAXFOCUS	= Perl_Player_Events.UNIT_MANA;

local function CheckStealth()
	local i = 0
	while (GetPlayerBuffTexture(i) ~= nil) do		-- check for rogue's stealth and druid's prowl
		if (GetPlayerBuffTexture(i) == "Interface\\Icons\\Ability_Stealth" or
          		GetPlayerBuffTexture(i) == "Interface\\Icons\\Ability_Ambush") then
			return true;
		else
			i = i + 1;
		end
	end
end

function Perl_Player_Events:UNIT_ENERGY()
	if (arg1=="player") then
		Perl_Player_UpdateMana();

		local e = UnitMana("player");
		local m = UnitManaMax("player");
		if (e == Perl_Player_Frame.EnergyLast + 20) then
			Perl_Player_Frame.EnergyTime = GetTime();
		end
		Perl_Player_Frame.EnergyLast = e;

		Perl_Player_TickerShowHide();
	end
end
Perl_Player_Events.UNIT_MAXENERGY       = Perl_Player_Events.UNIT_ENERGY;

-- UNIT_DISPLAYPOWER
function Perl_Player_Events:UNIT_DISPLAYPOWER()
	Perl_Player_UpdateManaType();
	Perl_Player_UpdateMana();
end

-- UNIT_NAME_UPDATE
function Perl_Player_Events:UNIT_NAME_UPDATE()
	if (arg1=="player") then
		Perl_Player_UpdateName();
	end
end

-- UNIT_LEVEL
function Perl_Player_Events:UNIT_LEVEL()
	if (arg1 == "player") then
		if (Perl_Config.ShowPlayerLevel) then
			Perl_Player_UpdateLevel()
			Perl_Player_UpdateXP();
		end
	end
end

Perl_Player_Events.PLAYER_LEVEL_UP = Perl_Player_Events.UNIT_LEVEL;

-- PLAYER_XP_UPDATE
function Perl_Player_Events:PLAYER_XP_UPDATE()
	Perl_Player_UpdateXP();
end

function Perl_Player_Events:UPDATE_FACTION()
	if (UnitLevel("player") == MAX_PLAYER_LEVEL) then
		Perl_Player_UpdateRep();
	end
end

-- UNIT_FACTION
function Perl_Player_Events:UNIT_FACTION()
	if (arg1 == "player") then
		Perl_Player_UpdatePVP();
	end
end

Perl_Player_Events.UNIT_FLAGS = Perl_Player_Events.UNIT_FACTION;

-- PLAYER_ENTER_COMBAT, PLAYER_LEAVE_COMBAT
function Perl_Player_Events:PLAYER_ENTER_COMBAT()
	Perl_Player_UpdateCombat();
end
Perl_Player_Events.PLAYER_LEAVE_COMBAT = Perl_Player_Events.PLAYER_ENTER_COMBAT;

-- PLAYER_REGEN_ENABLED, PLAYER_REGEN_DISABLED
function Perl_Player_Events:PLAYER_REGEN_ENABLED()
	Perl_Player_UpdateCombat();
	--Perl_Player_UpdateDisplay();
end
Perl_Player_Events.PLAYER_REGEN_DISABLED = Perl_Player_Events.PLAYER_REGEN_ENABLED;
Perl_Player_Events.PLAYER_UPDATE_RESTING = Perl_Player_Events.PLAYER_REGEN_ENABLED;

function Perl_Player_Events:PLAYER_AURAS_CHANGED()
	Perl_CheckDebuffs("player", {Perl_Player_NameFrame, Perl_Player_PortraitFrame, Perl_Player_LevelFrame, Perl_Player_StatsFrame});
	Perl_Player_TickerShowHide();

	if (DruidBarKey) then
		-- For DruidBar addon, we update the mana bar on shapeshift
		local _, engClass = UnitClass("player");
		if (engClass == "DRUID") then
			Perl_Player_UpdateMana();
		end
	end
end

-- Moving stuff
function Perl_Player_GetGap()
	if (Perl_Target_Frame) then
		local playerLeft = Perl_Player_Frame:GetRight() * Perl_Player_Frame:GetEffectiveScale();
		local targetLeft = Perl_Target_Frame:GetLeft() * Perl_Target_Frame:GetEffectiveScale();

		local a = targetLeft - playerLeft;

		a = a + 0.01;
		a = a * 100;
		a = math.floor (a);
		a = a / 100;

		return (math.floor(a + 4));
	end

	return 0;
end

function Perl_Player_SetGap(newGap)
	if (not Perl_Target_Frame) then
		return;
	end

	if (type(newGap) == "number") then
		newGap = newGap - 4;

		local top = Perl_Target_Frame:GetTop();
		local left = ((Perl_Player_Frame:GetRight() * Perl_Player_Frame:GetEffectiveScale()) + newGap) / Perl_Target_Frame:GetEffectiveScale();

		Perl_Target_Frame:ClearAllPoints();
		Perl_Target_Frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", left, top);

		Perl_Target_Frame:StartMoving();
		Perl_Target_Frame:StopMovingOrSizing();

		if (Perl_TargetTarget) then
			top = Perl_TargetTarget:GetTop();
			left = ((Perl_Target_Frame:GetRight() * Perl_Target_Frame:GetEffectiveScale()) + newGap) / Perl_TargetTarget:GetEffectiveScale();

			-- Can't use the CastClickOverlay for GetRight() as wow doesn't update anchors until it renders the frame, anchors
			-- are always 1 frame behind. Usually you won't notice, but when moving frames in this manner, it was ugly.
			-- So, we adjust right a little.
			if (Perl_Target_LevelFrame:IsShown()) then
				left = left + Perl_Target_LevelFrame:GetWidth();
			end

			Perl_TargetTarget:ClearAllPoints();
			Perl_TargetTarget:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", left, top);

			if (Perl_TargetTargetTarget) then
				top = Perl_TargetTargetTarget:GetTop();
				left = (Perl_TargetTarget:GetRight() * Perl_TargetTarget:GetEffectiveScale()) + newGap;
				if (Perl_Config.ShowTargetTargetPercent == 0) then
					left = left - 20;
				end
				left = left / Perl_TargetTargetTarget:GetEffectiveScale();

				Perl_TargetTargetTarget:ClearAllPoints();
				Perl_TargetTargetTarget:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", left, top);

				Perl_TargetTargetTarget:StartMoving();
				Perl_TargetTargetTarget:StopMovingOrSizing();
			end

			Perl_TargetTarget:StartMoving();
			Perl_TargetTarget:StopMovingOrSizing();
		end
	end
end

function Perl_Player_AlignTop()
	if (not Perl_Target_Frame) then
		return;
	end

	local playerLeft = Perl_Player_Frame:GetLeft();
	local targetLeft = Perl_Target_Frame:GetLeft();
	local playerTop = Perl_Player_Frame:GetTop();
	local top = playerTop * Perl_Player_Frame:GetEffectiveScale();
	local newTop;

	if (playerLeft == nil) then
		playerLeft = 5;
	end
	if (targetLeft == nil) then
		targetLeft = 220;
	end

	-- We set this for 1 reason, so that all the related frames scale in the same direction should the user do that...
	Perl_Player_Frame:ClearAllPoints();
	Perl_Player_Frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", playerLeft, playerTop);

	Perl_Target_Frame:ClearAllPoints();
	newTop = top / Perl_Target_Frame:GetEffectiveScale();
	Perl_Target_Frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", targetLeft, newTop);

	Perl_Target_Frame:StartMoving();
	Perl_Target_Frame:StopMovingOrSizing();

	if (Perl_TargetTarget) then
		local targettargetLeft = Perl_TargetTarget:GetLeft();

		if (targettargetLeft == nil) then
			targettargetLeft = targetLeft + 200;
		end

		Perl_TargetTarget:ClearAllPoints();
		newTop = top / Perl_TargetTarget:GetEffectiveScale();
		Perl_TargetTarget:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", targettargetLeft, newTop);

		if (Perl_TargetTargetTarget) then
			local targettargettargetLeft = Perl_TargetTargetTarget:GetLeft();
			if (targettargettargetLeft == nil) then
				targettargettargetLeft = targettargetLeft + 200;
			end

			Perl_TargetTargetTarget:ClearAllPoints();
			newTop = top / Perl_TargetTargetTarget:GetEffectiveScale();
			Perl_TargetTargetTarget:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", targettargettargetLeft, newTop);

			Perl_TargetTargetTarget:StartMoving();
			Perl_TargetTargetTarget:StopMovingOrSizing();
		end

		Perl_TargetTarget:StartMoving();
		Perl_TargetTarget:StopMovingOrSizing();
	end
end


-- Perl_Player_Energy_TickWatch
function Perl_Player_Energy_OnUpdate()

	local time = GetTime();
	local temp = time - Perl_Player_Frame.EnergyTime;
	local remainder = mod (temp, 2);

	Perl_Player_StatsFrame_EnergyTicker:SetValue(remainder);

	sparkPosition = Perl_Player_StatsFrame_EnergyTicker:GetWidth() * (remainder / 2);

	Perl_Player_StatsFrame_EnergyTicker_Spark:SetPoint("CENTER", Perl_Player_StatsFrame_EnergyTicker, "LEFT", sparkPosition, 0);
end

function Perl_Player_TickerShowHide()
       	local e = UnitMana("player");
       	local m = UnitManaMax("player");
       	if (Perl_Player_Frame.EnergyEnabled and (e < m or UnitAffectingCombat("player") or CheckStealth()) and not UnitIsDeadOrGhost("player") and Perl_Config.EnergyTicker == 1) then
       		Perl_Player_StatsFrame_EnergyTicker:Show();
       	else
       		Perl_Player_StatsFrame_EnergyTicker:Hide();
       	end
end

-- Perl_Player_Set_Bits()
function Perl_Player_Set_Bits()
	if (Perl_Config.ShowPlayerPortrait==0) then
		Perl_Player_PortraitFrame:Hide();
		Perl_Player_PortraitFrame:SetWidth(3);
		Perl_Config.ShowPlayerPortrait=0;
	else
		Perl_Player_PortraitFrame:Show();
		Perl_Player_PortraitFrame:SetWidth(60);
		Perl_Config.ShowPlayerPortrait=1;
		Perl_Player_UpdatePortrait();
	end

	if (Perl_Config.ShowPlayerLevel==0) then
		Perl_Player_LevelFrame:Hide();
	else
		Perl_Player_LevelFrame:Show();
	end

	if (Perl_Config.ShowPlayerPercent == 0) then
		Perl_Player_NameFrame:SetWidth(128);
		Perl_Player_StatsFrame:SetWidth(128);
		Perl_Player_StatsFrame_HealthBarPercent:Hide();
		Perl_Player_StatsFrame_ManaBarPercent:Hide();
		Perl_Player_StatsFrame_XPBarPercent:Hide();
	else
		Perl_Player_NameFrame:SetWidth(160);
		Perl_Player_StatsFrame:SetWidth(160);
		Perl_Player_StatsFrame_HealthBarPercent:Show();
		Perl_Player_StatsFrame_ManaBarPercent:Show();
		Perl_Player_StatsFrame_XPBarPercent:Show();
	end

	if (Perl_Config.ShowPlayerValues == 0) then
		Perl_Player_StatsFrame_HealthBarText:Hide();
		Perl_Player_StatsFrame_ManaBarText:Hide();
		Perl_Player_StatsFrame_XPBarText:Hide();
	else
		Perl_Player_StatsFrame_HealthBarText:Show();
		Perl_Player_StatsFrame_ManaBarText:Show();
		Perl_Player_StatsFrame_XPBarText:Show();
	end

	if (Perl_Config.ShowPlayerClassIcon == 0) then
		Perl_Player_ClassTexture:Hide();
	else
		Perl_Player_ClassTexture:Show();
	end

	if (Perl_Config.ShowPlayerPVPRank==0) then
		Perl_Player_PVPRankIcon:Hide();
	else
		Perl_Player_PVPRankIcon:Show();
	end

	if (Perl_Config.ShowPlayerXPBar==0) then
		Perl_Player_StatsFrame_XPBar:Hide();
		Perl_Player_StatsFrame_XPRestBar:Hide();
		Perl_Player_StatsFrame:SetHeight(40);
	else
		Perl_Player_StatsFrame_XPBar:Show();
		Perl_Player_StatsFrame_XPRestBar:Show();
		Perl_Player_StatsFrame:SetHeight(50);
	end
end
