local Perl_Player_Pet_Events = {}

----------------------
-- Loading Function --
----------------------
function Perl_Player_Pet_OnLoad()

	-- Events
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_LEAVING_WORLD");

	table.insert(UnitPopupFrames,"Perl_Player_Pet_DropDown");

	--Perl_Player_Pet_CastClickOverlay:SetFrameLevel(Perl_Player_Pet_StatsFrame:GetFrameLevel() + 2);
	--Perl_Player_PetHappiness:SetFrameLevel(Perl_Player_Pet_StatsFrame:GetFrameLevel() + 2);

	this.time = 0;
	Perl_InitFadeFrame(this);
end

-------------------
-- Event Handler --
-------------------
local function Perl_Player_Pet_RegisterSome()
	this:RegisterEvent("UNIT_PET");
	this:RegisterEvent("UNIT_HAPPINESS");
	this:RegisterEvent("PET_STABLE_UPDATE");
	this:RegisterEvent("PET_STABLE_SHOW");
	this:RegisterEvent("PET_STABLE_CLOSED");
	this:RegisterEvent("PET_STABLE_UPDATE_PAPERDOLL");
	this:RegisterEvent("UNIT_MODEL_CHANGED");
	this:RegisterEvent("UNIT_PORTRAIT_UPDATE");
end
local function Perl_Player_Pet_RegisterForPet()
	this:RegisterEvent("UNIT_AURA");
	this:RegisterEvent("UNIT_FOCUS");
	this:RegisterEvent("UNIT_MAXFOCUS");
	this:RegisterEvent("PET_ATTACK_START");
	this:RegisterEvent("UNIT_FACTION");
	this:RegisterEvent("UNIT_FLAGS");
	this:RegisterEvent("UNIT_DYNAMIC_FLAGS");
	this:RegisterEvent("UNIT_PET_EXPERIENCE");
	this:RegisterEvent("UNIT_MODEL_CHANGED");
	this:RegisterEvent("UNIT_PORTRAIT_UPDATE");
	Perl_RegisterBasics();
end

local function Perl_Player_Pet_UnregisterSome()
	this:UnregisterEvent("UNIT_PET");
	this:UnregisterEvent("UNIT_HAPPINESS");
	this:UnregisterEvent("PET_STABLE_UPDATE");
	this:UnregisterEvent("PET_STABLE_SHOW");
	this:UnregisterEvent("PET_STABLE_CLOSED");
	this:UnregisterEvent("PET_STABLE_UPDATE_PAPERDOLL");
end
local function Perl_Player_Pet_UnregisterForPet()
	this:UnregisterEvent("UNIT_AURA");
	this:UnregisterEvent("UNIT_FOCUS");
	this:UnregisterEvent("UNIT_MAXFOCUS");
	this:UnregisterEvent("PET_ATTACK_START");
	this:UnregisterEvent("UNIT_FACTION");
	this:UnregisterEvent("UNIT_FLAGS");
	this:UnregisterEvent("UNIT_DYNAMIC_FLAGS");
	this:UnregisterEvent("UNIT_PET_EXPERIENCE");
	Perl_UnregisterBasics();
end

-------------------------
-- The Update Function --
-------------------------
local function Perl_Player_Pet_CheckPet()
	if (UnitName("pet")) then
		if (UnitIsVisible("pet")) then
			Perl_CancelFade(Perl_Player_Pet_Frame);

			Perl_Player_Pet_Frame:Show();
			Perl_Player_Pet_RegisterForPet();
			return true;
		end
	end

	Perl_Player_Pet_UnregisterForPet();

	Perl_StartFade(Perl_Player_Pet_Frame);
	return false;
end

local function Perl_Player_Pet_UpdateName()
	local petname = UnitName("pet");

	if (petname == UNKNOWNOBJECT) then
		Perl_Player_Pet_NameBarText:SetText("");
	else
		Perl_Player_Pet_NameBarText:SetText(petname);
	end

	if (UnitIsPVP("player")) then
	        Perl_Player_Pet_NameBarText:SetTextColor(0, 1, 0, Perl_Config.TextTransparency);
	else
	        Perl_Player_Pet_NameBarText:SetTextColor(0.5, 0.5, 1, Perl_Config.TextTransparency);
	end
end

local function Perl_Player_Pet_UpdateLevel()
	local petlevel = UnitLevel("pet");
	local petxp, petxpmax = GetPetExperience();

	Perl_Player_Pet_StatsFrame_XPBar:SetStatusBarColor(0.3, 0.3, 1, 1);
	Perl_Player_Pet_StatsFrame_XPBarBG:SetVertexColor(0.3, 0.3, 1, 0.25);

	Perl_Player_Pet_StatsFrame_XPBar:SetMinMaxValues(0, petxpmax);
	Perl_Player_Pet_StatsFrame_XPBar:SetValue(petxp);

        Perl_Player_Pet_LevelBarText:SetText(petlevel);
end

local function Perl_Player_Pet_UpdateHealth()
	local pethealth = UnitHealth("pet");
	local pethealthmax = UnitHealthMax("pet");

	Perl_Player_Pet_StatsFrame_HealthBar:SetMinMaxValues(0, pethealthmax);
	Perl_Player_Pet_StatsFrame_HealthBar:SetValue(pethealth);

	Perl_SetSmoothBarColor(Perl_Player_Pet_StatsFrame_HealthBar, Perl_Player_Pet_StatsFrame_HealthBarBG, 0.25);

	if (UnitIsDead("pet")) then
		Perl_Player_Pet_StatsFrame_HealthBarText:SetText(PERL_LOC_DEAD);
		Perl_Player_Pet_StatsFrame_ManaBarText:Hide();
	else
		if (Perl_Config.HealerMode == 1) then
			Perl_Player_Pet_StatsFrame_HealthBarText:SetText(-(pethealthmax - pethealth));
		else
			Perl_Player_Pet_StatsFrame_HealthBarText:SetText(pethealth.."/"..pethealthmax);
		end
	end
end

local function Perl_Player_Pet_UpdateManaType()
	Perl_SetManaBarType("pet", Perl_Player_Pet_StatsFrame_ManaBar, Perl_Player_Pet_StatsFrame_ManaBarBG);
end

local function Perl_Player_Pet_UpdateMana()
	local petmana = UnitMana("pet");
	local petmanamax = UnitManaMax("pet");

	Perl_Player_Pet_StatsFrame_ManaBar:SetMinMaxValues(0, petmanamax);
	Perl_Player_Pet_StatsFrame_ManaBar:SetValue(petmana);

	Perl_Player_Pet_StatsFrame_ManaBarText:SetText(petmana.."/"..petmanamax);

	Perl_Player_Pet_StatsFrame_ManaBarText:Show();
end

local function Perl_Player_Pet_CombatFlash(elapsed, argNew, argGreen)
	if (Perl_CombatFlashSet (elapsed, this, argNew, argGreen)) then
		Perl_CombatFlashSetFrames(Perl_Player_Pet_NameFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Player_Pet_LevelFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Player_Pet_StatsFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Player_Pet_PortraitFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Player_Pet_HappyFrame, this.PlayerFlash, this.PlayerFlashGreen);
	end
end

function Perl_Player_Pet_UpdatePortrait()
	if (Perl_Config.ShowPlayerPetPortrait == 1) then
		if (Perl_Config.ShowPlayerPortrait3D == 1) then
			Perl_Player_Pet_PortraitFrame_Portrait:Hide();
			Perl_Player_Pet_PortraitFrame_Portrait3D:Show();
			PerlSetPortrait3D(Perl_Player_Pet_PortraitFrame_Portrait3D, "pet");
		else
			Perl_Player_Pet_PortraitFrame_Portrait:Show();
			Perl_Player_Pet_PortraitFrame_Portrait3D:Hide();
			SetPortraitTexture(Perl_Player_Pet_PortraitFrame_Portrait, "pet");
		end
	end
end

function Perl_Player_Pet_OnUpdate(elapsed)
	if elapsed then
		Perl_Player_Pet_CombatFlash(elapsed, false);
	end

	Perl_ProcessFade(Perl_Player_Pet_Frame);
end

--------------------
-- Buff Functions --
--------------------
local function Perl_Player_Pet_Buff_UpdateAll ()
	if (UnitName("pet")) then
		for buffnum=1,10 do
			local button = getglobal("Perl_Player_Pet_BuffFrame_Buff"..buffnum);
			local icon = getglobal("Perl_Player_Pet_BuffFrame_Buff"..buffnum.."Icon");
			local tex = UnitBuff("pet", buffnum);
			if (tex) then
				icon:SetTexture(tex);
				button:Show();
			else
				button:Hide();
			end
		end
		for buffnum=1,8 do
			local button = getglobal("Perl_Player_Pet_BuffFrame_DeBuff"..buffnum);
			local debuff, debuffApplications, debuffType = UnitDebuff("pet", buffnum);
			if (debuff) then
				local icon = getglobal("Perl_Player_Pet_BuffFrame_DeBuff"..buffnum.."Icon");
				local count = getglobal("Perl_Player_Pet_BuffFrame_DeBuff"..buffnum.."Count");
				local border = getglobal("Perl_Player_Pet_BuffFrame_DeBuff"..buffnum.."Border");

				icon:SetTexture(debuff);
				button:Show();

				if ( debuffApplications > 1 ) then
					count:SetText(debuffApplications);
					count:Show();
				else
					count:Hide();
				end

				local borderColor = DebuffTypeColor[(debuffType or "none")];
				border:SetVertexColor(borderColor.r, borderColor.g, borderColor.b);
			else
				button:Hide();
			end
		end

		Perl_CheckDebuffs("pet", {Perl_Player_Pet_NameFrame, Perl_Player_Pet_PortraitFrame, Perl_Player_Pet_LevelFrame, Perl_Player_Pet_StatsFrame});
	end
end

function Perl_Player_Pet_SetBuffTooltip ()
	GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");
	GameTooltip:SetUnitBuff("pet", this:GetID());
end

function Perl_Player_Pet_SetDeBuffTooltip ()
	GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");
	GameTooltip:SetUnitDebuff("pet", this:GetID());
end

---------------
-- Happiness --
---------------

local function Perl_Player_Pet_SetHappiness()
	local happiness, damagePercentage, loyaltyRate = GetPetHappiness();

	if (not happiness) then
		happiness = 3;
	end

	if ( happiness == 1 ) then
		Perl_Player_PetHappinessTexture:SetTexCoord(0.375, 0.5625, 0, 0.359375);
	elseif ( happiness == 2 ) then
		Perl_Player_PetHappinessTexture:SetTexCoord(0.1875, 0.375, 0, 0.359375);
	elseif ( happiness == 3 ) then
		Perl_Player_PetHappinessTexture:SetTexCoord(0, 0.1875, 0, 0.359375);
	end
	if ( happiness ~= nil ) then
		Perl_Player_PetHappiness.tooltip = getglobal("PET_HAPPINESS"..happiness);
		Perl_Player_PetHappiness.tooltipDamage = format(PET_DAMAGE_PERCENTAGE, damagePercentage);
		if ( loyaltyRate < 0 ) then
			Perl_Player_PetHappiness.tooltipLoyalty = getglobal("LOSING_LOYALTY");
		elseif ( loyaltyRate > 0 ) then
			Perl_Player_PetHappiness.tooltipLoyalty = getglobal("GAINING_LOYALTY");
		else
			Perl_Player_PetHappiness.tooltipLoyalty = nil;
		end
	end

	if (Perl_Config.PetHappiness == 1 and (Perl_Config.PetHappinessSad == 0 or happiness < 3)) then
		Perl_Player_Pet_HappyFrame:Show();

		if (Perl_Config.PetFlashWhenSad == 1 and happiness < 3) then
			if (not UIFrameIsFlashing(Perl_Player_Pet_HappyFrame)) then
				UIFrameFlash(Perl_Player_Pet_HappyFrame, 0.5, 0.5, 30, 1, 0, 1);
			end
		else
			if (UIFrameIsFlashing(Perl_Player_Pet_HappyFrame)) then
				UIFrameFlashStop(Perl_Player_Pet_HappyFrame);
                		UIFrameFlashUpdate(1);
			end
		end
	else
		if (UIFrameIsFlashing(Perl_Player_Pet_HappyFrame)) then
			UIFrameFlashStop(Perl_Player_Pet_HappyFrame);
                	UIFrameFlashUpdate(1);
		end
		Perl_Player_Pet_HappyFrame:Hide();
	end
end

--------------------
-- Click Handlers --
--------------------

function Perl_Player_Pet_DropDown_OnLoad()
	UIDropDownMenu_Initialize(this, Perl_Player_Pet_DropDown_Initialize, "MENU");
end

function Perl_Player_Pet_DropDown_Initialize ()
	UnitPopup_ShowMenu(Perl_Player_Pet_DropDown, "PET", "pet");
end

function Perl_Player_Pet_OnClick(button)

	if (Perl_OnClick_Handler(button, "pet")) then
		return;
	end

	if (button == "RightButton") then
		ToggleDropDownMenu(1, nil, Perl_Player_Pet_DropDown, "Perl_Player_Pet_NameFrame", 40, 0);
	end

end

local function Perl_Target_Update_Control()
        if (UnitIsCharmed("pet")) then
		Perl_Player_Pet_Warning:Show();
	else
		Perl_Player_Pet_Warning:Hide();
	end
end

function Perl_Player_Pet_UpdateCombat()
	if (UnitName("pet")) then
                if (UnitAffectingCombat("pet")) then
                        Perl_Player_Pet_ActivityStatus:Show();
                else
                        Perl_Player_Pet_ActivityStatus:Hide();
                end
		Perl_Target_Update_Control();
        end
end

function Perl_Player_Pet_UpdateDisplay ()
	if (Perl_Player_Pet_CheckPet()) then
		Perl_CancelFade(Perl_Player_Pet_Frame);

		Perl_Player_Pet_UpdatePortrait();
		Perl_Player_Pet_UpdateName();
		Perl_Player_Pet_UpdateHealth();
		Perl_Player_Pet_UpdateManaType();
		Perl_Player_Pet_UpdateMana();
		Perl_Player_Pet_UpdateLevel();
		Perl_Player_Pet_SetHappiness();
		Perl_Player_Pet_Buff_UpdateAll();

	        Perl_Player_Pet_UpdateCombat();
	end
end

-------------------
-- Event Handler --
-------------------
function Perl_Player_Pet_OnEvent()
	local func = Perl_Player_Pet_Events[event];
	if (func) then
		func();
	else
Perl_ShowMessage("EXTRA EVENT");
	end
end

-- PLAYER_ENTERING_WORLD
function Perl_Player_Pet_Events:PLAYER_ENTERING_WORLD()
	Perl_Player_Pet_Frame.PlayerFlash = 0;
	Perl_Player_Pet_RegisterSome();
end

-- PLAYER_LEAVING_WORLD
function Perl_Player_Pet_Events:PLAYER_LEAVING_WORLD()
	Perl_Player_Pet_UnregisterSome();
end

-- UNIT_AURA
function Perl_Player_Pet_Events:UNIT_AURA()
	if (arg1 == "pet") then
		Perl_Player_Pet_Buff_UpdateAll();
	end
end

-- UNIT_PET
function Perl_Player_Pet_Events:UNIT_PET()
	Perl_Player_Pet_UpdateDisplay();
end

Perl_Player_Pet_Events.PET_STABLE_SHOW = Perl_Player_Pet_Events.UNIT_PET;

-- UNIT_NAME_UPDATE
function Perl_Player_Pet_Events:UNIT_NAME_UPDATE()
	if (arg1 == "pet") then
		Perl_Player_Pet_UpdateName();
	end
end

-- UNIT_MODEL_CHANGED
function Perl_Player_Pet_Events:UNIT_MODEL_CHANGED()
	if (arg1 == "pet") then
		Perl_Player_Pet_UpdatePortrait();
	end
end
Perl_Player_Pet_Events.UNIT_PORTRAIT_UPDATE = Perl_Player_Pet_Events.UNIT_MODEL_CHANGED;

-- UNIT_HEALTH, UNIT_MAXHEALTH
function Perl_Player_Pet_Events:UNIT_HEALTH()
	if (arg1 == "pet") then
		Perl_Player_Pet_UpdateHealth();
	end
end
Perl_Player_Pet_Events.UNIT_MAXHEALTH = Perl_Player_Pet_Events.UNIT_HEALTH;

-- UNIT_RAGE
function Perl_Player_Pet_Events:UNIT_RAGE()
	if (arg1 == "pet") then
		Perl_Player_Pet_UpdateMana();
	end
end

Perl_Player_Pet_Events.UNIT_MAXRAGE	= Perl_Player_Pet_Events.UNIT_RAGE;
Perl_Player_Pet_Events.UNIT_ENERGY	= Perl_Player_Pet_Events.UNIT_RAGE;
Perl_Player_Pet_Events.UNIT_MAXENERGY	= Perl_Player_Pet_Events.UNIT_RAGE;
Perl_Player_Pet_Events.UNIT_MANA	= Perl_Player_Pet_Events.UNIT_RAGE;
Perl_Player_Pet_Events.UNIT_MAXMANA	= Perl_Player_Pet_Events.UNIT_RAGE;
Perl_Player_Pet_Events.UNIT_FOCUS	= Perl_Player_Pet_Events.UNIT_RAGE;
Perl_Player_Pet_Events.UNIT_MAXFOCUS	= Perl_Player_Pet_Events.UNIT_RAGE;

-- UNIT_LEVEL
function Perl_Player_Pet_Events:UNIT_LEVEL()
	if (arg1 == "pet") then
		Perl_Player_Pet_UpdateLevel();
	end
end

Perl_Player_Pet_Events.UNIT_PET_EXPERIENCE = Perl_Player_Pet_Events.UNIT_LEVEL;

-- UNIT_DISPLAYPOWER
function Perl_Player_Pet_Events:UNIT_DISPLAYPOWER()
	if (arg1 == "pet") then
		Perl_Player_Pet_UpdateManaType();
	end
end

-- UNIT_HAPPINESS
-- Happiness events come while you have a pet up, each 2 secs
function Perl_Player_Pet_Events:UNIT_HAPPINESS()
	if (arg1 == "pet") then
		--Perl_Player_Pet_UpdateDisplay();	-- Update all, fixes bug with stables in 1.11
		Perl_Player_Pet_SetHappiness();
		Perl_Player_Pet_UpdateCombat();
	end
end

-- PET_ATTACK_START
function Perl_Player_Pet_Events:PET_ATTACK_START()
	Perl_Player_Pet_UpdateCombat();
end

-- UNIT_COMBAT
function Perl_Player_Pet_Events:UNIT_COMBAT()
	if (arg1 == "pet") then
		if (arg2 == "HEAL") then
			Perl_Player_Pet_CombatFlash(0, true, true);
		elseif (arg4 and arg4 > 0) then
			Perl_Player_Pet_CombatFlash(0, true);
		end
	end
end

-- UNIT_DYNAMIC_FLAGS
function Perl_Player_Pet_Events:UNIT_FACTION()
	if (arg1 == "pet" or arg1 == "player") then
		Perl_Player_Pet_UpdateName();
	        Perl_Player_Pet_UpdateCombat();
	end
end

Perl_Player_Pet_Events.UNIT_FLAGS = Perl_Player_Pet_Events.UNIT_FACTION;
Perl_Player_Pet_Events.UNIT_DYNAMIC_FLAGS = Perl_Player_Pet_Events.UNIT_FACTION;

-- Perl_Player_Pet_Set_Bits
function Perl_Player_Pet_Set_Bits()
	if (Perl_Config.ShowPlayerPetPortrait==0) then
		Perl_Player_Pet_PortraitFrame:Hide();
		Perl_Player_Pet_PortraitFrame:SetWidth(3);
	else
		Perl_Player_Pet_PortraitFrame:Show();
		Perl_Player_Pet_PortraitFrame:SetWidth(50);
		Perl_Player_Pet_UpdatePortrait();
	end

	if (Perl_Config.ShowPlayerPetName == 1) then
		Perl_Player_Pet_NameFrame:Show();
		Perl_Player_Pet_NameFrame:SetHeight(24);
	else
		Perl_Player_Pet_NameFrame:Hide();
		Perl_Player_Pet_NameFrame:SetHeight(3);
	end

	if (Perl_Config.ShowPetLevel == 0) then
        	Perl_Player_Pet_LevelFrame:Hide();
		Perl_Player_Pet_CastClickOverlay:SetPoint("TOPLEFT", Perl_Player_Pet_PortraitFrame);
	else
        	Perl_Player_Pet_LevelFrame:Show();
		Perl_Player_Pet_CastClickOverlay:SetPoint("TOPLEFT", Perl_Player_Pet_LevelFrame);
	end

	if (Perl_Config.ShowPetXP==0) then
		Perl_Player_Pet_StatsFrame_XPBar:Hide();
		Perl_Player_Pet_StatsFrame:SetHeight(34);
	else
		Perl_Player_Pet_StatsFrame_XPBar:Show();
		Perl_Player_Pet_StatsFrame:SetHeight(44);
	end

	Perl_Config.PlayerPetBuffSize = tonumber(Perl_Config.PlayerPetBuffSize) or 20;
	Perl_SetBuffSize("Perl_Player_Pet_", Perl_Config.PlayerPetBuffSize, Perl_Config.PlayerPetBuffSize * 1.3);
end
