local Perl_Target_Events = {}

----------------------
-- Loading Function --
----------------------

function Perl_Target_OnLoad()
	CombatFeedback_Initialize(Perl_TargetHitIndicator, 30);

	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_LEAVING_WORLD");
	this:RegisterEvent("VARIABLES_LOADED");

	this.PlayerFlash = 0;
	this.lastTarget = "";
	this.lastDeathMsg = "";
	Perl_InitFadeFrame(this);

	table.insert(UnitPopupFrames,"Perl_Target_DropDown");

	if (Perl_Target_AssistFrame) then
		-- Since target module is loaded after raid helper, we have to attach this manually
		-- because the target frame did not exist when this frame was created
		Perl_Target_AssistFrame:SetParent(Perl_Target_CastClickOverlay);
		Perl_Target_AssistFrame:ClearAllPoints();
		Perl_Target_AssistFrame:SetPoint("TOPLEFT", Perl_Target_PortraitFrame, "TOPRIGHT", -2, -20);
	end

	--Perl_Target_CastClickOverlay:SetFrameLevel(Perl_Target_StatsFrame:GetFrameLevel() + 2);
end

local function Perl_Target_RegisterSome()
	this:RegisterEvent("UNIT_SPELLMISS");
	this:RegisterEvent("UNIT_FACTION");
	this:RegisterEvent("UNIT_DYNAMIC_FLAGS");
	this:RegisterEvent("UNIT_FLAGS");
	this:RegisterEvent("UNIT_CLASSIFICATION_CHANGED");
	--this:RegisterEvent("PLAYER_PVPLEVEL_CHANGED");
	this:RegisterEvent("PLAYER_TARGET_CHANGED");
	this:RegisterEvent("PLAYER_FLAGS_CHANGED");
	this:RegisterEvent("PLAYER_COMBO_POINTS");
	this:RegisterEvent("UNIT_MODEL_CHANGED");
	--this:RegisterEvent("UNIT_PORTRAIT_UPDATE");
	this:RegisterEvent("PARTY_MEMBER_DISABLE");
	this:RegisterEvent("PARTY_MEMBER_ENABLE");
	this:RegisterEvent("UNIT_AURA");
	this:RegisterEvent("RAID_TARGET_UPDATE");

	this:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH");
	Perl_RegisterBasics();
end
local function Perl_Target_UnregisterSome()
	this:UnregisterEvent("UNIT_SPELLMISS");
	this:UnregisterEvent("UNIT_FACTION");
	this:UnregisterEvent("UNIT_DYNAMIC_FLAGS");
	this:UnregisterEvent("UNIT_FLAGS");
	this:UnregisterEvent("UNIT_CLASSIFICATION_CHANGED");
	--this:UnregisterEvent("PLAYER_PVPLEVEL_CHANGED");
	this:UnregisterEvent("PLAYER_TARGET_CHANGED");
	this:UnregisterEvent("PLAYER_FLAGS_CHANGED");
	this:UnregisterEvent("PLAYER_COMBO_POINTS");
	this:UnregisterEvent("UNIT_MODEL_CHANGED");
	--this:UnregisterEvent("UNIT_PORTRAIT_UPDATE");
	this:UnregisterEvent("PARTY_MEMBER_DISABLE");
	this:UnregisterEvent("PARTY_MEMBER_ENABLE");
	this:UnregisterEvent("UNIT_AURA");
	this:UnregisterEvent("RAID_TARGET_UPDATE");

	this:UnregisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH");
	Perl_UnregisterBasics();
end

--------------------
-- Buff Functions --
--------------------
-- This gets complicated. When we're the target frame, we have to wrap to next line of buffs if there's no mana on the target, or if the mob type frame is shown
-- And if the mob type frame is shown, AND there's no mana, then we may have to use small width on 2 rows, depending on buff size.
-- displayWidth is our current row max
-- displayWidthExtra is the max size after any short rows
-- displayWidthStart remembers the short row size for 2nd pass
function Perl_Targets_BuffPositions(unitTarget, argFramePrefix, ttFrame)
	local above = 0
	if (ttFrame == Perl_Target_Frame) then
		above = Perl_Config.TargetBuffsAbove == 1
	else
		above = Perl_Config.TargetTargetBuffsAbove == 1
	end

	local buffFrame = getglobal(argFramePrefix.."BuffFrame")
	local nameFrame = getglobal(argFramePrefix.."NameFrame")
	local statsFrame = getglobal(argFramePrefix.."StatsFrame")
	if (above) then
		buffFrame:ClearAllPoints()
		buffFrame:SetPoint("BOTTOMLEFT", nameFrame, "TOPLEFT", 2, 0)
	else
		buffFrame:ClearAllPoints()
		buffFrame:SetPoint("TOPLEFT", statsFrame, "BOTTOMLEFT", 2, 0)
	end

	local buffHeight = math.floor(getglobal(argFramePrefix.."BuffFrame_Buff1"):GetHeight() + 1.5);
	local debuffHeight = math.floor(getglobal(argFramePrefix.."BuffFrame_DeBuff1"):GetHeight() + 2.5);
	local displayWidth = math.floor(getglobal(argFramePrefix.."StatsFrame"):GetWidth() - 3.5);
	local displayWidthStart = displayWidth;
	local displayWidthExtra = displayWidth;
	local mana = UnitManaMax(unitTarget);
	local buffRows = 1;
	local debuffRows = 1;

	if (not above) then
		local pframe = getglobal(argFramePrefix.."PortraitFrame");
		if (pframe) then
			displayWidthExtra = displayWidthExtra + math.floor(pframe:GetWidth() - 3.5);
		end

		if (unitTarget == "target") then
			if (Perl_Config.ShowTargetLevel == 1 or Perl_Config.ShowTargetClassIcon == 1) then
				pframe = getglobal(argFramePrefix.."LevelFrame");
				if (pframe) then
					displayWidthExtra = displayWidthExtra + math.floor(pframe:GetWidth() - 3.5);
				end
			end

			if (not Perl_Target_TypeFrame:IsShown() or UnitFactionGroup("player") ~= UnitFactionGroup(unitTarget)) then
				displayWidth = displayWidthExtra;
			end
		else
			displayWidthExtra = displayWidth;
			displayWidthStart = displayWidth;
		end
	end

	local buffPrev = getglobal(argFramePrefix.."BuffFrame_Buff1");
	local buffRowStart = buffPrev;
	local buffsWidth = buffHeight;
	ttFrame.buffData = {{row = 1, buff = buffRowStart}};
	for i = 2,20 do
		local buff = getglobal(argFramePrefix.."BuffFrame_Buff"..i);

		buff:ClearAllPoints();
		if (buffsWidth + buffHeight > displayWidth) then
			if (above) then
				buff:SetPoint("BOTTOMLEFT", buffRowStart, "TOPLEFT", 0, -1);
			else
				buff:SetPoint("TOPLEFT", buffRowStart, "BOTTOMLEFT", 0, -1);
			end
			buffRowStart = buff;

			if (buffRows > 1 or mana > 0 or buffHeight > 26 or not Perl_Target_TypeFrame:IsShown()) then
				displayWidth = displayWidthExtra;
			end

			buffsWidth = 0;
			buffRows = buffRows + 1;
		else
			buff:SetPoint("TOPLEFT", buffPrev, "TOPRIGHT", 1, 0);
		end

		buffsWidth = buffsWidth + buffHeight;
		buffPrev = buff;
		table.insert(ttFrame.buffData, {row = buffRows, buff = buffRowStart});
	end

	if (not above) then
		if ((UnitFactionGroup("player") == UnitFactionGroup(unitTarget))) then
			if (mana == 0) then
				if (buffRows > 1 or buffHeight > 25) then
					displayWidth = displayWidthExtra;
				else
					displayWidth = displayWidthStart;
				end
			else
				displayWidth = displayWidthExtra;
			end
		else
			if (unitTarget == "target" and (mana == 0 or Perl_Target_TypeFrame:IsShown())) then
				displayWidth = displayWidthStart;
			else
				displayWidth = displayWidthExtra;
			end
		end
	end

	buffPrev = getglobal(argFramePrefix.."BuffFrame_DeBuff1");
	buffRowStart = buffPrev;
	buffsWidth = debuffHeight;
	ttFrame.debuffData = {{row = 1, buff = buffRowStart}};
	for i = 2,16 do
		local buff = getglobal(argFramePrefix.."BuffFrame_DeBuff"..i);

		buff:ClearAllPoints();
		if (buffsWidth + debuffHeight > displayWidth) then
			buff:SetPoint("TOPLEFT", buffRowStart, "BOTTOMLEFT", 0, -2);
			buffRowStart = buff;

			if (debuffRows > 1 or mana > 0 or debuffHeight > 26 or not Perl_Target_TypeFrame:IsShown()) then
				displayWidth = displayWidthExtra;
			end

			buffsWidth = 0;
			debuffRows = debuffRows + 1;
		else
			buff:SetPoint("TOPLEFT", buffPrev, "TOPRIGHT", 2, 0);
		end

		buffsWidth = buffsWidth + debuffHeight;
		buffPrev = buff;
		table.insert(ttFrame.debuffData, {row = debuffRows, buff = buffRowStart});
	end
end

function Perl_Targets_BuffUpdate(unitTarget, argFramePrefix, ttFrame)
	if (this.Fading == 1) then
		return;
	end

	if (ttFrame == Perl_Target_Frame) then
		above = Perl_Config.TargetBuffsAbove == 1
	else
		above = Perl_Config.TargetTargetBuffsAbove == 1
	end

	if (UnitName(unitTarget)) then
		this.perlBuffs = 0;
		this.perlDebuffs = 0;

		for buffnum=1,20 do
			local button = getglobal(argFramePrefix.."BuffFrame_Buff"..buffnum);
			if (ttFrame.buffData[buffnum].row > Perl_Config.TargetBuffRows) then
				button:Hide();
			else
				local icon = getglobal(button:GetName().."Icon");
				local buff, count = UnitBuff(unitTarget, buffnum, Perl_Config.TargetCastableBuffs);
				if (buff) then
					this.perlBuffs = this.perlBuffs + 1;
					local buffCount = getglobal(argFramePrefix.."BuffFrame_Buff"..buffnum.."Count");
					icon:SetTexture(buff);
					if (count > 1) then
						buffCount:SetText(count);
						buffCount:Show();
					else
						buffCount:Hide();
					end
					button:Show();
				else
					button:Hide();
				end
			end
		end

		for buffnum=1,16 do
			local button = getglobal(argFramePrefix.."BuffFrame_DeBuff"..buffnum);
			if (ttFrame.debuffData[buffnum].row > Perl_Config.TargetBuffRows) then
				button:Hide();
			else
				local icon = getglobal(button:GetName().."Icon");

				local debuff, debuffApplications, debuffType = UnitDebuff(unitTarget, buffnum, Perl_Config.TargetCurableDebuffs);
				if (debuff) then
					this.perlDebuffs = this.perlDebuffs + 1;
					local debuffCount = getglobal(argFramePrefix.."BuffFrame_DeBuff"..buffnum.."Count");

					icon:SetTexture(debuff);
					if ( debuffApplications > 1 ) then
						debuffCount:SetText(debuffApplications);
						debuffCount:Show();
					else
						debuffCount:Hide();
					end

					local borderColor = DebuffTypeColor[(debuffType or "none")];
					local buffBorder = getglobal(argFramePrefix.."BuffFrame_DeBuff"..buffnum.."Border");
					buffBorder:SetVertexColor(borderColor.r, borderColor.g, borderColor.b);

					button:Show();
				else
					button:Hide();
				end
			end
		end

		local buff1 = getglobal(argFramePrefix.."BuffFrame_Buff1");
		local debuff1 = getglobal(argFramePrefix.."BuffFrame_DeBuff1");
		buff1:ClearAllPoints()
		debuff1:ClearAllPoints()
		if ((UnitFactionGroup("player") == UnitFactionGroup(unitTarget))) then
			if (above) then
				buff1:SetPoint("BOTTOMLEFT", 0, 0);
			else
				buff1:SetPoint("TOPLEFT", 0, 0);
			end
			local mark = this.perlBuffs;
			if (mark < 1) then
				mark = 1;
			end
			if (above) then
				debuff1:SetPoint("BOTTOMLEFT", ttFrame.buffData[mark].buff, "TOPLEFT", 0, -2);
			else
				debuff1:SetPoint("TOPLEFT", ttFrame.buffData[mark].buff, "BOTTOMLEFT", 0, -2);
			end
		else
			if (above) then
				debuff1:SetPoint("BOTTOMLEFT", 0, 0);
			else
				debuff1:SetPoint("TOPLEFT", 0, 0);
			end
			local mark = this.perlDebuffs;
			if (mark < 1) then
				mark = 1;
			end
			if (this.perlDebuffs > 0) then
				if (above) then
					buff1:SetPoint("BOTTOMLEFT", ttFrame.debuffData[mark].buff, "TOPLEFT", 0, -2);
				else
					buff1:SetPoint("TOPLEFT", ttFrame.debuffData[mark].buff, "BOTTOMLEFT", 0, -2);
				end
			end
		end
	end
end

function Perl_Target_SetBuffTooltip (target)
	GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");

	if (target == "targettarget") then
		if (string.find(this:GetName(), "TargetTargetTarget")) then
			target = "targettargettarget";
		end
	end

	GameTooltip:SetUnitBuff(target, this:GetID(), Perl_Config.TargetCastableBuffs);
	Perl_ToolTip_AddBuffDuration(target);
end
function Perl_Target_SetDeBuffTooltip (target)
	GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");

	if (target == "targettarget") then
		if (string.find(this:GetName(), "TargetTargetTarget")) then
			target = "targettargettarget";
		end
	end

	GameTooltip:SetUnitDebuff(target, this:GetID(), Perl_Config.TargetCurableDebuffs);
end

function Perl_Target_PlayerTip()
	Perl_Options_SelectFrame("player");
	Perl_PlayerTip("target");
end

---------------
-- Combo Points
---------------
local function Perl_Target_UpdateCombo()
	local combopoints = GetComboPoints();
	if (Perl_Config.UseCPMeter == 1) then
		Perl_Target_CPFrame:Hide();
		Perl_Target_CPMeter:SetMinMaxValues(0, 5);
		Perl_Target_CPMeter:SetValue(combopoints);
		Perl_Target_CPMeter:Show();
		if (combopoints == 5) then
			Perl_Target_CPMeter:SetStatusBarColor(1, 0, 0, 0.7);
		elseif (combopoints == 4) then
			Perl_Target_CPMeter:SetStatusBarColor(1, 0.5, 0, 0.7);
		elseif (combopoints == 3) then
			Perl_Target_CPMeter:SetStatusBarColor(1, 1, 0, 0.6);
		elseif (combopoints == 2) then
			Perl_Target_CPMeter:SetStatusBarColor(0.5, 1, 0, 0.4);
		elseif (combopoints == 1) then
			Perl_Target_CPMeter:SetStatusBarColor(0, 1, 0, 0.4);
		else
			Perl_Target_CPMeter:Hide();
		end
	elseif (Perl_Config.BlizzardCPMeter == 0) then
		Perl_Target_CPMeter:Hide();
		Perl_Target_CPFrame:Show();
		Perl_Target_CPText:SetText(combopoints);
		if (combopoints == 5) then
			Perl_Target_CPText:SetTextColor(1, 0, 0);
		elseif (combopoints == 4) then
			Perl_Target_CPText:SetTextColor(1, 0.5, 0);
		elseif (combopoints == 3) then
			Perl_Target_CPText:SetTextColor(1, 1, 0);
		elseif (combopoints == 2) then
			Perl_Target_CPText:SetTextColor(0.5, 1, 0);
		elseif (combopoints == 1) then
			Perl_Target_CPText:SetTextColor(0, 1, 0);
		else
			Perl_Target_CPFrame:Hide();
		end
	end
end

---------------------
--Debuffs          --
---------------------
local function Perl_Target_DebuffUpdate()
	if (this.Fading == 1) then
		return;
	end

	if (GetComboPoints()==0) then
		local numDebuffs = 0;
		local _, class = UnitClass("player");

		if (class == "WARRIOR") then
			numDebuffs = Perl_UnitDebuffInformation("Sunder Armor");
		elseif (class == "PRIEST") then
			numDebuffs = Perl_UnitDebuffInformation("Shadow Vulnerability");
		elseif (class == "MAGE") then
			numDebuffs = Perl_UnitDebuffInformation("Fire Vulnerability");

			if (numDebuffs == 0) then
				numDebuffs = Perl_UnitDebuffInformation("Winter's Chill");
			end
		end

		if (Perl_Config.UseCPMeter == 1) then
			Perl_Target_CPFrame:Hide();
			Perl_Target_CPMeter:SetMinMaxValues(0, 5);
			Perl_Target_CPMeter:SetValue(numDebuffs);
			Perl_Target_CPMeter:Show();
			if (numDebuffs == 5) then
				Perl_Target_CPMeter:SetStatusBarColor(1, 0, 0, 0.4);
			elseif (numDebuffs == 4) then
				Perl_Target_CPMeter:SetStatusBarColor(1, 0.5, 0, 0.4);
			elseif (numDebuffs == 3) then
				Perl_Target_CPMeter:SetStatusBarColor(1, 1, 0, 0.4);
			elseif (numDebuffs == 2) then
				Perl_Target_CPMeter:SetStatusBarColor(0.5, 1, 0, 0.4);
			elseif (numDebuffs == 1) then
				Perl_Target_CPMeter:SetStatusBarColor(0, 1, 0, 0.4);
			else
				Perl_Target_CPMeter:Hide();
			end
		else
			Perl_Target_CPMeter:Hide();
			Perl_Target_CPFrame:Show();
			Perl_Target_CPText:SetText(numDebuffs);
			if (cnumDebuffs == 5) then
				Perl_Target_CPText:SetTextColor(1, 0, 0);
			elseif (numDebuffs == 4) then
				Perl_Target_CPText:SetTextColor(1, 0.5, 0);
			elseif (numDebuffs == 3) then
				Perl_Target_CPText:SetTextColor(1, 1, 0);
			elseif (numDebuffs == 2) then
				Perl_Target_CPText:SetTextColor(0.5, 1, 0);
			elseif (numDebuffs == 1) then
				Perl_Target_CPText:SetTextColor(0, 1, 0);
			else
				Perl_Target_CPFrame:Hide();
			end
		end
	else
		Perl_Target_UpdateCombo();
	end
end

-------------------------
-- The Update Functions--
-------------------------
local function Perl_Target_UpdatePVP()
	local targetrankname, targetrank=GetPVPRankInfo(UnitPVPRank("target"), "target");
	if (targetrank and Perl_Config.ShowTargetPVPRank==1 and UnitIsPlayer("target")) then
		Perl_Target_PVPRankIcon:Show();
		if (targetrank==0) then
			Perl_Target_PVPRankIcon:Hide();
		else
			Perl_Target_PVPRankIcon:SetTexture(string.format("Interface\\PVPRankBadges\\PVPRank%02d", targetrank));
		end
	else
		Perl_Target_PVPRankIcon:Hide();
	end
	if (UnitIsPVP("target")) then
		Perl_Target_PVPStatus:SetTexture("Interface\\TargetingFrame\\UI-PVP-"..(UnitFactionGroup("target") or "FFA"));
		Perl_Target_PVPStatus:Show();
	else
		Perl_Target_PVPStatus:Hide();
	end
	Perl_SetUnitNameColor("target", Perl_Target_NameBarText);
end
--
local function Perl_Target_UpdateName()
	local targetname = UnitName("target");

	--if (strlen(targetname) > 20) then
	--	targetname = strsub(targetname, 1, 19).."...";
	--end

	Perl_Target_NameBarText:SetText(targetname);

	local remCount = 1;
	while ((Perl_Target_NameBarText:GetStringWidth() >= (Perl_Target_NameFrame:GetWidth() - 8)) and (string.len(targetname) > remCount)) do
		targetname = string.sub(targetname, 1, string.len(targetname) - remCount).."..";
		remCount = 3;
		Perl_Target_NameBarText:SetText(targetname);
	end

	Perl_Target_UpdatePVP();

	--guildname, guildtitle, guildrank = GetGuildInfo("target");
end

function Perl_Target_UpdatePortrait()
	if (Perl_Config.ShowTargetPortrait == 1) then
		if (UnitName("target")) then
			if (UnitIsVisible("target") and Perl_Config.ShowPlayerPortrait3D == 1) then
				Perl_Target_PortraitFrame_Portrait:Hide();
				Perl_Target_PortraitFrame_Portrait3D:Show();
				PerlSetPortrait3D(Perl_Target_PortraitFrame_Portrait3D, "target");
			else
				Perl_Target_PortraitFrame_Portrait:Show();
				Perl_Target_PortraitFrame_Portrait3D:Hide();
				SetPortraitTexture(Perl_Target_PortraitFrame_Portrait, "target");
			end
		end
	end
end

local function Perl_Target_UpdateLevel()
	local targetlevel = UnitLevel("target");
	Perl_Target_LevelBarText:SetText(targetlevel);
	-- Set Level
	if (Perl_Config.ShowTargetLevel==1) then
		Perl_Target_LevelFrame:Show();
		Perl_Target_LevelFrame:SetWidth(27);
		if (targetlevel < 0) then
			Perl_Target_LevelBarText:SetTextColor(1, 0, 0);
			Perl_Target_LevelFrame:Hide();
		else
			local color = GetDifficultyColor(targetlevel);
			Perl_Target_LevelBarText:SetTextColor(color.r, color.g, color.b);
		end
                if (Perl_Config.ShowTargetElite==0 and (UnitClassification("target") == "elite" or UnitClassification("target") == "worldboss")) then --(UnitIsPlusMob("target"))) then
			Perl_Target_LevelBarText:SetText(targetlevel.."+");
			Perl_Target_LevelFrame:SetWidth(33);
		end
	end
end

local function Perl_Target_UpdateClassification()
	local targetclassification = UnitClassification("target");

	if (UnitIsCivilian("target")) then
		Perl_Target_CreatureBossText:SetText(PERL_TYPE_CIVILIAN);
                Perl_Target_CreatureBossText:SetTextColor(1, 0, 0);
	        Perl_Target_BossFrame:Show();

	elseif (targetclassification == "normal" and UnitPlayerControlled("target")) then
		if (not UnitIsPlayer("target")) then
			Perl_Target_CreatureBossText:SetText("Pet");
	                Perl_Target_CreatureBossText:SetTextColor(1, 1, 1);

		        Perl_Target_BossFrame:Show();
		else
	        	Perl_Target_BossFrame:Hide();
		end

	elseif ((Perl_Config.ShowTargetLevel==1 and Perl_Config.ShowTargetElite==1) or targetclassification=="Rare+" or targetclassification=="Rare") then
		Perl_Target_BossFrame:Show();
		if (targetclassification == "worldboss") then
			Perl_Target_CreatureBossText:SetText("Boss");
	                Perl_Target_CreatureBossText:SetTextColor(1, 0.5, 0.5);

		elseif (targetclassification == "rareelite") then
			Perl_Target_CreatureBossText:SetText("Rare+");
	                Perl_Target_CreatureBossText:SetTextColor(0.8, 0.8, 0.8);

		elseif (targetclassification == "elite") then
			Perl_Target_CreatureBossText:SetText("Elite");
	                Perl_Target_CreatureBossText:SetTextColor(1, 1, 0.5);

		elseif (targetclassification == "rare") then
			Perl_Target_CreatureBossText:SetText("Rare");
	                Perl_Target_CreatureBossText:SetTextColor(0.8, 0.8, 0.8);

		else
			Perl_Target_BossFrame:Hide();
		end
        else
                Perl_Target_BossFrame:Hide();
	end

	Perl_Target_BossFrame:SetWidth(Perl_Target_CreatureBossText:GetStringWidth() + 10);
end

local function Perl_Target_UpdateType()
	local targettype = UnitCreatureType("target");

	if (targettype == PERL_TYPE_NOT_SPECIFIED) then
		targettype = nil;
	end

	if (Perl_Config.ShowTargetMobType == 1) then
		Perl_Target_TypeFrame:Show();
	end

	Perl_Target_TypeFramePlayer:Hide();
	Perl_Target_CreatureText:SetText(targettype);

	if (UnitIsPlayer("target")) then
		local _, PlayerClass = UnitClass("target");
		if (Perl_Config.ShowTargetClassIcon==1) then
			local r, l, t, b = Perl_ClassPos(PlayerClass);
			Perl_Target_ClassTexture:SetTexCoord(r, l, t, b);
			Perl_Target_TypeFramePlayer:Show();
		end
		Perl_Target_TypeFrame:Hide();
	else
		if (targettype and targettype ~= "") then
			Perl_Target_CreatureText:SetTextColor(1, 1, 1);
			Perl_Target_TypeFrame:SetWidth(Perl_Target_CreatureText:GetStringWidth() + 10);
		else
			Perl_Target_TypeFrame:Hide();
		end
	end

	-- If it's too long, we anchor it to left side of portrait instead of right, to avoid it overlapping some buffs
	Perl_Target_TypeFrame:ClearAllPoints();
	if (Perl_Target_TypeFrame:GetWidth() > Perl_Target_PortraitFrame:GetWidth()) then
		Perl_Target_TypeFrame:SetPoint("TOPLEFT", Perl_Target_PortraitFrame, "BOTTOMLEFT", 0, 2);
	else
		Perl_Target_TypeFrame:SetPoint("TOPRIGHT", Perl_Target_PortraitFrame, "BOTTOMRIGHT", 0, 2);
	end
end

local function Perl_Target_UpdateManaType()
	if (this.Fading == 1) then
		return;
	end

	Perl_Target_SetManaType("target", Perl_Target_StatsFrame_ManaBar, Perl_Target_StatsFrame_ManaBarText, Perl_Target_StatsFrame_ManaBarBG, Perl_Target_StatsFrame, Perl_Target_Frame);
end
function Perl_Target_SetManaType(unit, frameManaBar, frameManaBarText, frameManaBarBG, frameStatsFrame, frameFrame)
	local targetmanamax = UnitManaMax(unit);

	if ((targetmanamax == 0) or (unit == "target" and Perl_Config.ShowTargetMana==0) or (unit ~= "target" and Perl_Config.ShowTargetTargetMana == 0)) then
		if (frameManaBar:IsShown()) then
			frameManaBar:Hide();
			frameStatsFrame:SetHeight(28);
			Perl_StatsFrame_Setup(frameStatsFrame);
		end
		return;
	end

	Perl_SetManaBarType(unit, frameManaBar, frameManaBarBG);

	local targetmana = UnitMana(unit);

	if (not frameManaBar:IsShown()) then
		frameManaBarText:Show();
		frameManaBar:Show();
		frameStatsFrame:SetHeight(40);
		Perl_StatsFrame_Setup(frameStatsFrame);
	end
end

local function Perl_Target_UpdateMana()
	if (this.Fading == 1) then
		return;
	end

	Perl_Target_SetMana("target", Perl_Target_StatsFrame_ManaBar, Perl_Target_StatsFrame_HealthBarPercent, Perl_Target_StatsFrame_ManaBarPercent, Perl_Target_StatsFrame_ManaBarText);
end
function Perl_Target_SetMana(unit, frameManabar, frameHealthBarPercent, frameManaBarPercent, frameManaBarText)
	local targetmana = UnitMana(unit);
	local targetmanamax = UnitManaMax(unit);
	if (frameHealthBarPercent:IsVisible()) then
		frameManaBarPercent:Show();
	end

	if (targetmanamax == 1 and targetmana > targetmanamax) then
		targetmanamax = targetmana;
	end

	frameManabar:SetMinMaxValues(0, targetmanamax);
	frameManabar:SetValue(targetmana);

	frameManaBarPercent:SetText(string.format("%d",(100*(targetmana / targetmanamax))+0.5).."%");
	frameManaBarText:SetText(targetmana.."/"..targetmanamax);
end

local function Perl_Target_UpdateHealth()
	if (this.Fading == 1) then
		return;
	end

	Perl_Target_SetHealth("target", Perl_Target_StatsFrame_HealthBar, Perl_Target_StatsFrame_HealthBarBG, Perl_Target_StatsFrame_HealthBarPercent, Perl_Target_StatsFrame_HealthBarText, Perl_Target_StatsFrame_ManaBarPercent, Perl_Config.ShowTargetPercent)

	if (Perl_Config.ShowTargetValues == 1) then
		Perl_Target_StatsFrame_HealthBarText:Show();
		Perl_Target_StatsFrame_ManaBarText:Show();
	else
		Perl_Target_StatsFrame_HealthBarText:Hide();
		Perl_Target_StatsFrame_ManaBarText:Hide();
	end

	if (not UnitIsConnected("target")) then
		Perl_Target_StatsFrame_HealthBarText:SetText(PERL_LOC_OFFLINE);
		Perl_Target_Frame:SetAlpha(Perl_Config.Transparency / 2);
	else
		Perl_Target_Frame:SetAlpha(Perl_Config.Transparency);
	end
end
function Perl_Target_SetHealth(unit, frameHealthBar, frameHealthBarBG, frameHealthBarPercent, frameHealthBarText, frameManaBarPercent, bPercent)
	local targethealth, targethealthmax = UnitHealth(unit), UnitHealthMax(unit);

	frameHealthBar:SetMinMaxValues(0, targethealthmax);
	frameHealthBar:SetValue(targethealth);
	Perl_SetSmoothBarColor(frameHealthBar, frameHealthBarBG, 0.25);
	if (targethealthmax == 100) then
		frameHealthBarPercent:SetText(targethealth.."%");
	else
		frameHealthBarPercent:SetText(string.format("%d",(100*(targethealth / targethealthmax))+0.5).."%");
	end
	if (targethealthmax ~= 100) then
		if (Perl_Config.HealerMode == 1) then
			frameHealthBarText:SetText(-(targethealthmax - targethealth));
		else
			frameHealthBarText:SetText(targethealth.."/"..targethealthmax);
		end
	else
		-- Hide MobInfo-2 frame
		if MobHealthFrame then
			MobHealthFrame:Hide();
		end

		local current, max;
		if (MobHealth3) then
			local found;
			current, max, found = MobHealth3:GetUnitHealth(unit, targethealth, targethealthmax, UnitName(unit), UnitLevel(unit))

			if (not found) then
				current = nil;
			end

		elseif (MobHealthDB) then
			local index;
			if UnitIsPlayer(unit) then
				index = UnitName(unit);
			else
				index = UnitName(unit)..":"..UnitLevel(unit);
			end

			local table = MobHealthDB;
			if (not (table and table[index])) then
				table = MobHealthPlayerDB;
			end
			if (table and table[index]) then
				local s, e;
				local pts;
				local pct;

				if ( type(table[index]) ~= "string" ) then
					frameHealthBarText:SetText(targethealth.."%");
				end
				s, e, pts, pct = string.find(table[index], "^(%d+)/(%d+)$");

				if ( pts and pct ) then
					pts = pts + 0;
					pct = pct + 0;
					if( pct ~= 0 ) then
						pointsPerPct = pts / pct;
					else
						pointsPerPct = 0;
					end
				end
				local currentPct = UnitHealth(unit);
				if ( pointsPerPct > 0 ) then
					current = (currentPct * pointsPerPct) + 0.5;
					max = (100 * pointsPerPct) + 0.5;
				end
			end
		end

		if (current) then
			if (Perl_Config.HealerMode == 1) then
				if (max > 100000) then
					frameHealthBarText:SetText(string.format("%dK", -((max - current) / 1000)));
				else
					frameHealthBarText:SetText(string.format("%d", max - current));
				end
			else
				if (max > 100000) then
					frameHealthBarText:SetText(string.format("%.1fK/%.1fK", current / 1000, max / 1000));
				else
					frameHealthBarText:SetText(string.format("%d/%d", current, max));
				end
			end
		else
			frameHealthBarText:SetText(targethealth.."%");
		end
	end

	if (bPercent == 1) then
		if (UnitIsGhost(unit)) then
			frameManaBarPercent:Hide();
			frameHealthBarPercent:SetText(PERL_LOC_GHOST);
		elseif (UnitIsDead(unit)) then
			frameManaBarPercent:Hide();
			frameHealthBarPercent:SetText(PERL_LOC_DEAD);
		elseif (UnitName(unit) and not UnitIsConnected(unit)) then
			frameManaBarPercent:Hide();
			frameHealthBarPercent:SetText(PERL_LOC_OFFLINE);
		end
	else
		if (UnitIsGhost(unit)) then
			frameHealthBarText:SetText(PERL_LOC_GHOST);
		elseif (UnitIsDead(unit)) then
			frameHealthBarText:SetText(PERL_LOC_DEAD);
		elseif (UnitName(unit) and not UnitIsConnected(unit)) then
			frameHealthBarText:SetText(PERL_LOC_OFFLINE);
		end
	end
end

local function Perl_Target_Update_Control()
        if (UnitIsVisible("target") and UnitIsCharmed("target")) then
		Perl_Target_Warning:Show();
	else
		Perl_Target_Warning:Hide();
	end
end

function Perl_Target_Update_Combat()
        if (UnitAffectingCombat("target")) then
                Perl_Target_ActivityStatus:Show();
        else
                Perl_Target_ActivityStatus:Hide();
        end
end

local function Perl_Target_CombatFlash(elapsed, argNew, argGreen)
	if (Perl_CombatFlashSet (elapsed, this, argNew, argGreen)) then
		Perl_CombatFlashSetFrames(Perl_Target_PortraitFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Target_NameFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Target_LevelFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Target_StatsFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Target_BossFrame, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Target_TypeFramePlayer, this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(Perl_Target_TypeFrame, this.PlayerFlash, this.PlayerFlashGreen);
	end
end

function Perl_Target_Udpate_Range()
	if (Perl_Config.Show30YardSymbol == 0 or CheckInteractDistance("target", 4) or not UnitIsConnected("target")) then
		Perl_Target_RangeStatus:Hide();
	else
		Perl_Target_RangeStatus:Show();
		Perl_Target_RangeStatus:SetAlpha(1);
	end
end

function Perl_Target_UpdateDisplay()
	if (UnitName("target") ~= nil) then
		this.lastTarget = UnitName("target");
		this.lastDeathMsg = "";
		Perl_CancelFade(Perl_Target_Frame);

		Perl_Target_Frame:Show();
		Perl_Target_UpdateName();
		Perl_Target_UpdateLevel();
		Perl_Target_UpdateClassification();
		Perl_Target_UpdateType();
		Perl_Target_UpdatePVP();
		Perl_Target_UpdateCombo();
		Perl_Target_UpdateManaType();
		Perl_Target_UpdateMana();
		Perl_Target_UpdateHealth();
		Perl_Target_Update_Control();
                Perl_Target_Update_Combat();

		-- Work out where all our buffs can fit, we only do this for a fresh target
		Perl_Targets_BuffPositions("target", "Perl_Target_", Perl_Target_Frame);
		Perl_Targets_BuffUpdate("target", "Perl_Target_", Perl_Target_Frame);

		Perl_Target_DebuffUpdate();
		Perl_Target_Events:RAID_TARGET_UPDATE();

		if (not UnitIsConnected("target")) then
			Perl_Target_Frame:SetAlpha(Perl_Config.Transparency / 2);
		else
			Perl_Target_Frame:SetAlpha(Perl_Config.Transparency);
		end

		TargetFrame:Hide();  -- Hide default frame

		if (Perl_Config.BlizzardCPMeter~=1) then
			ComboFrame:Hide();  -- Hide Combo Points
		end
	else
		if (this.lastDeathMsg ~= nil and this.lastTarget ~= nil) then
			if (string.find (this.lastDeathMsg, this.lastTarget)) then
				Perl_Target_UpdateCombo();
				this.lastDeathMsg = "";
				Perl_Target_StatsFrame_HealthBar:SetMinMaxValues(0, 1);
				Perl_Target_StatsFrame_HealthBar:SetValue(0);
				Perl_Target_StatsFrame_ManaBar:SetMinMaxValues(0, 1);
				Perl_Target_StatsFrame_ManaBar:SetValue(0);
				Perl_SetSmoothBarColor (Perl_Target_StatsFrame_HealthBar, Perl_Target_StatsFrame_HealthBarBG, 0.25);

				local oldhp = Perl_Target_StatsFrame_HealthBarText:GetText();
				if (string.find(oldhp, "/")) then
					oldhp = string.gsub(oldhp, "%d+/(%d+)", "0/%1");
					Perl_Target_StatsFrame_HealthBarText:SetText(oldhp);
				else
					Perl_Target_StatsFrame_HealthBarText:SetText("0%");
				end

				local oldmana = Perl_Target_StatsFrame_ManaBarText:GetText();
				if (string.find(oldhp, "/")) then
					oldmana = string.gsub(oldmana, "%d+/(%d+)", "0/%1");
					Perl_Target_StatsFrame_ManaBarText:SetText(oldmana);
				else
					Perl_Target_StatsFrame_ManaBarText:SetText("0%");
				end

				Perl_Target_StatsFrame_HealthBarPercent:SetText(PERL_LOC_DEAD);
			end
		end

		Perl_StartFade(Perl_Target_Frame);
		if (Perl_Target_StatsFrame_HealthBarPercent:GetText() == PERL_LOC_OFFLINE) then
			Perl_Target_Frame.FadeTime = 0.5;
		end
	end
end

--------------------
-- Click Handlers --
--------------------

function Perl_TargetDropDown_OnLoad()
	UIDropDownMenu_Initialize(this, Perl_TargetDropDown_Initialize, "MENU");
end

function Perl_TargetDropDown_Initialize()
	local menu;
	--local name;
	if ( UnitExists("target") and UnitReaction("player", "target") and (((UnitReaction("player", "target") >= 4 and not UnitIsPlayer("target")) and not UnitIsUnit("player", "target")) or (UnitReaction("player", "target") < 4  and not UnitIsPlayer("target"))) ) then
		menu = "RAID_TARGET_ICON";
		--name = RAID_TARGET_ICON;
	elseif ( UnitIsUnit("target", "player") ) then
		menu = "SELF";
	elseif ( UnitIsUnit("target", "pet") ) then
		menu = "PET";
	elseif ( UnitIsPlayer("target") ) then
		if ( UnitInParty("target") or UnitInRaid("target") ) then
			menu = "PARTY";
		else
			menu = "PLAYER";
		end
	elseif ( UnitInParty("target") ) then
		menu = "RAID_TARGET_ICON";
		--name = RAID_TARGET_ICON;
	end
	if ( menu ) then
		UnitPopup_ShowMenu(Perl_Target_DropDown, menu, "target");
	end
end

function Perl_Target_OnClick(button)
	-- If player clicks the target frame while it's fading away. no actual target, but we can get that back:
	if (not UnitName("target")) then
		TargetLastTarget();
	end

	if (Perl_OnClick_Handler(button, "target")) then
		return;
	end

	if (button == "RightButton") then
		ToggleDropDownMenu(1, nil, Perl_Target_DropDown, "Perl_Target_NameFrame", 40, -25);
	end
end

function Perl_Target_OnUpdate()
	if (UnitName("target")) then
		Perl_Target_Udpate_Range();
		Perl_Target_CombatFlash(arg1, false);
	end

	Perl_ProcessFade(Perl_Target_Frame);
end

-------------------
-- Event Handler --
-------------------
function Perl_Target_OnEvent()
	local func = Perl_Target_Events[event];
	if (func) then
		func();
	else
Perl_ShowMessage("EXTRA EVENT");
	end
end

-- PLAYER_ENTERING_WORLD
function Perl_Target_Events:PLAYER_ENTERING_WORLD()
	Perl_Target_RegisterSome();
end

-- VARIABLES_LOADED
function Perl_Target_Events:VARIABLES_LOADED()
	Perl_Target_RegisterSome();
end

-- PLAYER_LEAVING_WORLD
function Perl_Target_Events:PLAYER_LEAVING_WORLD()
	Perl_Target_Frame.Fading = 0;
	Perl_Target_Frame.lastTarget = 0;
	Perl_Target_Frame:Hide();
	Perl_Target_UnregisterSome();
end

-- UNIT_COMBAT
function Perl_Target_Events:UNIT_COMBAT()
	if (arg1 == "target") then
	        Perl_Target_Update_Combat();

		if (Perl_Config.CombatHitIndicator == 1 and Perl_Config.ShowTargetPortrait == 1) then
			CombatFeedback_OnCombatEvent(arg2, arg3, arg4, arg5);
		end

		if (arg2 == "HEAL") then
			Perl_Target_CombatFlash(0, true, true);
		elseif (arg4 and arg4 > 0) then
			Perl_Target_CombatFlash(0, true);
		end
	end
end

-- UNIT_SPELLMISS
function Perl_Target_Events:UNIT_SPELLMISS()
	if (arg1 == "target") then
		if (Perl_Config.CombatHitIndicator == 1 and Perl_Config.ShowTargetPortrait == 1) then
			CombatFeedback_OnSpellMissEvent(arg2);
		end
	end
end

-- PLAYER_TARGET_CHANGED
function Perl_Target_Events:PLAYER_TARGET_CHANGED()
	Perl_Target_Frame.PlayerFlash = 0;
	Perl_Target_CombatFlash(0, false);
	Perl_Target_UpdateDisplay();
	Perl_Target_UpdatePortrait();
	return;
end

-- UNIT_HEALTH, UNIT_MAXHEALTH
function Perl_Target_Events:UNIT_HEALTH()
	if (arg1 == "target") then
		Perl_Target_UpdateHealth();
	end
end
Perl_Target_Events.UNIT_MAXHEALTH = Perl_Target_Events.UNIT_HEALTH;

-- UNIT_DYNAMIC_FLAGS
function Perl_Target_Events:UNIT_DYNAMIC_FLAGS()
	if (arg1 == "target") then
		Perl_Target_UpdateName();
		Perl_Target_Update_Control();
		Perl_Target_Update_Combat();
	end
end

Perl_Target_Events.UNIT_FLAGS = Perl_Target_Events.UNIT_DYNAMIC_FLAGS;

function Perl_Target_Events:RAID_TARGET_UPDATE()
	Perl_Update_RaidIcon("target", Perl_Target_RaidIcon);

	Perl_Target_RaidIcon:ClearAllPoints();
	if (Perl_Config.AlternateRaidIcon == 1) then
		Perl_Target_RaidIcon:SetHeight(16);
		Perl_Target_RaidIcon:SetWidth(16);
		Perl_Target_RaidIcon:SetPoint("CENTER", Perl_Target_NameFrame, "TOPRIGHT", -5, -4);
	else
		Perl_Target_RaidIcon:SetHeight(32);
		Perl_Target_RaidIcon:SetWidth(32);
		Perl_Target_RaidIcon:SetPoint("CENTER", Perl_Target_NameFrame, "CENTER", 0, 0);
	end
end

-- UNIT_MANA, UNIT_MAXMANA, UNIT_RAGE, UNIT_MAXRAGE, UNIT_ENERGY
-- UNIT_MAXENERGY, UNIT_FOCUS, UNIT_MAXFOCUS
function Perl_Target_Events:UNIT_MANA()
	if (arg1 == "target") then
		Perl_Target_UpdateMana();
	end
end
Perl_Target_Events.UNIT_MAXMANA		= Perl_Target_Events.UNIT_MANA;
Perl_Target_Events.UNIT_RAGE		= Perl_Target_Events.UNIT_MANA;
Perl_Target_Events.UNIT_MAXRAGE		= Perl_Target_Events.UNIT_MANA;
Perl_Target_Events.UNIT_ENERGY		= Perl_Target_Events.UNIT_MANA;
Perl_Target_Events.UNIT_MAXENERGY	= Perl_Target_Events.UNIT_MANA;
Perl_Target_Events.UNIT_FOCUS		= Perl_Target_Events.UNIT_MANA;
Perl_Target_Events.UNIT_MAXFOCUS	= Perl_Target_Events.UNIT_MANA;

-- UNIT_DISPLAYPOWER
function Perl_Target_Events:UNIT_DISPLAYPOWER()
	if (arg1 == "target") then
		Perl_Target_UpdateManaType();
		Perl_Target_UpdateMana();
	end
end

-- UNIT_MODEL_CHANGED
function Perl_Target_Events:UNIT_MODEL_CHANGED()
	if (arg1 == "target") then
		Perl_Target_UpdatePortrait();
	end
end
Perl_Target_Events.UNIT_PORTRAIT_UPDATE = Perl_Target_Events.UNIT_MODEL_CHANGED;

-- UNIT_NAME_UPDATE
function Perl_Target_Events:UNIT_NAME_UPDATE()
	if (arg1 == "target") then
		Perl_Target_UpdateName();
	end
end

-- UNIT_LEVEL
function Perl_Target_Events:UNIT_LEVEL()
	if (arg1 == "target") then
		Perl_Target_UpdateLevel()
	end
end

-- UNIT_CLASSIFICATION_CHANGED
function Perl_Target_Events:UNIT_CLASSIFICATION_CHANGED()
	if (arg1 and arg1 == "target") then
		Perl_Target_UpdateClassification();
	end
end

-- PLAYER_COMBO_POINTS
function Perl_Target_Events:PLAYER_COMBO_POINTS()
	Perl_Target_UpdateCombo();
end

-- UNIT_AURA
function Perl_Target_Events:UNIT_AURA()
	if (arg1 == "target") then
		Perl_Targets_BuffUpdate("target", "Perl_Target_", Perl_Target_Frame);
		Perl_Target_DebuffUpdate();
	end
end

-- UNIT_FACTION
function Perl_Target_Events:UNIT_FACTION()
	if (arg1 == "target") then
		Perl_Target_UpdatePVP();
		Perl_Target_Update_Control();
		Perl_Targets_BuffPositions("target", "Perl_Target_", Perl_Target_Frame);
	end
end

-- PLAYER_FLAGS_CHANGED
function Perl_Target_Events:PLAYER_FLAGS_CHANGED()
	Perl_Target_Update_Combat();
	Perl_Target_UpdatePVP();
	Perl_Target_UpdateHealth();

end

-- CHAT_MSG_COMBAT_HOSTILE_DEATH
function Perl_Target_Events:CHAT_MSG_COMBAT_HOSTILE_DEATH()
	if (string.find(arg1, this.lastTarget)) then
		this.lastDeathMsg = arg1;
	end
end

function Perl_Target_Events:PARTY_MEMBER_ENABLE()
	if (UnitInParty("target") or UnitInRaid("target")) then
		Perl_Target_Update_Combat();
		Perl_Target_UpdatePVP();
		Perl_Target_UpdateHealth();
	end
end

Perl_Target_Events.PARTY_MEMBER_DISABLE = Perl_Target_Events.PARTY_MEMBER_ENABLE;

-- Perl_Target_Set_Bits
function Perl_Target_Set_Bits()
	if (Perl_Config.ShowTargetPortrait==0) then
		Perl_Target_PortraitFrame:Hide();
		Perl_Target_PortraitFrame:SetWidth(2);
		Perl_Target_Frame:SetWidth(160);
	else
		Perl_Target_PortraitFrame:Show();
		Perl_Target_PortraitFrame:SetWidth(60);
		Perl_Target_Frame:SetWidth(220);

		Perl_Target_UpdatePortrait();
	end

	if (Perl_Config.ShowTargetPercent==0) then
		Perl_Target_NameFrame:SetWidth(128);
		Perl_Target_StatsFrame:SetWidth(128);
		Perl_Target_StatsFrame_HealthBarPercent:Hide();
		Perl_Target_StatsFrame_ManaBarPercent:Hide();
	else
		Perl_Target_NameFrame:SetWidth(160);
		Perl_Target_StatsFrame:SetWidth(160);
		Perl_Target_StatsFrame_HealthBarPercent:Show();
		Perl_Target_StatsFrame_ManaBarPercent:Show();
	end

	if (Perl_Config.ShowTargetLevel==0) then
		Perl_Target_LevelFrame:Hide();
	else
		Perl_Target_LevelFrame:Show();
	end

	if (Perl_Config.ShowTargetClassIcon==0) then
		Perl_Target_ClassTexture:Hide();
	else
		Perl_Target_ClassTexture:Show();
	end

	if (Perl_Config.ShowTargetPVPRank == 0) then
		Perl_Target_PVPRankIcon:Hide();
	else
		Perl_Target_PVPRankIcon:Show();
	end

	Perl_Config.TargetBuffSize = tonumber(Perl_Config.TargetBuffSize) or 20;
	Perl_SetBuffSize("Perl_Target_", Perl_Config.TargetBuffSize, Perl_Config.TargetBuffSize * 1.2);
end

-- Using the Blizzard Combo Point frame, but we move the buttons around a little
function Perl_Target_Set_BlizzCPFrame()
	ComboFrame:ClearAllPoints();
	ComboPoint1:ClearAllPoints();
	ComboPoint2:ClearAllPoints();
	ComboPoint3:ClearAllPoints();
	ComboPoint4:ClearAllPoints();
	ComboPoint5:ClearAllPoints();

	if (Perl_Config.BlizzardCPMeter == 1) then
		-- This setup will put the buttons along the top of the portrait frame
		ComboFrame:SetPoint("TOPLEFT", Perl_Target_PortraitFrame, "TOPLEFT", -1, 6);

		ComboPoint1:SetPoint("TOPLEFT", 0, -2);
		ComboPoint2:SetPoint("LEFT", ComboPoint1, "RIGHT", 0, 1);
		ComboPoint3:SetPoint("LEFT", ComboPoint2, "RIGHT", 0, 1);
		ComboPoint4:SetPoint("LEFT", ComboPoint3, "RIGHT", 0, -1);
		ComboPoint5:SetPoint("LEFT", ComboPoint4, "RIGHT", 0, -4);

		ComboFrame:SetScale(Perl_Config.Scale_TargetFrame);

		ComboFrame:RegisterEvent("PLAYER_TARGET_CHANGED");
		ComboFrame:RegisterEvent("PLAYER_COMBO_POINTS");
	else
		-- Not using blizzard's, and we'll put the buttons back where they were
--[[		No reason to do this, so commented

		ComboFrame:SetPoint("TOPRIGHT", TargetFrame, "TOPRIGHT", -44, -9);

		ComboPoint1:SetPoint("TOPRIGHT", 0, 0);
		ComboPoint2:SetPoint("TOP", ComboPoint1, "BOTTOM", 7, 4);
		ComboPoint3:SetPoint("TOP", ComboPoint2, "BOTTOM", 5, 2);
		ComboPoint4:SetPoint("TOP", ComboPoint3, "BOTTOM", 2, 1);
		ComboPoint5:SetPoint("TOP", ComboPoint4, "BOTTOM", 0, 1);

		ComboFrame:SetScale(1);
]]
		ComboFrame:Hide();

		ComboFrame:UnregisterEvent("PLAYER_TARGET_CHANGED");
		ComboFrame:UnregisterEvent("PLAYER_COMBO_POINTS");
	end
end
