local SavedRoster = nil
local XswapCount = 0
local XmoveCount = 0
local XPendingRosters = 0
local WaitingForRoster

-- AdminCommands
local function AdminCommands(msg)
	local args = {}
	for value in string.gfind(msg, "[^ ]+") do
		table.insert(args, string.lower(value))
	end

	if (not args[1]) then
        	XPerl_AdminFrame:Show()
	else
		if (not XPerl_AdminCommands[args[1]] or not XPerl_AdminCommands[args[1]](args[2], args[3], args[4])) then
			XPerl_AdminCommands.help()
		end
	end
end

-- DefaultVar
local function DefaultVar(name, value)
	if (XPerl_Admin[name] == nil or (type(value) ~= type(XPerl_Admin[name]))) then
		XPerl_Admin[name] = value;
	end
end

-- Defaults
local function Defaults()

	DefaultVar("AutoHideShow",	1)
	DefaultVar("SavedRosters",	{})

end

-- XPerl_AdminOnLoad
function XPerl_AdminOnLoad()

	XPerl_Admin = {}

	SlashCmdList["XPERLRAIDADMIN"] = AdminCommands
	SLASH_XPERLRAIDADMIN1 = "/rad"

	this:RegisterEvent("VARIABLES_LOADED")
	this:RegisterEvent("RAID_ROSTER_UPDATE")
end

-- XPerl_AdminStartup
local function XPerl_AdminStartup()

	Defaults()
        XPerl_AdminFrame_TitleBar_Pin:SetButtonTex()

end

-- XPerl_Help
local function XPerl_Help()
	XPerl_Message("/rad [save | load name] [auto]");
end

-- XPerl_ToggleAuto
local function XPerl_ToggleAuto()
	if (XPerl_Admin.AutoHideShow == 1) then
		XPerl_Admin.AutoHideShow = 0
	else
		XPerl_Admin.AutoHideShow = 1
	end
end

-- XPerl_SaveRoster
function XPerl_SaveRoster(saveName)

	if (not saveName or saveName == "") then
		local hours, mins = GetGameTime()
		saveName = hours..":"..mins
	end

	local Roster = {}

	for i = 1,GetNumRaidMembers() do
		local name, rank, subgroup, level, class, fileName, zone, online, isDead = GetRaidRosterInfo(i)
		Roster[name] = {group = subgroup, class = fileName}
	end

	if (not XPerl_Admin.SavedRosters) then
		XPerl_Admin.SavedRosters = {}
	end
	XPerl_Admin.SavedRosters[saveName] = Roster

	XPerl_Message("Saved roster called '"..saveName.."'")

	return true
end

local function LoadRoster()

--ChatFrame7:AddMessage("Arranging...");
	local swapCount = 0
	local moveCount = 0
	local CurrentRoster = {}
	local CurrentGroups = {}
	local FreeFloating = {}

	local metaGroups = {
		__index = function(t, k)
			local ret = {}
			for name,entry in CurrentRoster do
				if (entry.group == k) then
					table.insert(ret, name)
				end
			end
			return ret
		end
	}
	setmetatable(CurrentGroups, metaGroups)
	table.setn(CurrentGroups, 8)

	for i = 1,GetNumRaidMembers() do
		local name, rank, subgroup, level, class, fileName, zone, online, isDead = GetRaidRosterInfo(i)
		CurrentRoster[name] = {index = i, group = subgroup, class = fileName}

		if (not SavedRoster[name]) then
			FreeFloating[name] = {group = subgroup, class = fileName} -- Not in the saved roster, so doesn't matter where
		end
	end

	local function Swap(a, b)
--ChatFrame7:AddMessage("Swapping: "..a.." (Grp"..CurrentRoster[a].group..") with "..b.." (Grp"..CurrentRoster[b].group..")")
		SwapRaidSubgroup(CurrentRoster[a].index, CurrentRoster[b].index)
		local save = CurrentRoster[b].group
		CurrentRoster[b].group = CurrentRoster[a].group
		CurrentRoster[b].moved = true
		CurrentRoster[a].group = save
		CurrentRoster[a].moved = true

		if (FreeFloating[a]) then
			FreeFloating[a].group = CurrentRoster[a].group
		end
		if (FreeFloating[b]) then
			FreeFloating[b].group = CurrentRoster[b].group
		end

		swapCount = swapCount + 1
	end

	local function Move(name, target)
--ChatFrame7:AddMessage("Moving: "..name.." (Grp"..CurrentRoster[name].group..") to group "..target);
       		SetRaidSubgroup(CurrentRoster[name].index, target)
       		CurrentRoster[name].group = target
       		CurrentRoster[name].moved = true
       		if (FreeFloating[name]) then
       			FreeFloating[name].group = target
       		end

       		moveCount = moveCount + 1
       	end

	local function GroupCount(grp)
		local count = 0
		for name,entry in CurrentRoster do
			if (entry.group == grp) then
				count = count + 1
			end
		end
		return count
	end

       	for nameSaved, saved in SavedRoster do
		local name = nameSaved;
		local group = saved.group;

       		if (not CurrentRoster[name]) then
			-- Saved player not in raid, so find someone of the same class in the free floater list
			for floaterName,floater in FreeFloating do
				if (floater.class == saved.class) then
					name = floaterName
					FreeFloating[name] = nil
					break
				end
			end
		end

       		if (CurrentRoster[name] and not CurrentRoster[name].moved) then
       			if (CurrentRoster[name].group == group) then
       				CurrentRoster[name].moved = true;		-- They're in right group already
       			elseif (not FreeFloating[name]) then
       				local swapName
       				for name2, saved2 in SavedRoster do
       					if (name ~= name2) then
       						if (CurrentRoster[name] and CurrentRoster[name].group == saved2.group) then
       							if (CurrentRoster[name2] and CurrentRoster[name2].group == group) then
       								swapName = name2
       								break
       							end
       						end

       					end
       				end

       				if (swapName) then
       					Swap(name, swapName)
					--break
       				else
					local done
       					-- Nothing suitable found to swap, see if target group has space
       					if (table.getn(CurrentGroups[group]) < 5) then
       						Move(name, group)
						done = true
       						--break
       					end

       					-- No space in target group, put them anywhere
       					if (not done) then
       						for i = 1,8 do
       							if (CurrentRoster[name].group ~= i and getn(CurrentGroups[i]) < 5) then
       								Move(name, i)
								done = true
       								break
       							end
       						end
       					end

       					if (not done) then
       						local free = foreach(FreeFloating,
       								function(name, group)
       									if (group ~= CurrentRoster[name].group) then
       										return name
       									end
       								end)
       						if (free) then
       							Swap(CurrentRoster[name].index, CurrentRoster[free].index)
							--break
       						else
       							FreeFloating[name] = group
       						end
					--else
					--	break
       					end
       				end
       			end
       		end

       		--if (moveCount > 0) then		-- or swapCount > 0) then
		--	break
		--end
       	end

       	if (moveCount == 0 and swapCount == 0) then
--ChatFrame7:AddMessage("Finished!");
       		XFinishedLoad = true
		SavedRoster = nil
		WaitingForRoster = nil

		XPerl_AdminFrame_Controls:Details()
	else
		WaitingForRoster = true
		XswapCount = XswapCount + swapCount
		XmoveCount = XmoveCount + moveCount
		XPendingRosters = XPendingRosters + swapCount
		XPendingRosters = XPendingRosters + moveCount
	end
--ChatFrame7:AddMessage("Done arranging (swaps: "..swapCount..")  (moves: "..moveCount..")");
end

local function CheckMyRank()
	if (XPerl_Admin.AutoHideShow == 1) then
		if (GetNumRaidMembers() > 0) then
			local me = UnitName("player")
			for i = 1,GetNumRaidMembers() do
				local name, rank = GetRaidRosterInfo(i)
				if (name == me) then
					if (rank > 0) then
						XPerl_AdminFrame:Show()
					else
						XPerl_AdminFrame:Hide()
					end
					break;
				end
			end
		else
			XPerl_AdminFrame:Hide()
		end
	end
end

-- XPerl_AdminOnEvent
function XPerl_AdminOnEvent()
	CheckMyRank();

	if (event == "RAID_ROSTER_UPDATE") then
                XPerl_AdminFrame_Controls:Details()

		if (WaitingForRoster) then
			if (not XFinishedLoad) then
				XPendingRosters = XPendingRosters - 1
				if (XPendingRosters < 1) then
					LoadRoster()
				end
			end
		end

	elseif (event == "VARIABLES_LOADED") then
		this:UnregisterEvent(event)
		XPerl_AdminStartup()
	end
end

-- XPerl_LoadRoster
function XPerl_LoadRoster(loadName)

	if (not XPerl_Admin.SavedRosters) then
		return
	end

	SavedRoster = XPerl_Admin.SavedRosters[loadName]
	if (SavedRoster) then
		XFinishedLoad = false
		WaitingForRoster = nil
		LoadRoster()
	else
		if (not loadName) then
			XPerl_Message("No roster name given")
		else
			XPerl_Message("No roster saved called '"..loadName.."'")
		end
	end

	return true
end

function XPerl_Message(msg)
	DEFAULT_CHAT_FRAME:AddMessage(XPERL_MSG_PREFIX.."- "..msg)
end

XPerl_AdminCommands = {
	save = XPerl_SaveRoster,
	load = XPerl_LoadRoster,
	auto = XPerl_ToggleAuto,
	help = XPerl_Help
}

-- XPerl_Admin_CountDifferences
function XPerl_Admin_CountDifferences(rosterName)
	if (not XPerl_Admin.SavedRosters) then
		return
	end

	local count = 0

	SavedRoster = XPerl_Admin.SavedRosters[rosterName]
	if (SavedRoster) then
		for i = 1,GetNumRaidMembers() do
			local name, rank, subgroup = GetRaidRosterInfo(i)

			if (SavedRoster[name]) then
				if (SavedRoster[name].group ~= subgroup) then
					count = count + 1
				end
			else
				count = count + 1
			end
		end

		return count
	end
end
