local Perl_Raid_Events = {};
local flashID;
local RaidGroupCounts = {0,0,0,0,0,0,0,0,0};
local FrameArray = {};
local perlSetupDone = nil;
local ResArray = {};
local MovingMember = nil
local RosterUpdate = nil
local XReadyCheckTimer;

local CELL_MOTION_SPEED		= 0.8		-- Motion offset mulitplied by this per frame

-- Hold some raid roster information (AFK, DND etc.)
-- Is also stored between sessions to maintain timers and flags
XPerl_Roster = {}
local Last_Roster

-- Uses some variables from FrameXML\RaidFrame.lua:
-- MAX_RAID_MEMBERS = 40;
-- NUM_RAID_GROUPS = 8;
-- MEMBERS_PER_RAID_GROUP = 5;

-- This array will be built up on the fly whenever we pick up a localized class name that we don't already have.
-- The UnitClass function returns both english and localized versions, so it won't matter where the mod is run.
local Perl_ClassNames = {}

----------------------
-- Loading Function --
----------------------

function Perl_Raid_OnLoad()
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_LEAVING_WORLD");
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("RAID_ROSTER_UPDATE");

	this.Array = {};
end

local function Setup1Frame(frame)
	local prefix = frame:GetName();

	if (frame.partyid) then
		if (UnitIsConnected(frame.partyid)) then
			frame:SetAlpha(Perl_Config.Transparency);
		else
			frame:SetAlpha(Perl_Config.Transparency / 2);
		end
	end

	getglobal(prefix.."_NameFrame_NameBarText"):SetAlpha(Perl_Config.TextTransparency);
	getglobal(prefix.."_StatsFrame_HealthBar_HealthBarText"):SetAlpha(Perl_Config.TextTransparency);
	getglobal(prefix.."_StatsFrame_ManaBar_ManaBarText"):SetAlpha(Perl_Config.TextTransparency);

	Perl_SetupFrame(getglobal(prefix.."_StatsFrame"));
	Perl_SetupFrame(getglobal(prefix.."_NameFrame"));

	getglobal(prefix.."_StatsFrame_CastClickOverlay"):UnlockHighlight();
	if (Perl_Config.HighlightSelection == 1) then
		getglobal(prefix.."_StatsFrame_CastClickOverlay"):SetHighlightTexture("Interface\\Addons\\Perl\\images\\Perl_Highlight2x1", "ADD");
	else
		getglobal(prefix.."_StatsFrame_CastClickOverlay"):SetHighlightTexture("");
	end
end

local function GetRaidGroup(num, createIfAbsent)
	local frame = getglobal("Perl_Raid_Grp"..num);

	if (not frame and createIfAbsent) then
		frame = CreateFrame("Frame", "Perl_Raid_Grp"..num, Perl_Raid_Frame, "Perl_Raid_TitleFrameTemplate");
		frame:SetScale(Perl_Config.Scale_Raid);

		frame = getglobal("Perl_Raid_Grp"..num.."_NameFrame_NameBarText")
		frame:SetText("Group "..num);
	end

	return frame;
end

local function GetRaidUnit(num, createIfAbsent)
	local frame = getglobal("Perl_Raid"..num);

	if (not frame and createIfAbsent) then
		frame = CreateFrame("Frame", "Perl_Raid"..num, Perl_Raid_Frame, "Perl_Raid_FrameTemplate");

		frame:SetScale(Perl_Config.Scale_Raid);

		local tex = Perl_GetBarTexture();
		getglobal("Perl_Raid"..num.."_StatsFrame_HealthBar_HealthBarTex"):SetTexture(tex);
		getglobal("Perl_Raid"..num.."_StatsFrame_ManaBar_ManaBarTex"):SetTexture(tex);

		Setup1Frame(frame);
	end

	return frame;
end

local function Perl_Raid_FindID(object)
	local _, _, num = string.find(object:GetName(), "(%d+)")
	return tonumber(num);
end

local function Perl_Raid_FindNum(partyid)
	local _, _, num = string.find(partyid, "(%d+)")
	return tonumber(num);
end

function Perl_Raid_Group_OnLoad()
	local id = Perl_Raid_FindID(this);
	this:SetID(id);

	local x, y;
	if (Perl_Config.RaidPositions) then
		local pos = Perl_Config.RaidPositions[this:GetName()];

		if (pos) then
			if (pos.left or pos.right) then
				x = pos.left
				y = pos.right
			end
		end
	end

	if (not (x and y)) then
		y = 565
		x = (id - 1) * 77
	end

	this:ClearAllPoints();
	this:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y);

	function this:SavePosition()
		Perl_Config.RaidPositions[this:GetName()] = {["top"] = this:GetTop(), ["left"] = this:GetLeft()};
	end

	function this:SaveAllPositions()
		for i = 1,NUM_RAID_GROUPS do
			local frame = GetRaidGroup(i);
			if (frame) then
				Perl_Config.RaidPositions[frame:GetName()] = {["top"] = frame:GetTop(), ["left"] = frame:GetLeft()};
			end
		end
	end
end

function Perl_Raid_Single_OnLoad()
	local id = Perl_Raid_FindID(this);
	this:SetID(id);

	this.partyid = "raid"..this:GetID();
	this.showing = 0;
	this.time = 0;

	getglobal("Perl_Raid"..this:GetID().."_DropDown"):SetID(this:GetID());
	table.insert(UnitPopupFrames, "Perl_Raid"..this:GetID().."_DropDown");

	FrameArray[this.partyid] = this;

	Perl_SetupFrame(getglobal(this:GetName().."_NameFrame"));
	Perl_SetupFrame(getglobal(this:GetName().."_StatsFrame"));
	getglobal(this:GetName().."_StatsFrame_HealthBar_HealthBarText"):SetTextColor(1,1,1,1);
	getglobal(this:GetName().."_StatsFrame_ManaBar_ManaBarText"):SetTextColor(1,1,1,1);

	getglobal(this:GetName().."_StatsFrame_ManaBar_ManaBarText"):SetFont("GameFontNormalSmall", 6);
	getglobal(this:GetName().."_StatsFrame_ManaBar_ManaBarText"):SetTextHeight(8);

	if (Perl_Config.BarTextures==0) then
		getglobal(this:GetName().."_StatsFrame_HealthBar_HealthBarTex"):SetTexture("Interface\\TargetingFrame\\UI-StatusBar");
		getglobal(this:GetName().."_StatsFrame_ManaBar_ManaBarTex"):SetTexture("Interface\\TargetingFrame\\UI-StatusBar");
	else
		getglobal(this:GetName().."_StatsFrame_HealthBar_HealthBarTex"):SetTexture("Interface\\AddOns\\Perl\\Images\\Perl_StatusBar.tga");
		getglobal(this:GetName().."_StatsFrame_ManaBar_ManaBarTex"):SetTexture("Interface\\AddOns\\Perl\\Images\\Perl_StatusBar.tga");
	end
end

local function Perl_Raid_RegisterSome()
	this:RegisterEvent("UNIT_FACTION");
	this:RegisterEvent("UNIT_FLAGS");
	this:RegisterEvent("UNIT_DYNAMIC_FLAGS");
	this:RegisterEvent("UNIT_AURA");
	this:RegisterEvent("UNIT_SPELLMISS");
	Perl_RegisterBasics();
	this:UnregisterEvent("UNIT_LEVEL");

	this:RegisterEvent("CHAT_MSG_ADDON");
	--this:RegisterEvent("CHAT_MSG_CHANNEL");
end
local function Perl_Raid_UnregisterSome()
	this:UnregisterEvent("UNIT_FACTION");
	this:UnregisterEvent("UNIT_FLAGS");
	this:UnregisterEvent("UNIT_DYNAMIC_FLAGS");
	this:UnregisterEvent("UNIT_AURA");
	this:UnregisterEvent("UNIT_SPELLMISS");
	Perl_UnregisterBasics();

	this:UnregisterEvent("CHAT_MSG_ADDON");
	--this:UnregisterEvent("CHAT_MSG_CHANNEL");
end

-- Perl_Raid_UpdateName
local function Perl_Raid_UpdateName(thisFrame)
	if (thisFrame and thisFrame.partyid) then
		local name = UnitName(thisFrame.partyid);

		if (name) then
			local frame = getglobal(thisFrame:GetName().."_NameFrame_NameBarText");
			local frameParent = getglobal(thisFrame:GetName().."_NameFrame");

        	        frame:SetText(name);

			local remCount = 1;
			while ((frame:GetStringWidth() >= (frameParent:GetWidth() - 6)) and (string.len(name) > remCount)) do
				name = string.sub(name, 1, string.len(name) - remCount).."..";
				remCount = 3;
				frame:SetText(name);
			end

			Perl_ColourFriendlyUnit(frame, thisFrame.partyid);
		end
	end
end

-- Perl_CT_AKF
local function Perl_Raid_CheckFlags(partyid)

	local unitName = UnitName(partyid);

	local resser = foreach(ResArray, function(i,name)
						if (name == unitName) then
							return (i);
						end
					  end
			);

	if (resser) then
		-- Verify they're dead..
		if (UnitIsDeadOrGhost(partyid)) then
			return {flag = resser.." ressing", bgcolor = {r = 0, g = 0.5, b = 1}};
		end

		ResArray[resser] = nil;
	end

	local unitInfo = XPerl_Roster[unitName];
	if (unitInfo) then
		if (unitInfo.afk) then
			return {flag = "AFK"};

		elseif (unitInfo.dnd) then
			return {flag = "DND"};

		elseif (unitInfo.ressed) then
			if (UnitIsDead(partyid)) then
				if (unitInfo.ressed == 2) then
					return {flag = PERL_LOC_SS_AVAILABLE};
				else
					return {flag = PERL_LOC_RESURRECTED};
				end
			else
				unitInfo.ressed = nil;
				Perl_Raid_UpdateManaType(FrameArray[partyid], true);
			end
		elseif (unitInfo.ready) then
			if (not XReadyCheckTimer) then
				unitInfo.ready = nil
			else
				if (unitInfo.ready == -1) then
					return {flag = "No Reply"}
				elseif (unitInfo.ready == 0) then
					return {flag = "Not Ready"}
				elseif (unitInfo.ready == 1) then
					return {flag = "Ready!"}
				end
			end
		end
	end
end

-- Perl_Raid_UpdateManaType
function Perl_Raid_UpdateManaType(thisFrame, skipFlags)
	if (thisFrame and thisFrame.partyid) then
		local flags;
		if (not skipFlags) then
			flags = Perl_Raid_CheckFlags(thisFrame.partyid)
		end
		if (not flags) then
			local frame = getglobal(thisFrame:GetName().."_StatsFrame_ManaBar");
			local frameBG = getglobal(thisFrame:GetName().."_StatsFrame_ManaBarBG");

			Perl_SetManaBarType(thisFrame.partyid, frame, frameBG);
		end
	end
end

-- Perl_Raid_ShowFlags
local function Perl_Raid_ShowFlags(thisFrame, flags)
	--thisFrame.flags = flags;
	local f, color, bgcolor;

	if (flags.bgcolor) then
		bgcolor = flags.bgcolor;
	else
		bgcolor = {r = 0.5, g = 0.5, b = 0.5};
	end

	if (flags.color) then
		color = flags.color;
	else
		color = {r = 1, g = 1, b = 1};
	end

	f = getglobal(thisFrame:GetName().."_StatsFrame_HealthBar"); if (f) then f:SetStatusBarColor(bgcolor.r, bgcolor.g, bgcolor.b); end
	f = getglobal(thisFrame:GetName().."_StatsFrame_HealthBarBG"); if (f) then f:SetVertexColor(bgcolor.r * 0.60, bgcolor.g * 0.60, bgcolor.b * 0.60); end
	f = getglobal(thisFrame:GetName().."_StatsFrame_ManaBar"); if (f) then f:SetStatusBarColor(bgcolor.r, bgcolor.g, bgcolor.b); end
	f = getglobal(thisFrame:GetName().."_StatsFrame_ManaBarBG"); if (f) then f:SetVertexColor(bgcolor.r * 0.60, bgcolor.g * 0.60, bgcolor.b * 0.60); end
	f = getglobal(thisFrame:GetName().."_StatsFrame_HealthBar_HealthBarText");
	if (f) then
		f:SetText(flags.flag);
		f:SetTextColor(color.r, color.g, color.b);
	end
end

local function IsFeignDeath(unit)
	for i = 1,20 do
		local buff = UnitBuff(unit, i);

		if (buff) then
			if (strfind(strlower(buff), "feigndeath")) then
				return true;
			end
		else
			break;
		end
	end
end

-- Perl_Raid_UpdateHealth
local function Perl_Raid_UpdateHealth(thisFrame)
	if (thisFrame and thisFrame.partyid) then
		local partyid = thisFrame.partyid;
		local health = UnitHealth(partyid);
		local healthmax = UnitHealthMax(partyid);

		local frameText = getglobal(thisFrame:GetName().."_StatsFrame_HealthBar_HealthBarText");
		local phealthPct = (health * 100.0) / healthmax;
		phealthPct = string.format("%3.0f", phealthPct);

		getglobal(thisFrame:GetName().."_StatsFrame_HealthBar"):SetMinMaxValues(0, healthmax);
		getglobal(thisFrame:GetName().."_StatsFrame_HealthBar"):SetValue(health);

		local name = UnitName(partyid);
		local myRoster = XPerl_Roster[name];
		local feigning;
		if (name and UnitIsConnected(partyid)) then
			local flags = Perl_Raid_CheckFlags(partyid)
			if (flags) then
				Perl_Raid_ShowFlags(thisFrame, flags);

				if (UnitIsDeadOrGhost(partyid)) then
					thisFrame.dead = true
					Perl_Raid_UpdateName(thisFrame);
				end
				return;

			elseif (UnitIsDead(partyid) or myRoster and myRoster.fd) then
				if (myRoster and myRoster.fd) then		--IsFeignDeath(partyid)) then
					feigning = true;

	                		getglobal(thisFrame:GetName().."_StatsFrame_HealthBar"):SetMinMaxValues(0, 1);
	                		getglobal(thisFrame:GetName().."_StatsFrame_HealthBar"):SetValue(0);
					frameText:SetText(PERL_LOC_FEIGNDEATH);
					Perl_SetSmoothBarColor(getglobal(thisFrame:GetName().."_StatsFrame_HealthBar"),getglobal(thisFrame:GetName().."_StatsFrame_HealthBarBG"), 0.25);
				else
					thisFrame.dead = true
					Perl_Raid_ShowFlags(thisFrame, {flag = PERL_LOC_DEAD});
					Perl_Raid_UpdateName(thisFrame);
				end

			elseif (UnitIsGhost(partyid)) then
				thisFrame.dead = true
				Perl_Raid_ShowFlags(thisFrame, {flag = PERL_LOC_GHOST});
				Perl_Raid_UpdateName(thisFrame);

			else
				if (thisFrame.dead or (myRoster and (myRoster.fd or myRoster.ressed))) then
					Perl_Raid_UpdateManaType(thisFrame, true);
				end
				thisFrame.dead = nil

				if (Perl_Config.HealerMode == 1) then
					frameText:SetText(-(healthmax - health));
				else
					frameText:SetText(phealthPct.."%");
				end

				Perl_SetSmoothBarColor(getglobal(thisFrame:GetName().."_StatsFrame_HealthBar"),getglobal(thisFrame:GetName().."_StatsFrame_HealthBarBG"), 0.25);

				if (myRoster) then
					myRoster.resCount = nil
					if (myRoster.ressed) then
						myRoster.ressed = nil
						Perl_Raid_UpdateManaType(thisFrame, true);
					end
				end
			end
		else
			thisFrame.dead = nil
			Perl_Raid_ShowFlags(thisFrame, {flag = PERL_LOC_OFFLINE});

			if (name and myRoster and not myRoster.offline) then
				myRoster.offline = GetTime();
			end
		end

		if (myRoster) then
			if (not feigning) then
				myRoster.fd = nil;
			end
		end
	end
end

-- Perl_Raid_UpdateMana
local function Perl_Raid_UpdateMana(thisFrame)
	if (thisFrame and thisFrame.partyid) then
		local mana = UnitMana(thisFrame.partyid);
		local manamax = UnitManaMax(thisFrame.partyid);

		if (UnitPowerType(thisFrame.partyid) == 0) then
			local pmanaPct = (mana * 100.0) / manamax;
			getglobal(thisFrame:GetName().."_StatsFrame_ManaBar_ManaBarText"):SetText(string.format("%3.0f%%", pmanaPct));
		else
			getglobal(thisFrame:GetName().."_StatsFrame_ManaBar_ManaBarText"):SetText("");
		end

		getglobal(thisFrame:GetName().."_StatsFrame_ManaBar"):SetMinMaxValues(0, manamax);
		getglobal(thisFrame:GetName().."_StatsFrame_ManaBar"):SetValue(mana);
	end
end

-- Perl_Party_CombatFlash
local function Perl_Raid_CombatFlash(elapsed, argNew, argGreen)
	local thisFrame = FrameArray[flashID];
	if (thisFrame and thisFrame.partyid) then
		if (Perl_CombatFlashSet (elapsed, thisFrame, argNew, argGreen)) then
			Perl_CombatFlashSetFrames(getglobal(thisFrame:GetName().."_NameFrame"), thisFrame.PlayerFlash, thisFrame.PlayerFlashGreen);
			Perl_CombatFlashSetFrames(getglobal(thisFrame:GetName().."_StatsFrame"), thisFrame.PlayerFlash, thisFrame.PlayerFlashGreen);
		end
	end
end

-- UpdateUnitByName
local function UpdateUnitByName(name,flagsOnly)
	local id = Perl_GetRaidPosition(name);
	if (id) then
		local frame = FrameArray["raid"..id];
		if (frame) then
			if (flagsOnly) then
				Perl_Raid_UpdateHealth(frame);
			else
				Perl_Raid_UpdateDisplay(frame);
			end
		end
	end
end

-- Perl_Raid_OnUpdate
function Perl_Raid_OnUpdate(elapsed)
	if (RosterUpdate) then
		RosterUpdate = nil

		Perl_Raid_Position();
		if (not UnitName("raid1")) then
			ResArray = {};
			XPerl_Roster = {};
		end
	end

	if (this.PlayerFlash) then
		flashID = this.partyid;
		Perl_Raid_CombatFlash(elapsed, false);
	end

	if (this.motion) then
		Perl_Raid_Motion(this)
	end

	if (XReadyCheckTimer) then
		XReadyCheckTimer = XReadyCheckTimer - elapsed;
		if (XReadyCheckTimer <= 0) then
			XReadyCheckTimer = nil
			for name,unit in XPerl_Roster do
				unit.ready = nil
				UpdateUnitByName(name, true)
			end
		end
	end

	if elapsed then this.time = this.time + elapsed; end
	if this.time >= 0.2 then
		this.time = 0;
		if (Perl_Config.RaidHighlightTanker == 1) then
			local set;
			if (UnitName("target")) then
				if (UnitCanAttack("player", "target")) then
					if (UnitInRaid("targettarget")) then
						set = UnitIsUnit("targettarget", this.partyid);
					end
				elseif (UnitCanAttack("player", "targettarget")) then
					if (UnitInRaid("targettargettarget")) then
						set = UnitIsUnit("targettargettarget", this.partyid);
					end
				end
			end

			local highlightFrame = getglobal(this:GetName().."_StatsFrame_CastClickOverlay");
			if (Perl_Config.HighlightSelection == 0) then
				if (set) then
					highlightFrame:SetHighlightTexture("Interface\\Addons\\Perl\\images\\Perl_Highlight2x1", "ADD");
				else
					highlightFrame:SetHighlightTexture("");
				end
			else
				highlightFrame:SetHighlightTexture("Interface\\Addons\\Perl\\images\\Perl_Highlight2x1", "ADD");
			end

			if (set) then
				highlightFrame:LockHighlight();
			else
				highlightFrame:UnlockHighlight();
			end
		end
	end

	if (MovingMember == this) then
		Perl_Raid_CheckMovingMember()
	end
end

-- CheckRosterBuffs
-- Check buffs exist in roster that we've received, if not then strip them out
local function CheckRosterBuffs(unitName)

	local id = Perl_GetRaidPosition(unitName);
	if (id) then
		id = "raid"..id;
		if (UnitIsConnected(id)) then
			local buffs = XPerl_Roster[unitName].Buffs;
			if (buffs) then
				for buffName,buffTimeLeft in buffs do
					local found;
					for i = 1,20 do
						local buffTex, count = UnitBuff(id, i);
						if (not buffTex) then
							break;
						end

						for texName,texBuff in Perl_CTBuffTextures do
							if ("Interface\\Icons\\"..texBuff[1] == buffTex) then
								if (texName == buffName) then
									found = true;
									break;
								end
							end
						end
					end

					if (not found) then
						buffs[buffName] = nil;
					end
				end
			end
		end
	end
end

-- Perl_Raid_UpdateBuffs
local function Perl_Raid_UpdateBuffs(thisFrame)
	if (thisFrame and thisFrame.partyid) then
		local cursed, color = Perl_CheckDebuffs(thisFrame.partyid, {getglobal(thisFrame:GetName().."_NameFrame"), getglobal(thisFrame:GetName().."_StatsFrame")});

		local frameText = getglobal(thisFrame:GetName().."_NameFrame_NameBarText");
		if (cursed) then
			frameText:SetTextColor(1,0,0);
		else
			Perl_ColourFriendlyUnit(frameText, thisFrame.partyid);
		end

		local buffCount = 0;
		if (Perl_Config.RaidBuffs == 1 or Perl_Config.RaidDebuffs == 1) then
			for buffnum=1,8 do
        	                local button = getglobal(thisFrame:GetName().."_BuffFrame_Buff"..buffnum);
				local icon = getglobal(button:GetName().."Icon");
				local buff, count;

				if (Perl_Config.RaidBuffs == 1) then
					buff, count = UnitBuff(thisFrame.partyid, buffnum, Perl_Config.BuffsCastableCurable);
				else
					buff, count = UnitDebuff(thisFrame.partyid, buffnum, Perl_Config.BuffsCastableCurable);
				end

				if (buff) then
					buffCount = buffCount + 1;

					icon:SetTexture(buff);
					button:Show();
				else
					button:Hide();
				end
			end
		end

		if (Perl_Config.RaidBuffs == 1 and buffCount == 0 and Perl_Config.BuffsCastableCurable == 0) then
			-- Already scanned raid buffs and there are none
			local user = XPerl_Roster[thisFrame.partyid];
			if (user) then
				user.Buffs = nil;
			end
		else
			CheckRosterBuffs(thisFrame.partyid);
		end

		if (buffCount > 0) then
			if (Perl_Config.RaidBuffsRight == 1) then
				getglobal(thisFrame:GetName().."_BuffFrame"):ClearAllPoints();
				getglobal(thisFrame:GetName().."_BuffFrame"):SetPoint("BOTTOMLEFT", thisFrame:GetName().."_StatsFrame", "BOTTOMRIGHT", -2, 2);

				if (Perl_Config.RaidBuffsInside == 1) then
					if (buffCount > 6) then
						getglobal(thisFrame:GetName().."_NameFrame"):SetWidth(60);
					elseif (buffCount > 2) then
						getglobal(thisFrame:GetName().."_NameFrame"):SetWidth(70);
					else
						getglobal(thisFrame:GetName().."_NameFrame"):SetWidth(80);
					end

					if (buffCount > 4) then
						getglobal(thisFrame:GetName().."_StatsFrame"):SetWidth(60);
					else
						getglobal(thisFrame:GetName().."_StatsFrame"):SetWidth(70);
					end
					getglobal(thisFrame:GetName().."_BuffFrame"):Show();
				else
					getglobal(thisFrame:GetName().."_NameFrame"):SetWidth(80);
					getglobal(thisFrame:GetName().."_StatsFrame"):SetWidth(80);
				end

				getglobal(thisFrame:GetName().."_BuffFrame_Buff1"):ClearAllPoints();
				getglobal(thisFrame:GetName().."_BuffFrame_Buff1"):SetPoint("BOTTOMLEFT", 0, 0);
				for i = 2,7 do
					if (i > buffCount) then
						break;
					end
					getglobal(thisFrame:GetName().."_BuffFrame_Buff"..i):ClearAllPoints();

					if (i == 5) then
						if (Perl_Config.RaidBuffsInside == 1) then
							getglobal(thisFrame:GetName().."_BuffFrame_Buff5"):SetPoint("BOTTOMLEFT", 0, 0);
							getglobal(thisFrame:GetName().."_BuffFrame_Buff1"):SetPoint("BOTTOMLEFT", thisFrame:GetName().."_BuffFrame_Buff5", "BOTTOMRIGHT", 0, 0);
						else
							getglobal(thisFrame:GetName().."_BuffFrame_Buff"..i):SetPoint("BOTTOMLEFT", thisFrame:GetName().."_BuffFrame_Buff"..(i-4), "BOTTOMRIGHT", 0, 0);
						end
					else
						getglobal(thisFrame:GetName().."_BuffFrame_Buff"..i):SetPoint("BOTTOMLEFT", thisFrame:GetName().."_BuffFrame_Buff"..(i-1), "TOPLEFT", 0, 0);
					end
				end
			else
				getglobal(thisFrame:GetName().."_NameFrame"):SetWidth(80);
				getglobal(thisFrame:GetName().."_StatsFrame"):SetWidth(80);

				getglobal(thisFrame:GetName().."_BuffFrame"):ClearAllPoints();
				getglobal(thisFrame:GetName().."_BuffFrame"):SetPoint("TOPLEFT", thisFrame:GetName().."_StatsFrame", "BOTTOMLEFT", 0, 2);

				getglobal(thisFrame:GetName().."_BuffFrame_Buff1"):ClearAllPoints();
				getglobal(thisFrame:GetName().."_BuffFrame_Buff1"):SetPoint("TOPLEFT", 0, 0);
				for i = 2,7 do
					if (i > buffCount) then
						break;
					end
					getglobal(thisFrame:GetName().."_BuffFrame_Buff"..i):ClearAllPoints();
					getglobal(thisFrame:GetName().."_BuffFrame_Buff"..i):SetPoint("TOPLEFT", thisFrame:GetName().."_BuffFrame_Buff"..(i-1), "TOPRIGHT", 0, 0);
				end
			end
		else
			getglobal(thisFrame:GetName().."_NameFrame"):SetWidth(80);
			getglobal(thisFrame:GetName().."_StatsFrame"):SetWidth(80);
			getglobal(thisFrame:GetName().."_BuffFrame"):Hide();
		end

		local myRoster = XPerl_Roster[thisFrame.partyid];
		if (myRoster) then
			if (IsFeignDeath(thisFrame.partyid)) then
				if (not myRoster.fd) then
					myRoster.fd = GetTime();
				end
			else
				Perl_Raid_UpdateManaType(thisFrame, true);
				myRoster.fd = nil;
			end
		end
	end
end

local function Perl_Raid_Update_Control(thisFrame)
	if (thisFrame and thisFrame.partyid) then
	        if (UnitIsVisible(thisFrame.partyid) and UnitIsCharmed(thisFrame.partyid)) then
			getglobal(thisFrame:GetName().."_NameFrame_Warning"):Show();
		else
			getglobal(thisFrame:GetName().."_NameFrame_Warning"):Hide();
		end
	end
end


local function Perl_Raid_UpdateCombat(thisFrame)
	if (thisFrame and thisFrame.partyid) then
		local frame = getglobal(thisFrame:GetName().."_NameFrame_ActivityStatus");
		if (not (frame == nil)) then
		        if (UnitName(thisFrame.partyid) and UnitAffectingCombat(thisFrame.partyid)) then
		                frame:Show();
			else
		                frame:Hide();
			end
		end
	end
end

-------------------------
-- The Update Function --
-------------------------
function Perl_Raid_UpdateDisplay(thisFrame)
	if (thisFrame and thisFrame.partyid and thisFrame.showing == 1) then
		if (UnitName(thisFrame.partyid) ~= nil) then
			if (UnitIsConnected(thisFrame.partyid) and MovingMember ~= thisFrame) then
				thisFrame:SetAlpha(Perl_Config.Transparency);
			else
				thisFrame:SetAlpha(Perl_Config.Transparency/2);
				getglobal(thisFrame:GetName().."_StatsFrame_HealthBar_HealthBarText"):SetText("Offine");
			end

			if (Perl_Config.ShowRaidPercents==0) then
				getglobal(thisFrame:GetName().."_StatsFrame_ManaBar_ManaBarText"):Hide();
				getglobal(thisFrame:GetName().."_StatsFrame_HealthBar_HealthBarText"):Hide();
			else
				getglobal(thisFrame:GetName().."_StatsFrame_ManaBar_ManaBarText"):Show();
				getglobal(thisFrame:GetName().."_StatsFrame_HealthBar_HealthBarText"):Show();
			end

			-- Health must be updated after mana, since ctra flag checks are done here.
			Perl_Raid_UpdateManaType(thisFrame);
			Perl_Raid_UpdateMana(thisFrame);
			Perl_Raid_UpdateHealth(thisFrame);		-- <<< -- AFTER MANA -- <<< --
			Perl_Raid_UpdateName(thisFrame);
			Perl_Raid_UpdateCombat(thisFrame);
			Perl_Raid_Update_Control(thisFrame);
			Perl_Raid_UpdateBuffs(thisFrame);
		else
			thisFrame:StopMovingOrSizing();
			thisFrame:Hide();
		end
	else
		thisFrame:StopMovingOrSizing();
		thisFrame:Hide();
	end
end

-------------------
-- Event Handler --
-------------------
function Perl_Raid_OnEvent()
	local func = Perl_Raid_Events[event];
	if (func) then
		func();
	else
Perl_ShowMessage("EXTRA EVENT");
	end
end

function Perl_Raid_Events:VARIABLES_LOADED()
	this:UnregisterEvent("VARIABLES_LOADED");

	Perl_Raid_RegisterSome();

	Perl_ScaleRaid(Perl_Config.Scale_Raid);
	Perl_RaidTitles();
end

-- PLAYER_ENTERING_WORLD
function Perl_Raid_Events:PLAYER_ENTERING_WORLD()
	Perl_Raid_RegisterSome();

	if (not Perl_Raid1 and GetNumRaidMembers() > 0) then
		Perl_Raid_Position();
	end
end

-- PLAYER_LEAVING_WORLD
function Perl_Raid_Events:PLAYER_LEAVING_WORLD()
	Perl_Raid_UnregisterSome();
end

-- RAID_ROSTER_UPDATE
function Perl_Raid_Events:RAID_ROSTER_UPDATE()
	if (MovingMember) then
		Perl_Raid_StopMovingMember()
	end

	if (not Perl_Raid1) then
		-- If no raid yet, just setup anyway
		Perl_Raid_Position();
		if (not UnitName("raid1")) then
			ResArray = {};
			XPerl_Roster = {};
		end
	else
		-- Otherwise, we defer the roster update until the next frame for
		-- cases where we get more than 1 RAID_ROSTER_UPDATE per data packet
		RosterUpdate = true
	end
end

-- UNIT_FLAGS
function Perl_Raid_Events:UNIT_FLAGS()
	Perl_Raid_UpdateCombat(FrameArray[arg1]);
end

Perl_Raid_Events.UNIT_DYNAMIC_FLAGS = Perl_Raid_Events.UNIT_FLAGS;

-- UNIT_FACTION
function Perl_Raid_Events:UNIT_FACTION()
	Perl_Raid_Update_Control(FrameArray[arg1]);
	Perl_Raid_UpdateName(FrameArray[arg1]);
end

-- UNIT_COMBAT
function Perl_Raid_Events:UNIT_COMBAT()
	if (arg2 == "HEAL") then
		flashID = arg1;
		Perl_Raid_CombatFlash(0, true, true);
	elseif (arg4 and arg4 > 0) then
		flashID = arg1;
		Perl_Raid_CombatFlash(0, true);
	end
end

-- UNIT_HEALTH
function Perl_Raid_Events:UNIT_HEALTH()
	Perl_Raid_UpdateHealth(FrameArray[arg1]);
	Perl_Raid_UpdateCombat(FrameArray[arg1]);
end
Perl_Raid_Events.UNIT_MAXHEALTH = Perl_Raid_Events.UNIT_HEALTH;

function Perl_Raid_Events:UNIT_DISPLAYPOWER()
	Perl_Raid_UpdateManaType(FrameArray[arg1]);
	Perl_Raid_UpdateMana(FrameArray[arg1]);
end

-- UNIT_MANA
function Perl_Raid_Events:UNIT_MANA()
	Perl_Raid_UpdateMana(FrameArray[arg1]);
end
Perl_Raid_Events.UNIT_MAXMANA   = Perl_Raid_Events.UNIT_MANA;
Perl_Raid_Events.UNIT_RAGE      = Perl_Raid_Events.UNIT_MANA;
Perl_Raid_Events.UNIT_MAXRAGE   = Perl_Raid_Events.UNIT_MANA;
Perl_Raid_Events.UNIT_ENERGY    = Perl_Raid_Events.UNIT_MANA;
Perl_Raid_Events.UNIT_MAXENERGY = Perl_Raid_Events.UNIT_MANA;
Perl_Raid_Events.UNIT_FOCUS     = Perl_Raid_Events.UNIT_MANA;
Perl_Raid_Events.UNIT_MAXFOCUS  = Perl_Raid_Events.UNIT_MANA;

-- UNIT_NAME_UPDATE
function Perl_Raid_Events:UNIT_NAME_UPDATE()
	Perl_Raid_UpdateName(FrameArray[arg1]);
end

function Perl_Raid_Events:UNIT_AURA()
	Perl_Raid_UpdateBuffs(FrameArray[arg1]);
end

-- SetRes
local function SetResStatus(resserName, resTargetName)

	--frame.beingRessed = true;
	local resEnd;

	if (resTargetName) then
		ResArray[resserName] = resTargetName;
	else
		resEnd = true
		resTargetName = foreach(ResArray,
					function(i,name)
						if (i == resserName) then
							return (name);
						end
					end
				);
		ResArray[resserName] = nil;
	end

	if (resTargetName) then
		if (XPerl_Roster[resTargetName]) then
			if (resEnd) then
				if (not XPerl_Roster[resTargetName].resCount) then
					XPerl_Roster[resTargetName].resCount = 1
				else
					XPerl_Roster[resTargetName].resCount = XPerl_Roster[resTargetName].resCount + 1
				end
			end
			UpdateUnitByName(resTargetName);
		end
	end
end

local function ProcessCTRAMessage(unitName, msg)
	local update = true;
	local myRoster = XPerl_Roster[unitName];

	if (not myRoster) then
--ChatFrame7:AddMessage("No roster for "..(unitName or "nil"));
		return;
	end

	if (strsub(msg, 1, 4) == "RES ") then
		SetResStatus(unitName, strsub(msg, 5));
		return;

	elseif (msg == "RESNO") then
		SetResStatus(unitName);
		return;

      	elseif (msg == "AFK") then
      		myRoster.afk = GetTime()
      		myRoster.dnd = nil

      	elseif (msg == "UNAFK") then
      		myRoster.afk = nil

      	elseif (msg == "DND") then
      		myRoster.dnd = GetTime()
      		myRoster.afk = nil

      	elseif (msg == "UNDND") then
      		myRoster.dnd = nil

      	elseif (msg == "RESSED") then
		myRoster.ressed = 1

	elseif (msg == "CANRES") then
		myRoster.ressed = 2

	elseif (msg == "NORESSED") then
		myRoster.ressed = nil
		myRoster.resCount = nil

	elseif (msg == "CHECKREADY" or msg == "CHECKRLY") then
		XReadyCheckTimer = 30
		if (unitName == UnitName("player") or IsRaidOfficer()) then
			for name,unit in XPerl_Roster do
				unit.ready = -1
				UpdateUnitByName(name, true)
			end
			myRoster.ready = 1
		end
		update = nil

	elseif (msg == "READY" or msg == "NOTREADY" or msg == "YARLY" or msg == "NORLY") then
		if (msg == "READY" or msg == "YARLY") then
			myRoster.ready = 1
		else
			myRoster.ready = 0
		end

		local all_ready = true
		local none_ready = true
		for name,unit in XPerl_Roster do
			if (unit.ready ~= 1) then
				all_ready = false
			elseif (unit.ready == 0) then
				none_ready = false
			end
		end
		if (all_ready or none_ready) then
			XReadyCheckTimer = 0
		end

 	elseif (strsub(msg, 1, 3) == "RN ") then
		-- Buff durations

		local _, _, secsLeft, buffIndex, buffSub = string.find(msg, "^RN ([^%s]+) ([^%s]+) ([^%s]+)$");

		secsLeft = tonumber(secsLeft);
		buffIndex = tonumber(buffIndex);
		buffSub = tonumber(buffSub);
		if (secsLeft and buffIndex and buffSub) then
			local buff = Perl_CTBuffArray[buffIndex];

			if (buff) then
if (buffSub > 1 and type(buff.name) == "string") then
Perl_ShowMessage();
ChatFrame7:AddMessage("string spell '"..buff.name.."' and index > 0   ("..buffSub..")");
elseif (buffSub == 0 and type(buff.name) == "table") then
Perl_ShowMessage();
ChatFrame7:AddMessage("table spell '"..buff.name[1].."' and index == 0");
end

				local buffName;
				-- Shouldn't come with string and sub index > 0, but it does.. Will investigate when and why
				if (buffSub == 0 or type(buff.name) == "string") then
					buffName = buff.name;
				else
					buffName = buff.name[buffSub];
				end

				if (buffName) then
					if (not myRoster.Buffs) then
						myRoster.Buffs = {}
					end
					myRoster.Buffs[buffName] = GetTime() + secsLeft;
					CheckRosterBuffs(unitName);
				end
			end
		end
		update = nil;

	elseif (string.find(msg, "^CD %d+ %d+$")) then
		local _, _, num, cooldown = string.find(msg, "^CD (%d+) (%d+)$");
		if ( num == "1" ) then
			myRoster.Rebirth = GetTime() + tonumber(cooldown)*60;
		elseif ( num == "2" ) then
			myRoster.Reincarnation = GetTime() + tonumber(cooldown)*60;
		elseif ( num == "3" ) then
			myRoster.Soulstone = GetTime() + tonumber(cooldown)*60;
		end
		update = nil;

	elseif (strsub(msg, 1, 2) == "V ") then
		myRoster.version = strsub(msg, 3);
		update = nil;
	else
		update = nil;
	end

	if (update) then
		UpdateUnitByName(unitName);
	end
end

local function Split(msg, char)
	local arr = { }
	while (string.find(msg, char) ) do
		local iStart, iEnd = string.find(msg, char)
		tinsert(arr, strsub(msg, 1, iStart-1))
		msg = strsub(msg, iEnd+1, strlen(msg))
	end
	if ( strlen(msg) > 0 ) then
		tinsert(arr, msg)
	end
	return arr
end

local function ParseCTRA(nick, msg)
	-- Stip out CTRA silly characters
	msg = string.gsub(msg, "%$", "s")
	msg = string.gsub(msg, "", "S")

	if (string.find(msg, "#")) then
		local arr = Split(msg, "#")
		for i,subMsg in arr do
			ProcessCTRAMessage(nick, subMsg)
		end
	else
		ProcessCTRAMessage(nick, msg)
	end
end

-- CHAT_MSG_ADDON
function Perl_Raid_Events:CHAT_MSG_ADDON()		-- For 1.2
	if (arg1 == "CTRA" and arg3 == "RAID") then
		ParseCTRA(arg4, arg2);
	end
end

-- CHAT_MSG_CHANNEL
--function Perl_Raid_Events:CHAT_MSG_CHANNEL()		-- TODO Remove after 1.12
--	if (CT_RA_Channel and arg9 and strlower(arg9) == strlower(CT_RA_Channel) and type(arg1) == "string") then
--		if (XPerl_Roster[arg2]) then
--			if (strsub(arg1, 1, 5) == "KLHTM") then
--				return;
--			end
--
--			local msg = arg1
--			if ( strsub(msg, strlen(msg)-7) == " ...hic!") then
--				msg = strsub(msg, 1, strlen(msg)-8)
--			end
--
--			ParseCTRA(arg2, msg);
--		end
--	end
--end

-- SetRaidRoster
local function SetRaidRoster()

	local NewRoster = {}

	Last_Roster = XPerl_Roster
	-- Instead, go round and save all the current group/row positions

	for i = 1,GetNumRaidMembers() do
		local name = UnitName("raid"..i)

		if (XPerl_Roster[name]) then
			NewRoster[name] = XPerl_Roster[name]
		else
			NewRoster[name] = {}
		end

		if (Last_Roster[name]) then
			local frame = FrameArray["raid"..i]
			if (frame and frame.lastPos) then
				Last_Roster[name].lastPos = {group = frame.lastPos.group, row = frame.lastPos.row}
			end
		end
	end

	XPerl_Roster = NewRoster
end

--------------------
-- Raid Functions --
--------------------
function Perl_Raid_Position()
	if (not perlSetupDone) then
		perlSetupDone = true;
		Perl_SetupRaidFrames();
	end

	SetRaidRoster();
	RaidGroupCounts = {0,0,0,0,0,0,0,0,0}

	local p=1;
	if (Perl_Config.SortRaidByClass==1) then
		Perl_Raid_Classes();
	else
		Perl_Raid_Groups();
	end

	if Perl_Config.ShowRaid == 0 then
		Perl_Raid_Frame:Hide();
	else
		Perl_Raid_Frame:Show();

		for num=1,MAX_RAID_MEMBERS do
			local frame = getglobal("Perl_Raid"..num);

			if (frame) then
				if (frame.showing == 1) then
					frame:Show();
					Perl_Raid_UpdateDisplay(frame);
				else
					frame:Hide();
				end
			end
		end
	end

	Perl_RaidTitles();

	Last_Roster = nil
	for i,unit in XPerl_Roster do
		unit.lastPos = nil
	end
end

-- MoveCell
local function MoveCell(cell, grp)

	local parentGroup = getglobal("Perl_Raid_Grp"..grp)
	local noMotion;

	if (cell.lastPos) then
		if (cell.lastPos.group == grp and cell.lastPos.row == RaidGroupCounts[grp] + 1) then
			noMotion = true
		else
			local oldCell = Last_Roster[UnitName(cell.partyid)];
			if (oldCell and oldCell.lastPos) then
				if (oldCell.lastPos.group == grp and cell.lastPos.row == RaidGroupCounts[grp] + 1) then
					noMotion = true
				end
			end
		end
	end

	cell.lastPos = {group = grp, row = RaidGroupCounts[grp] + 1}

	if (cell.motion or (not noMotion and cell.showing == 1 and cell:IsShown() and Perl_Config.RaidMotion == 1)) then
		local t = 2-(RaidGroupCounts[grp] * Perl_Config.RaidVerticalSpacing);
		cell.motion = {
			targetTop = t,
			parent = parentGroup,
			currentLeft = cell:GetLeft() - parentGroup:GetLeft(),
			currentTop = cell:GetTop() - parentGroup:GetBottom() - t
		}
	else
		cell:SetPoint("TOPLEFT", parentGroup, "BOTTOMLEFT", 0,(2-(RaidGroupCounts[grp] * Perl_Config.RaidVerticalSpacing)));
	end
end

-- Perl_Raid_Motion
function Perl_Raid_Motion(cell)

	local motion = cell.motion;

	motion.currentLeft = motion.currentLeft * CELL_MOTION_SPEED;
	if (motion.currentLeft > -1 and motion.currentLeft < 1) then
		motion.currentLeft = 0
	end

	motion.currentTop = motion.currentTop * CELL_MOTION_SPEED;
	if (motion.currentTop > -1 and motion.currentTop < 1) then
		motion.currentTop = 0
	end

	cell:SetPoint("TOPLEFT", motion.parent, "BOTTOMLEFT", motion.currentLeft, motion.currentTop + motion.targetTop);

	if (motion.currentLeft == 0 and motion.currentTop == 0) then
		cell.motion = nil
	end
end

-- Perl_Raid_Classes
function Perl_Raid_Classes()
	local groups = {WARRIOR = 1, MAGE = 2, PRIEST = 3, WARLOCK = 4, DRUID = 5, ROGUE = 6, HUNTER = 7, PALADIN = 8, SHAMAN = 9}

	for num=1,MAX_RAID_MEMBERS do
		local localClass, class = UnitClass("raid"..num);
		if (class) then
			if (not Perl_ClassNames[class]) then
				Perl_ClassNames[class] = localClass;
			end

			local frame = GetRaidUnit(num, true);

			local subgroup = groups[class];
			if (subgroup) then
				if (subgroup == 9) then		-- Burning Crusade - REMOVE THIS
					subgroup = 8;
				end

				GetRaidGroup(subgroup, true);
				frame.showing = Perl_Config["ShowGroup"..subgroup];
				MoveCell(frame, subgroup);
				RaidGroupCounts[subgroup] = RaidGroupCounts[subgroup] + 1;
			end

		else
			local frame = GetRaidUnit(num);
			if (frame) then
		    		frame.showing = 0;
			end
		end
	end

	for name,index in groups do
		local frame = getglobal("Perl_Raid_Grp"..index.."_NameFrame_NameBarText");
		if (frame) then
			if (index == 8) then	-- Burning Crusade - REMOVE THIS
				if (UnitFactionGroup("player") == "Horde") then
					frame:SetText(Perl_ClassNames["SHAMAN"]);
				else
					frame:SetText(Perl_ClassNames["PALADIN"]);
				end
				break;
			end

			frame:SetText(Perl_ClassNames[name]);
		end
	end
end

-- Perl_Raid_Groups
function Perl_Raid_Groups()
	for num=1,MAX_RAID_MEMBERS do
		local name, rank, subgroup= GetRaidRosterInfo(num);
		if (name and subgroup) then
			local frame = GetRaidUnit(num, true);

			if (frame) then
				GetRaidGroup(subgroup, true);
				local localClass, class = UnitClass("raid"..num);

				if (class and not Perl_ClassNames[class]) then
					Perl_ClassNames[class] = localClass;
				end

				if (name and subgroup and subgroup >= 1 and subgroup <= NUM_RAID_GROUPS) then
					frame.showing = Perl_Config["ShowGroup"..subgroup];
					MoveCell(frame, subgroup);
					RaidGroupCounts[subgroup] = RaidGroupCounts[subgroup] + 1;
				else
				    	frame.showing = 0;
				end
			end
		else
			local frame = GetRaidUnit(num);
			if (frame) then
				frame.showing = 0;
			end
		end
	end

	for index = 1,NUM_RAID_GROUPS do
		local frame = getglobal("Perl_Raid_Grp"..index.."_NameFrame_NameBarText");
		if (frame) then
			frame:SetText("Group "..index);
		end
	end
end

--------------------
-- Click Handlers --
--------------------
function Perl_Raid_OnClick(button)
	local id = Perl_Raid_FindID(this);
	local unitid = "raid"..id;

	if (Perl_OnClick_Handler(button, unitid)) then
		return;
	end

	if (button == "RightButton") then
		ToggleDropDownMenu(1, nil, getglobal("Perl_Raid"..id.."_DropDown"), "Perl_Raid"..id.."_StatsFrame_CastClickOverlay", 0, 0);
	end
end

function Perl_ScaleRaid(num)
	for frame=1,NUM_RAID_GROUPS do
		local f = GetRaidGroup(frame);
		if (f) then
			f:SetScale(num);
		end
	end
	for frame=1,MAX_RAID_MEMBERS do
		local f = GetRaidUnit(frame);
		if (f) then
			f:SetScale(num);
		end
	end
end

function Perl_RaidTitles()
	for i = 1,NUM_RAID_GROUPS do
		local frame = GetRaidGroup(i);
		if (frame) then
			frame:SetAlpha(Perl_Config.Transparency);
			if (MovingMember or perl_locked == 0 or (Perl_Config.ShowRaidTitles == 1 and RaidGroupCounts[i] > 0 and Perl_Config["ShowGroup"..i] == 1)) then
				frame:Show();
			else
				frame:Hide();
			end
		end
	end
end

function Perl_RaidDropDown_Initialize()
	if ( not UIDROPDOWNMENU_OPEN_MENU or not getglobal(UIDROPDOWNMENU_OPEN_MENU).perlRaidDropDown ) then
		return;
	end

	local id = this:GetID();
	if (id == 0) then
		id = Perl_Raid_FindID(this);
		this:SetID(id);
	end

	local dropdown;

	if ( UIDROPDOWNMENU_OPEN_MENU ) then
		dropdown = getglobal(UIDROPDOWNMENU_OPEN_MENU);
	else
		dropdown = this;
	end

	if (CT_RATab_newRaidFrameDropDown_Initialize and UIDROPDOWNMENU_OPEN_MENU) then
		this.unit = "raid"..id;
		this.name = UnitName(this.unit);
		this.id = id;
		CT_RATab_newRaidFrameDropDown_Initialize();
	else
		UnitPopup_ShowMenu(dropdown, "RAID", "raid"..id, UnitName("raid"..id), id);
	end
end

function Perl_RaidDropDown_OnLoad()
	this.perlRaidDropDown = true;
	UIDropDownMenu_Initialize(this, Perl_RaidDropDown_Initialize, "MENU");
end

-- Moving stuff
function Perl_Raid_GetGap()
	if (Perl_Raid_Grp1 and Perl_Raid_Grp2) then
		return math.floor((math.floor((Perl_Raid_Grp2:GetLeft() - Perl_Raid_Grp1:GetRight() + 0.01) * 100) / 100) + 4);
	end

	return (0);
end

function Perl_Raid_SetGap(newGap)
	if (not Perl_Raid_Grp1) then
		return;
	end

	local top = Perl_Raid_Grp1:GetTop();
	if (type(newGap) == "number") then
		local left = Perl_Raid_Grp1:GetLeft();
		Perl_Raid_Grp1:ClearAllPoints();
		Perl_Raid_Grp1:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", left, top);

		for i = 2,NUM_RAID_GROUPS do
			local framePrev = GetRaidGroup(i - 1, true);
			local frame = GetRaidGroup(i, true);

			if (framePrev and frame) then
				local right = framePrev:GetRight();

				frame:ClearAllPoints();
				frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", right + newGap - 4, top);

				frame:StartMoving();
				frame:StopMovingOrSizing();
			end
		end

		Perl_Raid_Grp1:SaveAllPositions();
	end
end

function Perl_Raid_AlignTop()
	if (not Perl_Raid_Grp1) then
		return;
	end

	local top = Perl_Raid_Grp1:GetTop();

	for i = 1,NUM_RAID_GROUPS do
		local frame = GetRaidGroup(i, true);

		if (frame) then
			local left = frame:GetLeft();

			frame:ClearAllPoints();
			frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", left, top);

			frame:StartMoving();
			frame:StopMovingOrSizing();
		end
	end

	Perl_Raid_Grp1:SaveAllPositions();
end

-- General setup
function Perl_SetupRaidFrames()

	for num = 1,MAX_RAID_MEMBERS do
		local frame = GetRaidUnit(num);

		if (frame) then
			Setup1Frame(frame);
		end
       	end

	Perl_SetRaidBarTextures();
end

function Perl_Raid_SetBuffTooltip ()
	local partyid = "raid"..this:GetParent():GetParent():GetID();
	GameTooltip:SetOwner(this,"ANCHOR_BOTTOMRIGHT",30,0);

	if (Perl_Config.RaidBuffs == 1) then
		GameTooltip:SetUnitBuff(partyid, this:GetID(), tostring(Perl_Config.BuffsCastableCurable));
		Perl_ToolTip_AddBuffDuration(partyid);
	else
		GameTooltip:SetUnitDebuff(partyid, this:GetID(), tostring(Perl_Config.BuffsCastableCurable));
	end
end

------- Perl_ToggleRaidBuffs -------
-- Raid Buff Key Binding function --
function Perl_ToggleRaidBuffs(castable)

	if (castable) then
		if (Perl_Config.BuffsCastableCurable == 1) then
			Perl_Config.BuffsCastableCurable = 0;
			Perl_Notice(PERL_KEY_NOTICE_RAID_BUFFANY);
		else
			Perl_Config.BuffsCastableCurable = 1;
			Perl_Notice(PERL_KEY_NOTICE_RAID_BUFFCURECAST);
		end
	else
		if (Perl_Config.RaidBuffs == 1) then
			Perl_Config.RaidBuffs = 0;
			Perl_Config.RaidDebuffs = 1;
			Perl_Notice(PERL_KEY_NOTICE_RAID_DEBUFFS);

		elseif (Perl_Config.RaidDebuffs == 1) then
			Perl_Config.RaidDebuffs = 0;
			Perl_Config.RaidBuffs = 0;
			Perl_Notice(PERL_KEY_NOTICE_RAID_NOBUFFS);

		else
			Perl_Config.RaidBuffs = 1;
			Perl_Config.RaidDebuffs = 0;
			Perl_Notice(PERL_KEY_NOTICE_RAID_BUFFS);
		end
	end

	for i = 1,GetNumRaidMembers() do
		local frame = GetRaidUnit(i);
		if (frame) then
			Perl_Raid_UpdateDisplay(frame);
		end
	end
end

function Perl_SetRaidBarTextures()
	local tex = Perl_GetBarTexture();
	for num = 1,MAX_RAID_MEMBERS do
		if (GetRaidUnit(num)) then
			getglobal("Perl_Raid"..num.."_StatsFrame_HealthBar_HealthBarTex"):SetTexture(tex);
			getglobal("Perl_Raid"..num.."_StatsFrame_ManaBar_ManaBarTex"):SetTexture(tex);
		end
	end
end

-- Perl_RaidTipExtra
function Perl_RaidTipExtra(unitid)

	--if (not UnitName("raid1")) then
	--	return;
	--end

	if (UnitInRaid(unitid)) then
		local unitName = UnitName(unitid);
		local zone;
		local name, rank, subgroup, level, class, fileName, zone, online, isDead;

		for i = 1,GetNumRaidMembers() do
			name, rank, subgroup, level, class, fileName, zone, online, isDead = GetRaidRosterInfo(i);
			if (name == unitName) then
				break;
			end
			zone = "";
		end

		local stats = XPerl_Roster[unitName];
		if (stats) then
			local t = GetTime();

			if (stats.version) then
				GameTooltip:AddLine("CTRA " .. stats.version, 1, 1, 1);
			else
				GameTooltip:AddLine("No CTRA Found", 0.7, 0.7, 0.7);
			end

			if (stats.offline and UnitIsConnected(unitid)) then
				stats.offline = nil;
			end

			if (stats.offline) then
				GameTooltip:AddLine("Offline for " .. SecondsToTime(t - stats.offline));

			elseif (stats.afk) then
				GameTooltip:AddLine("AFK for " .. SecondsToTime(t - stats.afk));

			elseif (stats.dnd) then
				GameTooltip:AddLine("DND for " .. SecondsToTime(t - stats.dnd));

			elseif (stats.fd) then
				if (not UnitIsDead(unitid)) then
					stats.fd = nil;
				else
					local x = stats.fd + 360 - t;
					if (x > 0) then
						GameTooltip:AddLine("Dying in " .. SecondsToTime(x));
					end
				end
			end

			if (stats.Rebirth) then
				if (stats.Rebirth - t > 0) then
					GameTooltip:AddLine("Rebirth up in: " .. SecondsToTime(stats.Rebirth - t));
				else
					stats.Rebirth = nil;
				end

			elseif (stats.Reincarnation) then
				if (stats.Reincarnation - t > 0) then
					GameTooltip:AddLine("Ankh up in: " .. SecondsToTime(stats.Reincarnation - t));
				else
					stats.Reincarnation = nil;
				end

			elseif (stats.Soulstone) then
				if (stats.Soulstone - t > 0) then
					GameTooltip:AddLine("Soulstone up in: " .. SecondsToTime(stats.Soulstone - t));
				else
					stats.Soulstone = nil;
				end
			end

			if (UnitIsDeadOrGhost(unitid) and stats.resCount) then
				GameTooltip:AddLine(PERL_LOC_RESURRECTED.." x"..stats.resCount);
			end
		end

		GameTooltip:Show();
	end
end

local matches = {
	{"Holy_WordFortitude", "Holy_PrayerOfFortitude"},
	{"Holy_MagicalSentry", "Holy_ArcaneIntellect"},
	{"Shadow_AntiShadow", "Holy_PrayerofShadowProtection"},
	{"Holy_DivineSpirit", "Holy_PrayerofSpirit"},
	{"Holy_FistOfJustice", "Holy_GreaterBlessingofKings"},
	{"Holy_SealOfWisdom", "Holy_GreaterBlessingofWisdom"},
	{"Magic_MageArmor", "Magic_GreaterBlessingofKings"},
	{"Holy_SealOfSalvation", "Holy_GreaterBlessingofSalvation"},
	{"Holy_PrayerOfHealing02", "Holy_GreaterBlessingofLight"},
	{"Nature_LightningShield", "Holy_GreaterBlessingofSanctuary"}
};

local function GetNamesWithoutBuff(tex, with)
	local count = 0;
	local names;

	for i = 1,GetNumRaidMembers() do
		local name = UnitName("raid"..i);
		if (name and UnitIsConnected("raid"..i) and not UnitIsDeadOrGhost("raid"..i)) then
			local hasBuff = false;
			for num = 1,25 do
				local buffTexture = UnitBuff("raid"..i, num);
				if (not buffTexture) then
					break;
				end

				if (buffTexture == tex) then
					hasBuff = true;
					break;
				else
					for dups,pair in matches do
						if (tex == "Interface\\Icons\\Spell_"..pair[1] or tex == "Interface\\Icons\\Spell_"..pair[2]) then
							if (buffTexture == "Interface\\Icons\\Spell_"..pair[1] or buffTexture == "Interface\\Icons\\Spell_"..pair[2]) then
								hasBuff = true;
							end
						end
					end
					if (hasBuff) then
						break;
					end
				end
			end

			if ((with and hasBuff) or (not with and not hasBuff)) then
				count = count + 1;
				local _, class = UnitClass("raid"..i);
				if (class) then
					names = (names or "")..PerlColourTable[class]..name.." ";
				else
					names = (names or "")..name.." ";
				end
			end
		end
	end

	return names, count;
end

function Perl_Raid_AddBuffDuration(partyid)
	if (UnitName("raid1") and GameTooltip:IsOwned(this)) then
		if (partyid ~= "player") then
			local stats = XPerl_Roster[UnitName(partyid)];

			if (stats and stats.Buffs) then
				local buffName = getglobal("GameTooltipTextLeft1"):GetText();
				if (buffName) then
					local buff = stats.Buffs[buffName];
					if (buff) then
						if (buff > GetTime()) then
							GameTooltip:AddLine(SecondsToTime(buff - GetTime()).."remaining", 1, 0.82, 0);
						end
					end
				end
			end
		end

		if (Perl_Config.BuffTooltipHelper == 1 and partyid and UnitInRaid(partyid)) then
			local icon = getglobal(this:GetName().."Icon");
			if (icon) then
				local tex = icon:GetTexture();
				if (tex) then
					local names, count = GetNamesWithoutBuff(tex, IsAltKeyDown());
					if (names) then
						if (IsAltKeyDown()) then
							GameTooltip:AddLine("With buff: ("..count..")", 0.3, 1, 0.2);
						else
							GameTooltip:AddLine("Without buff: ("..count..")", 1, 0.3, 0.1);
						end
						GameTooltip:AddLine(names, 0.8, 0.5, 0.4, 1);
					end
				end
			end
		end

		GameTooltip:Show();
	end
end

-- Perl_Raid_StartMovingMember
function Perl_Raid_StartMovingMember()

	if (not MovingMember) then
		MovingMember = this:GetParent()
        	MovingMember:StartMoving()
		MovingMember:SetAlpha(Perl_Config.Transparency / 2)
		Perl_RaidTitles()
	end
end

-- Perl_Raid_StopMovingMember
function Perl_Raid_StopMovingMember(commit)
	if (MovingMember) then
		local good
		MovingMember:StopMovingOrSizing()

		if (commit) then
			local group,subpos = Perl_Raid_CheckMovingMember()

--ChatFrame7:AddMessage("group "..(group or "nil").."   row "..(subpos or "nil"))

			if (group) then
				if (group ~= MovingMember.lastPos.group) then
					local unit;
					for i,cell in FrameArray do
						if (cell.lastPos.group == group and cell.lastPos.row == subpos) then
							unit = cell;
							break;
						end
					end

					local index = Perl_Raid_FindNum(MovingMember.partyid);

--ChatFrame7:AddMessage("  index "..(index or "nil"))

					if (unit) then
						local targetIndex = Perl_Raid_FindNum(unit.partyid);
--ChatFrame7:AddMessage("  targetIndex "..(targetIndex or "nil"))
						SwapRaidSubgroup(index, targetIndex);
					else
       						SetRaidSubgroup(index, group);
					end
					good = true
				else
--ChatFrame7:AddMessage("Cancelled, same as my current group")
				end
			else
--ChatFrame7:AddMessage("Cancelled, no group")
			end
		end

		for i,cell in FrameArray do
			getglobal(cell:GetName().."_StatsFrame_CastClickOverlay"):UnlockHighlight()
		end

		if (not good) then
			local parentGroup = GetRaidGroup(MovingMember.lastPos.group)
			MovingMember:SetPoint("TOPLEFT", parentGroup, "BOTTOMLEFT", 0,(2-((MovingMember.lastPos.row - 1) * Perl_Config.RaidVerticalSpacing)));
		end

		if (UnitIsConnected(MovingMember.partyid)) then
			MovingMember:SetAlpha(Perl_Config.Transparency);
		else
			MovingMember:SetAlpha(Perl_Config.Transparency / 2);
		end

		MovingMember = nil
		Perl_RaidTitles()

		Perl_Raid_Mover:Hide()
	end
end

-- Perl_Raid_CheckMovingMember
function Perl_Raid_CheckMovingMember()

	local x = MovingMember:GetLeft() + ((MovingMember:GetRight() - MovingMember:GetLeft()) / 2)
	local y = MovingMember:GetBottom() + ((MovingMember:GetTop() - MovingMember:GetBottom()) / 2)

	Perl_Raid_Mover:ClearAllPoints()
	Perl_Raid_Mover:Hide()
	Perl_Raid_Mover:SetScale(Perl_Config.Scale_Raid)

	for i,cell in FrameArray do
		getglobal(cell:GetName().."_StatsFrame_CastClickOverlay"):UnlockHighlight()
	end

	for i = 1,NUM_RAID_GROUPS do
		if (i ~= MovingMember.lastPos.group) then
			local frame = GetRaidGroup(i, true)

			if (frame) then
				local left = frame:GetLeft()
				local right = frame:GetRight()
				local top = frame:GetBottom()
				if (left) then
					local bottom = top - (MEMBERS_PER_RAID_GROUP * Perl_Config.RaidVerticalSpacing);

					if (x >= left and x <= right and y >= bottom and y <= top) then
						Perl_Raid_Mover:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 0, 0);
						Perl_Raid_Mover:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 0, (bottom - top));
						Perl_Raid_Mover:Show();

						local size = (bottom - top) / 5;
						local sub = math.floor((y - top - size) / size) + 2;
						if (sub < 1) then
							sub = 1;
						elseif (sub > 5) then
							sub = 5;
						end
--ChatFrame7:AddMessage("Top offset: "..(top - y).."   Bottom offset: "..(y - bottom));
--ChatFrame7:AddMessage("Top "..floor(top).."  Bottom "..floor(bottom).." x "..floor(x).."y "..floor(y).." Group "..i.."  Row "..sub);

						for j,cell in FrameArray do
							if (cell ~= MovingMember and cell.lastPos.group == i and cell.lastPos.row == sub) then
								getglobal(cell:GetName().."_StatsFrame_CastClickOverlay"):LockHighlight()
								break;
							end
						end
						return i, sub;
					end
				end
			end
		end
	end
end
