XPerl_Old_CT_RA_UpdateMTs = nil;
XPerl_Old_CT_RAOptions_Update = nil;

function XPerl_Message(msg)
	DEFAULT_CHAT_FRAME:AddMessage(XPERL_MSG_PREFIX.."- "..msg);
end

function XPerl_SetupFrames()

	local function ValidAlpha(alpha)
		alpha = tonumber(alpha);
		if (alpha < 0 or alpha > 1) then
			alpha = 1;
		end
		return alpha;
	end

	local function ValidScale(scale)
		scale = tonumber(scale);
		if (scale < 0.5) then
			scale = 0.5;
		elseif (scale > (Perl_Config.MaximumScale or 1.5)) then
			scale = (Perl_Config.MaximumScale or 1.5);
		end
		return scale;
	end

	XPerl_Config.AssistsFrame_Transparency = ValidAlpha(XPerl_Config.AssistsFrame_Transparency);
        XPerl_Assists_Frame:SetAlpha(XPerl_Config.AssistsFrame_Transparency);

	XPerl_Config.Targets_Transparency = ValidAlpha(XPerl_Config.Targets_Transparency);
        XPerl_Frame:SetAlpha(XPerl_Config.Targets_Transparency);

	XPerl_Config.Scale_AssistsFrame = ValidScale(XPerl_Config.Scale_AssistsFrame);
	XPerl_Assists_Frame:SetScale(XPerl_Config.Scale_AssistsFrame);

	XPerl_Config.Targets_Scale = ValidScale(XPerl_Config.Targets_Scale);
	XPerl_Frame:SetScale(XPerl_Config.Targets_Scale);

	-- Assist Counters

	XPerl_SetupFrame(XPerl_Frame, XPerl_Config.Background_Transparency);
	XPerl_SetupFrame(XPerl_MTTargets);
	XPerl_SetupFrame(XPerl_OtherTargets);
	XPerl_SetupFrame(XPerl_Stats, XPerl_Config.Background_Transparency);
	XPerl_SetupFrame(XPerl_Stats_List, XPerl_Config.Background_Transparency);
	XPerl_SetupFrame(XPerl_Assists_Frame, XPerl_Config.Assists_BackTransparency);
	XPerlScrollSeperator:SetAlpha(XPerl_Config.Assists_BackTransparency);
	XPerl_SetupFrame(XPerl_Threat);

	XPerl_Frame_ToggleMTTargets:SetButtonTex();
	XPerl_Frame_ToggleTargets:SetButtonTex();
	XPerl_Frame_ExpandLock:SetButtonTex();
	XPerl_Frame_ToggleLabels:SetButtonTex();
	XPerl_Frame_ToggleShowMT:SetButtonTex();
	XPerl_Frame_Pin:SetButtonTex();

	if (Perl_Player_TargettingFrame) then	Perl_Player_TargettingFrame:Show();	end
	if (Perl_Target_AssistFrame) then	Perl_Target_AssistFrame:Show();		end

	if (Perl_GetBarTexture) then
		local tex = Perl_GetBarTexture();
		if (tex) then
			XPerl_MyThreatTex:SetTexture(tex);
			XPerl_ThreatBGTex:SetTexture(tex);
		end
	end
end


function XPerl_Slash(msg)

	local commands = {};
	for x in string.gfind(msg, "[^ ]+") do
		table.insert(commands, string.lower(x));
	end

	local function SubCommandMatch(cmd, match)
		return strsub(match, 1, strlen(cmd)) == cmd;
	end

	local function setAlpha()
		if (commands[2] and commands[3]) then
			if (SubCommandMatch(commands[2], "raid")) then
				XPerl_Config.Targets_Transparency = commands[3];
				return true;
			elseif (SubCommandMatch(commands[2], "assists")) then
				XPerl_Config.AssistsFrame_Transparency = commands[3];
				return true;
			end
		end
	end

	local function setScale()
		if (commands[2] and commands[3]) then
			if (SubCommandMatch(commands[2], "raid")) then
				XPerl_Config.Targets_Scale = commands[3];
				return true;
			elseif (SubCommandMatch(commands[2], "assists")) then
				XPerl_Config.Scale_AssistsFrame = commands[3];
				return true;
			elseif (SubCommandMatch(commands[2], "other")) then
				XPerl_Config.OtherTargets_Scale = commands[3];
				return true;
			end
		end
	end

	local options = {
		{"ma",		XPerl_SetMainAssist},
		{"assists",	XPerl_AssistsView_Open,		"Open Assists View"},
		{"raid",	XPerl_RaidHelp_Show,		"Open Raid Helper"},
		{"alpha",	setAlpha,			"Set Alpha Level"},
		{"scale",	setScale,			"Set Frame Scale"},
		{"labels",      XPerl_Toggle_ToggleLabels,	"Toggle Tank Labels"},
		{"ctra",	XPerl_Toggle_UseCTRATargets,	"Toggle Use of CTRA MT Targets"},
		{"othertargets",XPerl_Toggle_OtherTargets,	"Toggle display of other mob's targets"},
		{"find",	XPerl_SetFind},
	};

	local foundFunc;
	local foundDesc;
	if (commands[1]) then
		local smallest = 100;
		local len = strlen(commands[1]);
		if (len) then
			for i,entry in options do
				if (strsub(entry[1], 1, len) == commands[1]) then
					if (foundFunc) then
						XPerl_Message("Ambiguous command, failed.")
						foundFunc = nil;
						break;
					end
					foundFunc = entry[2];
					foundDesc = entry[3];
				end
			end
		end
	end

	if (foundFunc) then
		if (foundFunc(msg, commands[2], commands[3], commands[4])) then
			XPerl_SetupFrames();
			if (foundDesc) then
				XPerl_Message(foundDesc.." - |c0000C020done!|r")
			end
			return;
		end
	end

	XPerl_Message("Options: /xp [|c00FFFF00find|r] [|c00FFFF00assists|r] [|c00FFFF00raid|r] [|c00FFFF00labels|r] [|c00FFFF00alpha|r raid|assists] [|c00FFFF00scale|r raid|assists|other] [|c00FFFF00ctra|r] [|c00FFFF00othertargets|r]")
end

function XPerl_OnLoad()

	XPerl_Frame:RegisterEvent("VARIABLES_LOADED");
	XPerl_Frame:RegisterEvent("RAID_ROSTER_UPDATE");

        SlashCmdList["XPERL"] = XPerl_Slash;
        SLASH_XPERL1 = "/xperl";
        SLASH_XPERL2 = "/xp";
end

local function DefaultVar(name, value)
	if (XPerl_Config[name] == nil or (type(value) ~= type(XPerl_Config[name]))) then
		XPerl_Config[name] = value;
	end
end

local function XPerl_Defaults()
	DefaultVar("RaidHelper",		1);
	DefaultVar("UnitWidth",			100);
	DefaultVar("UnitHeight",		26);
	DefaultVar("UseCTRATargets",		1);
	DefaultVar("ExpandLock",		0);
	DefaultVar("ShowMT",			1);
	DefaultVar("MTLabels",			0);
	DefaultVar("MTTargetTargets",		1);
	DefaultVar("CollectOtherTargets",	1);
	DefaultVar("OtherTargetTargets",	1);
	DefaultVar("Targets_Scale",		1);
	DefaultVar("OtherTargets_Scale",	1);
	DefaultVar("Targets_Transparency",	0.8);
	DefaultVar("Background_Transparency",	1);
	DefaultVar("ThreatBar",			1);
	DefaultVar("Tooltips",			0);

	DefaultVar("TargetCounters",		1);
	DefaultVar("TargetCountersSelf",	1);
	DefaultVar("TargetCountersEnemy",	1);
	DefaultVar("AssistsFrame",		1);
	DefaultVar("TargettingFrame",		1);
	DefaultVar("Scale_AssistsFrame",	1);
	DefaultVar("AssistsFrame_Transparency",	0.8);
	DefaultVar("Assists_BackTransparency",	1);

	DefaultVar("BorderColour",		{r = 0.5, g = 0.5, b = 0.5, a = 1});
	DefaultVar("BackgroundColour",		{r = 0, g = 0, b = 0, a = 1});
end

local XPerl_Old_KLHTM_Redraw = nil;
function XPerl_KLHTM_Redraw(forceRedraw)
	XPerl_Old_KLHTM_Redraw(forceRedraw);
	XPerl_UpdateThreat();
end

-- XPerl_Startup
-- Called after VARIABLES_LOADED
function XPerl_Startup()

	if (not XPerl_Config) then
		XPerl_Config = {}
	end
	XPerl_Defaults();

	if (CT_RAMenu_Options and CT_RAOptions_Update and CT_RA_UpdateMTs) then
		if (CT_RAMenu_Options and XPerl_Config.RaidHelper == 1 and CT_RA_UpdateVisibility) then
			CT_RAMenu_Options["temp"]["HideMTs"] = true;
			CT_RA_UpdateVisibility(1);
		end

		if (CT_RAOptions_Update ~= XPerl_CT_RAOptions_Update) then
			XPerl_Old_CT_RAOptions_Update = CT_RAOptions_Update;
			CT_RAOptions_Update = XPerl_CT_RAOptions_Update;
		end

		if (CT_RA_UpdateMTs ~= XPerl_CT_RA_UpdateMTs) then
			XPerl_Old_CT_RA_UpdateMTs = CT_RA_UpdateMTs;
			CT_RA_UpdateMTs = XPerl_CT_RA_UpdateMTs;
		end
	else
		XPerl_CT_RAOptions_Update = nil;
		XPerl_CT_RA_UpdateMTs = nil;
	end

	if (KLHTM_Redraw and KLHTM_GetRaidData) then
		if (KLHTM_Redraw ~= XPerl_KLHTM_Redraw) then
			XPerl_Old_KLHTM_Redraw = KLHTM_Redraw;
			KLHTM_Redraw = XPerl_KLHTM_Redraw;
		end
	else
		XPerl_UpdateThreat = nil;
	end

	if (oRA_Core and oRA_CoreModuleClass and AceDbClass) then
		oRA_XPerlMainTankHook = oRA_CoreModuleClass:new({
			type		= "xperlmthook",
			db		= AceDbClass:new("oRAXPerlMainTankHook"),

			Enable = function(self)
				self:RegisterEvent("CTRASPY_MAINTANKS_UPDATED")
			end,

			CTRASPY_MAINTANKS_UPDATED = function()
				XPerl_MakeTankList(true)
			end,
		});

		oRA_Core:Register(oRA_XPerlMainTankHook)
	end

	XPerl_SetupFrames();

        XPerlAssistPin:SetButtonTex()
        XPerl_Frame_Pin:SetButtonTex()
end

if (not Perl_SetSmoothBarColor) then
	Perl_SetSmoothBarColor = function(bar, backBar, alpha)
		if (not alpha) then
			alpha = 1;
		end
		if (bar) then
			local barmin, barmax = bar:GetMinMaxValues();
			if (barmin == barmax) then
				return false;
			end

			local percentage = bar:GetValue()/(barmax-barmin);

			local r, g, b;
			if (Perl_Config.ClassicHealthBar == 1) then
				if (percentage < 0.5) then
					r = 1;
					g = 2*percentage;
					b = 0;
				else
					g = 1;
					r = 2*(1 - percentage);
					b = 0;
				end
			else
				r = Perl_Config.ColourHealthEmpty.r + ((Perl_Config.ColourHealthFull.r - Perl_Config.ColourHealthEmpty.r) * percentage);
				g = Perl_Config.ColourHealthEmpty.g + ((Perl_Config.ColourHealthFull.g - Perl_Config.ColourHealthEmpty.g) * percentage);
				b = Perl_Config.ColourHealthEmpty.b + ((Perl_Config.ColourHealthFull.b - Perl_Config.ColourHealthEmpty.b) * percentage);
			end

			if ((r>=0) and (g>=0) and (b>=0) and (alpha>=0) and (r<=1) and (g<=1) and (b<=1) and (alpha<=1)) then
				bar:SetStatusBarColor(r, g, b);

				if (backBar) then
					backBar:SetVertexColor(r, g, b, alpha);
				end
			end
		else
			return false;
		end
	end
end

if (not Perl_SetUnitNameColor) then
	Perl_SetUnitNameColor = function(argUnit,argFrame)

		local r, g, b = 0.5, 0.5, 1;

	        if (UnitPlayerControlled(argUnit) or not UnitIsVisible(argUnit)) then
			local _, class = UnitClass(argUnit)
			r, g, b = Perl_GetClassColour(class);

	                if (UnitFactionGroup("player") == UnitFactionGroup(argUnit)) then
	                        if (UnitIsEnemy("player", argUnit)) then
					-- Dueling
	                                r, g, b = 1, 0, 0;

	                        elseif (UnitIsPVP(argUnit)) then
	                                r, g, b = 0, 1, 0;
				end
			else
	                        if (UnitIsPVP(argUnit)) then
					if (UnitIsPVP("player")) then
						r, g, b = 1, 0, 0;
					else
						r, g, b = 1, 1, 0;
					end
				end
			end
		else
	                if (UnitIsTapped(argUnit) and not UnitIsTappedByPlayer(argUnit)) then
	                        r, g, b = 0.5, 0.5, 0.5;
	                else
				local reaction = UnitReaction(argUnit, "player");

				if (reaction) then
					if (reaction >= 5) then
						r, g, b = 0, 1, 0;
					elseif (reaction <= 2) then
						r, g, b = 1, 0, 0;
					elseif (reaction == 3) then
						r, g, b = 1, 0.5, 0;
					else
						r, g, b = 1, 1, 0;
					end
				else
	                        	if (UnitFactionGroup("player") == UnitFactionGroup(argUnit)) then
						r, g, b = 0, 1, 0;
	                        	elseif (UnitIsEnemy("player", argUnit)) then
						r, g, b = 1, 0, 0;
	                        	else
						r, g, b = 1, 1, 0;
	                        	end
				end
			end
		end

		argFrame:SetTextColor(r, g, b);
	end
end

-- Perl UnitFrame function copies:
if (not Perl_ColourFriendlyUnit) then
	Perl_ColourFriendlyUnit = function(frame, partyid)
		if (UnitCanAttack("player", partyid) and UnitIsEnemy("player", partyid)) then	-- For dueling
	                frame:SetTextColor(1, 0, 0);
		else
			if (not Perl_Config or Perl_Config.ClassColouredNames == 1) then
				local _, engClass = UnitClass(partyid);
				local r, g, b = Perl_GetClassColour(engClass);
				frame:SetTextColor(r, g, b);
			else
				if (UnitIsPVP(partyid)) then
	                	        frame:SetTextColor(0, 1, 0);
				else
	                	        frame:SetTextColor(0.5, 0.5, 1);
				end
			end
		end
	end
end

if (not Perl_GetClassColour) then
	Perl_GetClassColour = function(class)
		if (class) then
			local color = RAID_CLASS_COLORS[class];		-- Now using the WoW class color table
			if (color) then
				return color.r, color.g, color.b;
			end
		end
		return 0.5,0.5,1;
	end
end
